@@ -446,7 +446,7 @@ int CControlSocket::CheckOverwriteFile()
 	{
 		if (!pData->fileTime.IsValid())
 		{
-			if (entry.hasDate)
+			if (entry.hasTimestamp != CDirentry::timestamp_none)
 			{
 				pNotification->remoteTime = entry.time;
 				pData->fileTime = entry.time;
@@ -459,8 +459,9 @@ int CControlSocket::CheckOverwriteFile()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-CFileTransferOpData::CFileTransferOpData() :
+CFileTransferOpData::CFileTransferOpData(const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path) :
 	COpData(cmd_transfer),
+ 	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
 	localFileSize(-1), remoteFileSize(-1),
 	tryAbsolutePath(false), resume(false), transferInitiated(false)
 {
@@ -949,7 +950,7 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 		}
 
 		if (written)
-			m_pEngine->SetActive(false);
+			m_pEngine->SetActive(CFileZillaEngine::send);
 
 		if (written < len)
 		{
@@ -1049,7 +1050,7 @@ void CRealControlSocket::OnSend()
 		if (written)
 		{
 			SetAlive();
-			m_pEngine->SetActive(false);
+			m_pEngine->SetActive(CFileZillaEngine::send);
 		}
 
 		if (written == m_nSendBufferLen)
@@ -1303,10 +1304,17 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			{
 				wxLongLong size = entry.size;
 				pData->remoteFileSize = size.GetLo() + ((wxFileOffset)size.GetHi() << 32);
+				if (entry.hasTimestamp != CDirentry::timestamp_none)
+					pData->fileTime = entry.time;
 
 				if (CheckOverwriteFile() != FZ_REPLY_OK)
 					break;
 			}
+			else
+			{
+				pData->fileTime = wxDateTime();
+				pData->remoteFileSize = -1;
+			}
 
 			SendNextCommand();
 		}
