@@ -147,7 +147,8 @@ bool CFileZillaEngine::SetAsyncRequestReply(CAsyncRequestNotification *pNotifica
 		return false;
 
 	m_pControlSocket->SetAlive();
-	m_pControlSocket->SetAsyncRequestReply(pNotification);
+	if (!m_pControlSocket->SetAsyncRequestReply(pNotification))
+		return false;
 
 	return true;
 }
@@ -163,34 +164,15 @@ bool CFileZillaEngine::IsPendingAsyncRequestReply(const CAsyncRequestNotificatio
 	return pNotification->requestNumber == m_asyncRequestCounter;
 }
 
-bool CFileZillaEngine::IsActive(bool recv)
+bool CFileZillaEngine::IsActive(enum CFileZillaEngine::_direction direction)
 {
-	if (recv)
+	if (m_activeStatus[direction] == 2)
 	{
-		if (m_activeStatusRecv == 2)
-		{
-			m_activeStatusRecv = 1;
-			return true;
-		}
-		else
-		{
-			m_activeStatusRecv = 0;
-			return false;
-		}
-	}
-	else
-	{
-		if (m_activeStatusSend == 2)
-		{
-			m_activeStatusSend = 1;
-			return true;
-		}
-		else
-		{
-			m_activeStatusSend = 0;
-			return false;
-		}
+		m_activeStatus[direction] = 1;
+		return true;
 	}
+
+	m_activeStatus[direction] = 0;
 	return false;
 }
 
