@@ -11,8 +11,7 @@ std::map<wxString, int> CDirectoryListingParser::m_MonthNamesMap;
 //#define LISTDEBUG_MVS
 //#define LISTDEBUG
 #ifdef LISTDEBUG
-static char data[][110]={
-	
+static char data[][150]={
 	"" // Has to be terminated with empty string
 };
 
@@ -308,7 +307,7 @@ protected:
 class CLine
 {
 public:
-	CLine(wxChar* p, int len = -1)
+	CLine(wxChar* p, int len = -1, int trailing_whitespace = 0)
 	{
 		m_pLine = p;
 		if (len != -1)
@@ -320,6 +319,7 @@ public:
 
 		m_Tokens.reserve(10);
 		m_LineEndTokens.reserve(10);
+		m_trailing_whitespace = trailing_whitespace;
 	}
 
 	~CLine()
@@ -333,7 +333,7 @@ public:
 			delete *iter;
 	}
 
-	bool GetToken(unsigned int n, CToken &token, bool toEnd = false)
+	bool GetToken(unsigned int n, CToken &token, bool toEnd = false, bool include_whitespace = false)
 	{
 		if (!toEnd)
 		{
@@ -380,6 +380,28 @@ public:
 		}
 		else
 		{
+			if (include_whitespace)
+			{
+				const wxChar* p;
+				if (!n)
+				{
+					CToken ref;
+					if (!GetToken(n, ref))
+						return false;
+					p = ref.GetToken() + ref.GetLength() + 1;
+				}
+				else
+				{
+					CToken ref;
+					if (!GetToken(n - 1, ref))
+						return false;
+					p = ref.GetToken() + ref.GetLength() + 1;
+				}
+
+				token = CToken(p, m_len - (p - m_pLine));
+				return true;
+			}
+
 			if (m_LineEndTokens.size() > n)
 			{
 				token = *(m_LineEndTokens[n]);
@@ -394,7 +416,7 @@ public:
 			{
 				const CToken *refToken = m_Tokens[i];
 				const wxChar* p = refToken->GetToken();
-				CToken *pToken = new CToken(p, m_len - (p - m_pLine));
+				CToken *pToken = new CToken(p, m_len - (p - m_pLine) - m_trailing_whitespace);
 				m_LineEndTokens.push_back(pToken);
 			}
 			token = *(m_LineEndTokens[n]);
@@ -410,7 +432,7 @@ public:
 		p[m_len] = ' ';
 		memcpy(p + m_len + 1, pLine->m_pLine, pLine->m_len * sizeof(wxChar));
 
-		return new CLine(p, m_len + pLine->m_len + 1);
+		return new CLine(p, m_len + pLine->m_len + 1, pLine->m_trailing_whitespace);
 	}
 
 protected:
@@ -418,6 +440,7 @@ protected:
 	std::vector<CToken *> m_LineEndTokens;
 	int m_parsePos;
 	int m_len;
+	int m_trailing_whitespace;
 	wxChar* m_pLine;
 };
 
@@ -701,8 +724,7 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 			CDirentry entry;
 			entry.name = *iter;
 			entry.dir = false;
-			entry.hasDate = false;
-			entry.hasTime = false;
+			entry.hasTimestamp = CDirentry::timestamp_none;
 			entry.link = false;
 			entry.size = -1;
 			entry.unsure = false;
@@ -721,6 +743,7 @@ bool CDirectoryListingParser::ParseLine(CLine *pLine, const enum ServerType serv
 {
 	CDirentry entry;
 	bool res;
+	int ires;
 
 	if (serverType == ZVM)
 	{
@@ -735,13 +758,15 @@ bool CDirectoryListingParser::ParseLine(CLine *pLine, const enum ServerType serv
 			goto done;
 	}
 
+	ires = ParseAsMlsd(pLine, entry);
+	if (ires == 1)
+		goto done;
+	else if (ires == 2)
+		goto skip;
 	res = ParseAsUnix(pLine, entry, true); // Common 'ls -l'
 	if (res)
 		goto done;
 	res = ParseAsDos(pLine, entry);
-	if (res)
-		goto done;
-	res = ParseAsMlsd(pLine, entry);
 	if (res)
 		goto done;
 	res = ParseAsEplf(pLine, entry);
@@ -830,17 +855,24 @@ done:
 			entry.name = entry.name.Left(pos);
 	}
 
-	int offset = m_server.GetTimezoneOffset();
-	if (offset && entry.hasTime)
 	{
-		// Apply timezone offset
-		wxTimeSpan span(0, offset, 0, 0);
-		entry.time.Add(span);
+		int offset = m_server.GetTimezoneOffset();
+		if (offset && entry.hasTimestamp >= CDirentry::timestamp_time)
+		{
+			// Apply timezone offset
+			wxTimeSpan span(0, offset, 0, 0);
+			entry.time.Add(span);
+		}
 	}
 
 	entry.unsure = false;
 	m_entryList.push_back(entry);
 
+skip:
+	m_maybeMultilineVms = false;
+	m_fileList.clear();
+	m_fileListOnly = false;
+
 	return true;
 }
 
@@ -936,10 +968,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 				continue;
 		}
 		else
-		{
-			entry.hasDate = false;
-			entry.hasTime = false;
-		}
+			entry.hasTimestamp = CDirentry::timestamp_none;
 
 		// Get the filename
 		if (!pLine->GetToken(++index, token, 1))
@@ -964,7 +993,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 			}
 		}
 
-		if (entry.hasTime)
+		if (entry.hasTimestamp >= CDirentry::timestamp_time)
 			entry.time.Add(m_timezoneOffset);
 
 		return true;
@@ -978,6 +1007,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 {
 	bool mayHaveTime = true;
 	bool bHasYearAndTime = false;
+	bool hasTime = false;
 
 	CToken token;
 
@@ -1020,10 +1050,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 				return false;
 
 			if (token[pos] == '.')
-			{
-				entry.hasTime = false;
 				return true;
-			}
 
 			year = entry.time.GetYear();
 			month = entry.time.GetMonth() - wxDateTime::Jan + 1;
@@ -1139,7 +1166,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 		if (minute < 0 || minute > 59)
 			return false;
 
-		entry.hasTime = true;
+		hasTime = true;
 
 		// Some servers use times only for files newer than 6 months
 		if (!year)
@@ -1195,27 +1222,19 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 				if (minute < 0 || minute > 59)
 					return false;
 
-				entry.hasTime = true;
+				hasTime = true;
 			}
 			else
-			{
-				entry.hasTime = false;
 				index--;
-			}
 		}
-		else
-			entry.hasTime = false;
 	}
 	else
-	{
-		entry.hasTime = false;
 		index--;
-	}
 
 	entry.time = wxDateTime();
 	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day, hour, minute))
 		return false;
-	entry.hasDate = true;
+	entry.hasTimestamp = hasTime ? CDirentry::timestamp_time : CDirentry::timestamp_date;
 
 	return true;
 }
@@ -1369,15 +1388,13 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		gotYear = true;
 	}
 
-	entry.hasDate = true;
-
 	if (!gotMonth || !gotDay || !gotYear)
 		return false;
 
 	entry.time = wxDateTime();
 	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day))
 		return false;
-	entry.hasDate = true;
+	entry.hasTimestamp = CDirentry::timestamp_date;
 
 	return true;
 }
@@ -1443,7 +1460,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1451,7 +1468,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 
 bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 {
-	if (!entry.hasDate)
+	if (entry.hasTimestamp == CDirentry::timestamp_none)
 		return false;
 
 	int pos = token.Find(':');
@@ -1478,6 +1495,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 		return false;
 
 	wxLongLong seconds;
+	bool hasSeconds = false;
 	if (pos2 == -1)
 		seconds = 0;
 	else
@@ -1486,6 +1504,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 		seconds = token.GetNumber(pos2 + 1, -1);
 		if (seconds < 0 || seconds > 59)
 			return false;
+		hasSeconds = true;
 	}
 
 	// Convert to 24h format
@@ -1503,7 +1522,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 
 	wxTimeSpan span(hour.GetLo(), minute.GetLo(), seconds.GetLo());
 	entry.time.Add(span);
-	entry.hasTime = true;
+	entry.hasTimestamp = hasSeconds ? CDirentry::timestamp_seconds : CDirentry::timestamp_time;
 
 	return true;
 }
@@ -1527,8 +1546,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 	entry.permissions = _T("");
 	entry.dir = false;
 	entry.link = false;
-	entry.hasDate = false;
-	entry.hasTime = false;
+	entry.hasTimestamp = CDirentry::timestamp_none;
 	entry.size = -1;
 
 	int fact = 1;
@@ -1560,8 +1578,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 				return false;
 			entry.time = wxDateTime((time_t)number.GetValue());
 
-			entry.hasTime = true;
-			entry.hasDate = true;
+			entry.hasTimestamp = CDirentry::timestamp_seconds;
 		}
 		else if (type == 'u' && len > 2 && token[fact + 1] == 'p')
 			entry.permissions = token.GetString().Mid(fact + 2, len - 2);
@@ -1699,7 +1716,6 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 			return false;
 		if (token[0] != '(' && token[len - 1] == ')')
 			return false;
-		entry.hasTime = false;
 		index--;
 	}
 
@@ -1750,7 +1766,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		}
 	}
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1803,7 +1819,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 	else
 		entry.dir = false;
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1861,8 +1877,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			return false;
 		entry.time = wxDateTime((time_t)number.GetValue());
 
-		entry.hasTime = true;
-		entry.hasDate = true;
+		entry.hasTimestamp = CDirentry::timestamp_seconds;
 
 		// Get filename
 		if (!pLine->GetToken(++index, token, true))
@@ -1958,7 +1973,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			if (!VerifySetDate(entry.time, year.GetLo(), (wxDateTime::Month)(month - 1), day.GetLo()))
 				return false;
 
-			entry.hasDate = true;
+			entry.hasTimestamp = CDirentry::timestamp_date;
 
 			// Get time
 			if (!pLine->GetToken(++index, token))
@@ -1986,7 +2001,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 		entry.ownerGroup = _T("");
 		entry.permissions = _T("");
 
-		if (entry.hasTime)
+		if (entry.hasTimestamp >= CDirentry::timestamp_time)
 			entry.time.Add(m_timezoneOffset);
 	}
 
@@ -2061,13 +2076,11 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 		int currentOffset = m_currentOffset;
 		while ((iter->p[currentOffset] != '\n') && (iter->p[currentOffset] != '\r'))
 		{
-			if (iter->p[currentOffset] != ' ' && iter->p[currentOffset] != '\t')
-			{
-				reslen += emptylen + 1;
-				emptylen = 0;
-			}
-			else
+			if (iter->p[currentOffset] == ' ' || iter->p[currentOffset] == '\t')
 				emptylen++;
+			else
+				emptylen = 0;
+			reslen++;
 
 			currentOffset++;
 			if (currentOffset >= len)
@@ -2085,7 +2098,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 		}
 		m_currentOffset = currentOffset;
 
-		// Reslen is now the length of the line, excluding any terminating whitespace
+		// Reslen is now the length of the line, including any terminating whitespace
 		char *res = new char[reslen + 1];
 		res[reslen] = 0;
 
@@ -2106,19 +2119,13 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 			delete [] i->p;
 			i++;
 		}
-		// Delete all extra whitespace
-		while (i != iter)
-		{
-			delete [] i->p;
-			i++;
-		}
 
 		// Copy last chunk
 		if (iter != m_DataList.end() && reslen)
 		{
 			int copylen = m_currentOffset-startpos;
-			if (copylen>reslen)
-				copylen=reslen;
+			if (copylen > reslen)
+				copylen = reslen;
 			memcpy(&res[respos], &iter->p[startpos], copylen);
 			if (reslen >= iter->len)
 			{
@@ -2157,7 +2164,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 			continue;
 		}
 
-		return new CLine(buffer);
+		return new CLine(buffer, -1, emptylen);
 	}
 
 	return 0;
@@ -2209,7 +2216,7 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine *pLine, CDirentry &entry)
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -2233,7 +2240,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 		return false;
 
 	if (token.GetString() == _T("**NONE**"))
-		entry.hasDate = false;
+		entry.hasTimestamp = CDirentry::timestamp_none;
 	else if (!ParseShortDate(token, entry))
 	{
 		// Perhaps of the following type:
@@ -2252,8 +2259,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 		entry.dir = false;
 		entry.ownerGroup = _T("");
 		entry.permissions = _T("");
-		entry.hasDate = false;
-		entry.hasTime = false;
+		entry.hasTimestamp = CDirentry::timestamp_none;
 		entry.link = false;
 
 		return true;
@@ -2319,7 +2325,6 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
-	entry.hasTime = false;
 	entry.link = false;
 
 	return true;
@@ -2385,7 +2390,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry)
 	entry.permissions = _T("");
 	entry.link = false;
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -2414,8 +2419,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine *pLine, CDirentry &e
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasDate = false;
-	entry.hasTime = false;
+	entry.hasTimestamp = CDirentry::timestamp_none;
 
 	if (pLine->GetToken(++index, token))
 		return false;
@@ -2437,8 +2441,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasDate = false;
-	entry.hasTime = false;
+	entry.hasTimestamp = CDirentry::timestamp_none;
 
 	if (!pLine->GetToken(++index, token))
 		return true;
@@ -2514,8 +2517,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine *pLine, CDirentry &entry
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasDate = false;
-	entry.hasTime = false;
+	entry.hasTimestamp = CDirentry::timestamp_none;
 
 	if (pLine->GetToken(index++, token))
 		return false;
@@ -2614,7 +2616,7 @@ bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, wxLongLong& si
 	return true;
 }
 
-bool CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
+int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 {
 	// MLSD format as described here: http://www.ietf.org/internet-drafts/draft-ietf-ftpext-mlst-16.txt
 
@@ -2623,39 +2625,53 @@ bool CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 	CToken token;
 
 	if (!pLine->GetToken(0, token))
-		return false;
+		return 0;
 
 	wxString facts = token.GetString();
 	if (facts.IsEmpty())
-		return false;
+		return 0;
 
 	entry.dir = false;
 	entry.link = false;
 	entry.size = -1;
-	entry.hasDate = false;
-	entry.hasTime = false;
+	entry.hasTimestamp = CDirentry::timestamp_none;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
+	wxString owner, group, uid, gid;
+
 	while (!facts.IsEmpty())
 	{
 		int delim = facts.Find(';');
 		if (delim < 3)
-			return false;
+		{
+			if (delim != -1)
+				return 0;
+			else
+				delim = facts.Len();
+		}
 
 		int pos = facts.Find('=');
-		if (pos < 1 || (pos + 2) >= delim)
-			return false;
+		if (pos < 1 || pos > delim)
+			return 0;
 
 		wxString factname = facts.Left(pos);
 		MakeLowerAscii(factname);
 		wxString value = facts.Mid(pos + 1, delim - pos - 1);
 		if (factname == _T("type"))
 		{
-			if (!value.CmpNoCase(_T("dir")) ||
-				!value.CmpNoCase(_T("cdir")) ||
-				!value.CmpNoCase(_T("pdir")))
+			if (!value.CmpNoCase(_T("dir")))
 				entry.dir = true;
+			else if (!value.Left(13).CmpNoCase(_T("OS.unix=slink")))
+			{
+				entry.dir = true;
+				entry.link = true;
+				if (value[13] == ':' && value[14] != 0)
+					entry.target = value.Mid(14);
+			}
+			else if (!value.CmpNoCase(_T("cdir")) ||
+					 !value.CmpNoCase(_T("pdir")))
+				return 2;
 		}
 		else if (factname == _T("size"))
 		{
@@ -2664,45 +2680,77 @@ bool CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 			for (unsigned int i = 0; i < value.Len(); i++)
 			{
 				if (value[i] < '0' || value[i] > '9')
-					return false;
+					return 0;
 				entry.size *= 10;
 				entry.size += value[i] - '0';
 			}
 		}
 		else if (factname == _T("modify") ||
-			(!entry.hasDate && factname == _T("create")))
+			(entry.hasTimestamp == CDirentry::timestamp_none && factname == _T("create")))
 		{
 			wxDateTime dateTime;
 			const wxChar* time = dateTime.ParseFormat(value, _T("%Y%m%d"));
 
 			if (!time)
-				return false;
+				return 0;
 
 			if (*time)
 			{
 				if (!dateTime.ParseFormat(time, _T("%H%M%S"), dateTime))
-					return false;
-				entry.hasTime = true;
+					return 0;
+				entry.hasTimestamp = CDirentry::timestamp_seconds;
 			}
 			else
-				entry.hasTime = false;
+				entry.hasTimestamp = CDirentry::timestamp_date;
 
 			entry.time = dateTime.FromTimezone(wxDateTime::GMT0);
-
-			entry.hasDate = true;
 		}
 		else if (factname == _T("perm"))
-			entry.permissions = value;
+		{
+			if (!value.empty())
+			{
+				if (!entry.permissions.empty())
+					entry.permissions = value + _T(" (") + entry.permissions + _T(")");
+				else
+					entry.permissions = value;
+			}
+		}
+		else if (factname == _T("unix.mode"))
+		{
+			if (!entry.permissions.empty())
+				entry.permissions = entry.permissions + _T(" (") + value + _T(")");
+			else
+				entry.permissions = value;
+		}
+		else if (factname == _T("unix.owner") || factname == _T("unix.user"))
+			owner = value;
+		else if (factname == _T("unix.group"))
+			group = value;
+		else if (factname == _T("unix.uid"))
+			uid = value;
+		else if (factname == _T("unix.gid"))
+			gid = value;
 
 		facts = facts.Mid(delim + 1);
 	}
 
-	if (!pLine->GetToken(1, token, true))
-		return false;
+	// The order of the facts is undefined, so assemble ownerGroup in correct
+	// order
+	if (!owner.empty())
+		entry.ownerGroup += owner;
+	else if (!uid.empty())
+		entry.ownerGroup += uid;
+	if (!group.empty())
+		entry.ownerGroup += _T(" ") + group;
+	else if (!gid.empty())
+		entry.ownerGroup += _T(" ") + gid;
+
+	if (!pLine->GetToken(1, token, true, true))
+		return 0;
 
 	entry.name = token.GetString();
 
-	return true;
+	return 1;
 }
 
 bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
@@ -2765,7 +2813,6 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.hasTime = false;
 	entry.link = false;
 
 	return true;
@@ -2864,7 +2911,7 @@ bool CDirectoryListingParser::ParseAsZVM(CLine* pLine, CDirentry &entry)
 	entry.permissions = _T("");
 	entry.target = _T("");
 
-	if (entry.hasTime)
+	if (entry.hasTimestamp >= CDirentry::timestamp_time)
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
