@@ -13,7 +13,8 @@
 class CSftpFileTransferOpData : public CFileTransferOpData
 {
 public:
-	CSftpFileTransferOpData()
+	CSftpFileTransferOpData(const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path)
+		: CFileTransferOpData(local_file, remote_file, remote_path)
 	{
 	}
 };
@@ -32,7 +33,11 @@ struct sftp_message
 {
 	sftpEventTypes type;
 	wxString text;
-	sftpRequestTypes reqType;
+	union
+	{
+		sftpRequestTypes reqType;
+		int value;
+	};
 };
 
 DECLARE_EVENT_TYPE(fzEVT_SFTP, -1)
@@ -93,6 +98,42 @@ protected:
 			wxPostEvent(m_pOwner, evt);
 	}
 
+	int ReadNumber(wxInputStream* pInputStream, bool &error)
+	{
+		int number = 0;
+
+		while(!pInputStream->Eof())
+		{
+			char c;
+			pInputStream->Read(&c, 1);
+			if (pInputStream->LastRead() != 1)
+			{
+				if (pInputStream->Eof())
+					m_pOwner->LogMessage(Debug_Warning, _T("Unexpected EOF."));
+				else
+					m_pOwner->LogMessage(Debug_Warning, _T("Uknown input stream error"));
+				error = true;
+				return 0;
+			}
+
+			if (c == '\n')
+				break;
+			else if (c >= '0' && c <= '9')
+			{
+				number *= 10;
+				number += c - '0';
+			}
+		}
+		if (pInputStream->Eof())
+		{
+			m_pOwner->LogMessage(Debug_Warning, _T("Unexpected EOF."));
+			error = true;
+			return 0;
+		}
+
+		return number;
+	}
+
 	wxString ReadLine(wxInputStream* pInputStream, bool &error)
 	{
 		int read = 0;
@@ -234,13 +275,16 @@ protected:
 				{
 					sftp_message* message = new sftp_message;
 					message->type = (sftpEventTypes)eventType;
-					message->text = ReadLine(pInputStream, error);
-					if (pInputStream->LastRead() <= 0 || message->text == _T(""))
+					message->value = ReadNumber(pInputStream, error);
+					if (error)
 					{
 						delete message;
 						goto loopexit;
 					}
-					SendMessage(message);
+					if (!message->value)
+						delete message;
+					else
+						SendMessage(message);
 				}
 				break;
 			default:
@@ -653,9 +697,6 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 		case sftpRead:
 		case sftpWrite:
 			{
-				long number = 0;
-				message->text.ToLong(&number);
-				
 				if (m_pTransferStatus && !m_pTransferStatus->madeProgress)
 				{
 					if (m_pCurOpData && m_pCurOpData->opId == cmd_transfer)
@@ -663,7 +704,7 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 						CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 						if (pData->download)
 						{
-							if (number > 0)
+							if (message->value > 0)
 								SetTransferStatusMadeProgress();
 						}
 						else
@@ -674,14 +715,14 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 					}
 				}
 
-				UpdateTransferStatus(number);
+				UpdateTransferStatus(message->value);
 			}
 			break;
 		case sftpRecv:
-			SetActive(true);
+			SetActive(CFileZillaEngine::recv);
 			break;
 		case sftpSend:
-			SetActive(false);
+			SetActive(CFileZillaEngine::send);
 			break;
 		case sftpUsedQuotaRecv:
 			OnQuotaRequest(CRateLimiter::inbound);
@@ -932,7 +973,7 @@ int CSftpControlSocket::List(CServerPath path /*=CServerPath()*/, wxString subDi
 	pData->path = path;
 	pData->subDir = subDir;
 	pData->refresh = (flags & LIST_FLAG_REFRESH) != 0;
-	pData->fallback_to_current = (flags & LIST_FLAG_FALLBACK_CURRENT) != 0;
+	pData->fallback_to_current = !path.IsEmpty() && (flags & LIST_FLAG_FALLBACK_CURRENT) != 0;
 
 	int res = ChangeDir(path, subDir, (flags & LIST_FLAG_LINK) != 0);
 	if (res != FZ_REPLY_OK)
@@ -1012,11 +1053,18 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 				if (date.IsValid())
 				{
 					date.MakeTimezone(wxDateTime::GMT0);
-					wxASSERT(pData->directoryListing[pData->mtime_index].hasTime);
+					wxASSERT(pData->directoryListing[pData->mtime_index].hasTimestamp != CDirentry::timestamp_none);
 					wxDateTime listTime = pData->directoryListing[pData->mtime_index].time;
 					listTime -= wxTimeSpan(0, m_pCurrentServer->GetTimezoneOffset(), 0);
 
 					int serveroffset = (date - listTime).GetSeconds().GetLo();
+					if (pData->directoryListing[pData->mtime_index].hasTimestamp != CDirentry::timestamp_seconds)
+					{
+						// Round offset to full minutes
+						if (serveroffset < 0)
+							serveroffset -= 59;
+						serveroffset -= serveroffset % 60;
+					}
 
 					wxDateTime now = wxDateTime::Now();
 					wxDateTime now_utc = now.ToTimezone(wxDateTime::GMT0);
@@ -1031,7 +1079,7 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 					for (int i = 0; i < count; i++)
 					{
 						CDirentry& entry = pData->directoryListing[i];
-						if (!entry.hasTime)
+						if (entry.hasTimestamp < CDirentry::timestamp_time)
 							continue;
 
 						entry.time += span;
@@ -1590,12 +1638,9 @@ int CSftpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 		delete m_pCurOpData;
 	}
 
-	CSftpFileTransferOpData *pData = new CSftpFileTransferOpData;
+	CSftpFileTransferOpData *pData = new CSftpFileTransferOpData(localFile, remoteFile, remotePath);
 	m_pCurOpData = pData;
 
-	pData->localFile = localFile;
-	pData->remotePath = remotePath;
-	pData->remoteFile = remoteFile;
 	pData->download = download;
 	pData->transferSettings = transferSettings;
 
@@ -1659,11 +1704,11 @@ int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 					if (matchedCase)
 					{
 						pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
-						if (entry.hasDate)
+						if (entry.hasTimestamp != CDirentry::timestamp_none)
 							pData->fileTime = entry.time;
 
 						if (pData->download &&
-							(!entry.hasDate || !entry.hasTime) &&
+							entry.hasTimestamp < CDirentry::timestamp_time &&
 							m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
 						{
 							pData->opState = filetransfer_mtime;
@@ -1722,11 +1767,11 @@ int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 				if (matchedCase && !entry.unsure)
 				{
 					pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
-					if (entry.hasDate)
+					if (entry.hasTimestamp != CDirentry::timestamp_none)
 						pData->fileTime = entry.time;
 
 					if (pData->download &&
-						(!entry.hasDate || !entry.hasTime) &&
+						entry.hasTimestamp < CDirentry::timestamp_time &&
 						m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
 					{
 						pData->opState = filetransfer_mtime;
@@ -2721,7 +2766,7 @@ int CSftpControlSocket::ListCheckTimezoneDetection()
 		const int count = pData->directoryListing.GetCount();
 		for (int i = 0; i < count; i++)
 		{
-			if (!pData->directoryListing[i].hasTime)
+			if (pData->directoryListing[i].hasTimestamp < CDirentry::timestamp_time)
 				continue;
 
 			if (pData->directoryListing[i].link)
