@@ -40,6 +40,8 @@ CTlsSocket::CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CCont
 	m_shutdown_requested = false;
 
 	m_socket_eof = false;
+
+	m_require_root_trust = false;
 }
 
 CTlsSocket::~CTlsSocket()
@@ -139,6 +141,8 @@ void CTlsSocket::Uninit()
 
 	delete [] m_implicitTrustedCert.data;
 	m_implicitTrustedCert.data = 0;
+
+	m_require_root_trust = false;
 }
 
 void CTlsSocket::LogError(int code)
@@ -814,7 +818,7 @@ int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != handshake)
 	{
-		m_pOwner->LogMessage(Debug_Warning, _T("VerifyCertificate called at wrong time"));
+		m_pOwner->LogMessage(::Debug_Warning, _T("VerifyCertificate called at wrong time"));
 		return FZ_REPLY_ERROR;
 	}
 
@@ -822,16 +826,45 @@ int CTlsSocket::VerifyCertificate()
 
 	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("Unsupported certificate type"));
+		m_pOwner->LogMessage(::Error, _("Unsupported certificate type"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	unsigned int status = 0;
+	if (gnutls_certificate_verify_peers2(m_session, &status) < 0)
+	{
+		m_pOwner->LogMessage(::Error, _("Failed to verify peer certificate"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (status & GNUTLS_CERT_REVOKED)
+	{
+		m_pOwner->LogMessage(::Error, _("Beware! Certificate has been revoked"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (status & GNUTLS_CERT_SIGNER_NOT_CA)
+	{
+		m_pOwner->LogMessage(::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (m_require_root_trust && status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+	{
+		m_pOwner->LogMessage(::Error, _("Root certificate is not trusted"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
 	unsigned int cert_list_size;
-	const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
+	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
 	if (!cert_list || !cert_list_size)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_certificate_get_peers returned no certificates"));
+		m_pOwner->LogMessage(::Error, _("gnutls_certificate_get_peers returned no certificates"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -853,124 +886,133 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_OK;
 	}
 
-	gnutls_x509_crt_t cert;
-	if (gnutls_x509_crt_init(&cert))
+	std::vector<CCertificate> certificates;
+	for (unsigned int i = 0; i < cert_list_size; i++)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_init failed"));
-		Failure(0, ECONNABORTED);
-		return FZ_REPLY_ERROR;
-	}
-
-	if (gnutls_x509_crt_import(cert, cert_list, GNUTLS_X509_FMT_DER))
-	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_import failed"));
-		Failure(0, ECONNABORTED);
-		gnutls_x509_crt_deinit(cert);
-		return FZ_REPLY_ERROR;
-	}
-
-	wxDateTime expirationTime = gnutls_x509_crt_get_expiration_time(cert);
-	wxDateTime activationTime = gnutls_x509_crt_get_activation_time(cert);
-
-	// Get the serial number of the certificate
-	unsigned char buffer[40];
-	size_t size = sizeof(buffer);
-	int res = gnutls_x509_crt_get_serial(cert, buffer, &size);
-
-	wxString serial = bin2hex(buffer, size);
-
-	unsigned int bits;
-	int algo = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
-
-	wxString algoName;
-	const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)algo);
-	if (pAlgo)
-		algoName = wxString(pAlgo, wxConvUTF8);
-
-	//int version = gnutls_x509_crt_get_version(cert);
-
-	wxString subject, issuer;
-
-	size = 0;
-	res = gnutls_x509_crt_get_dn(cert, 0, &size);
-	if (size)
-	{
-		char* dn = new char[size + 1];
-		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size)))
+		gnutls_x509_crt_t cert;
+		if (gnutls_x509_crt_init(&cert))
 		{
+			m_pOwner->LogMessage(::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
+			Failure(0, ECONNABORTED);
+			return FZ_REPLY_ERROR;
+		}
+
+		if (gnutls_x509_crt_import(cert, cert_list, GNUTLS_X509_FMT_DER))
+		{
+			m_pOwner->LogMessage(::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
+			Failure(0, ECONNABORTED);
+			gnutls_x509_crt_deinit(cert);
+			return FZ_REPLY_ERROR;
+		}
+
+		wxDateTime expirationTime = gnutls_x509_crt_get_expiration_time(cert);
+		wxDateTime activationTime = gnutls_x509_crt_get_activation_time(cert);
+
+		// Get the serial number of the certificate
+		unsigned char buffer[40];
+		size_t size = sizeof(buffer);
+		int res = gnutls_x509_crt_get_serial(cert, buffer, &size);
+
+		wxString serial = bin2hex(buffer, size);
+
+		unsigned int bits;
+		int algo = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
+
+		wxString algoName;
+		const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)algo);
+		if (pAlgo)
+			algoName = wxString(pAlgo, wxConvUTF8);
+
+		//int version = gnutls_x509_crt_get_version(cert);
+
+		wxString subject, issuer;
+
+		size = 0;
+		res = gnutls_x509_crt_get_dn(cert, 0, &size);
+		if (size)
+		{
+			char* dn = new char[size + 1];
 			dn[size] = 0;
-			subject = wxString(dn, wxConvUTF8);
+			if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size)))
+			{
+				dn[size] = 0;
+				subject = wxString(dn, wxConvUTF8);
+			}
+			else
+				LogError(res);
+			delete [] dn;
 		}
 		else
 			LogError(res);
-		delete [] dn;
-	}
-	else
-		LogError(res);
-	if (subject == _T(""))
-	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_dn failed"));
-		Failure(0, ECONNABORTED);
-		gnutls_x509_crt_deinit(cert);
-		return FZ_REPLY_ERROR;
-	}
-
-	size = 0;
-	res = gnutls_x509_crt_get_issuer_dn(cert, 0, &size);
-	if (size)
-	{
-		char* dn = new char[size + 1];
-		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size)))
+		if (subject == _T(""))
 		{
+			m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
+			Failure(0, ECONNABORTED);
+			gnutls_x509_crt_deinit(cert);
+			return FZ_REPLY_ERROR;
+		}
+
+		size = 0;
+		res = gnutls_x509_crt_get_issuer_dn(cert, 0, &size);
+		if (size)
+		{
+			char* dn = new char[++size + 1];
 			dn[size] = 0;
-			issuer = wxString(dn, wxConvUTF8);
+			if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size)))
+			{
+				dn[size] = 0;
+				issuer = wxString(dn, wxConvUTF8);
+			}
+			else
+				LogError(res);
+			delete [] dn;
 		}
 		else
 			LogError(res);
-		delete [] dn;
-	}
-	else
-		LogError(res);
-	if (issuer == _T(""))
-	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_issuer_dn failed"));
-		Failure(0, ECONNABORTED);
-		gnutls_x509_crt_deinit(cert);
-		return FZ_REPLY_ERROR;
-	}
+		if (issuer == _T(""))
+		{
+			m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
+			Failure(0, ECONNABORTED);
+			gnutls_x509_crt_deinit(cert);
+			return FZ_REPLY_ERROR;
+		}
 
-	wxString fingerprint_md5;
-	wxString fingerprint_sha1;
+		wxString fingerprint_md5;
+		wxString fingerprint_sha1;
 
-	unsigned char digest[100];
-	size = sizeof(digest) - 1;
-	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_MD5, digest, &size))
-	{
-		digest[size] = 0;
-		fingerprint_md5 = bin2hex(digest, size);
-	}
-	size = sizeof(digest) - 1;
-	if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA1, digest, &size))
-	{
-		digest[size] = 0;
-		fingerprint_sha1 = bin2hex(digest, size);
+		unsigned char digest[100];
+		size = sizeof(digest) - 1;
+		if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_MD5, digest, &size))
+		{
+			digest[size] = 0;
+			fingerprint_md5 = bin2hex(digest, size);
+		}
+		size = sizeof(digest) - 1;
+		if (!gnutls_x509_crt_get_fingerprint(cert, GNUTLS_DIG_SHA1, digest, &size))
+		{
+			digest[size] = 0;
+			fingerprint_sha1 = bin2hex(digest, size);
+		}
+
+		certificates.push_back(CCertificate(
+			cert_list->data, cert_list->size,
+			activationTime, expirationTime,
+			serial,
+			algoName, bits,
+			fingerprint_md5,
+			fingerprint_sha1,
+			subject,
+			issuer));
+
+		cert_list++;
 	}
 
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		m_pOwner->GetCurrentServer()->GetHost(),
 		m_pOwner->GetCurrentServer()->GetPort(),
-		cert_list->data, cert_list->size,
-		activationTime, expirationTime,
-		serial,
-		algoName, bits,
-		fingerprint_md5,
-		fingerprint_sha1,
-		subject,
-		issuer,
 		GetCipherName(),
-		GetMacName());
+		GetMacName(),
+		certificates);
 
 	m_pOwner->SendAsyncRequest(pNotification);
 
@@ -1000,3 +1042,22 @@ wxString CTlsSocket::GetMacName()
 	else
 		return _T("unknown");
 }
+
+bool CTlsSocket::AddTrustedRootCertificate(const wxString& cert)
+{
+	if (!m_initialized)
+		return false;
+
+	if (cert == _T(""))
+		return false;
+
+	m_require_root_trust = true;
+
+	const wxWX2MBbuf str = cert.mb_str();
+
+	gnutls_datum_t datum;
+	datum.data = (unsigned char*)(const char*)str;
+	datum.size = strlen(str);
+
+	return gnutls_certificate_set_x509_trust_mem(m_certCredentials, &datum, GNUTLS_X509_FMT_PEM) > 0;
+}
