@@ -6,9 +6,8 @@
 #include "backend.h"
 #include "socket.h"
 
-class CSocket;
 class CControlSocket;
-class CTlsSocket : protected wxEvtHandler, public CBackend
+class CTlsSocket : protected CSocketEventHandler, public CBackend, public CSocketEventSource
 {
 public:
 	enum TlsState
@@ -21,7 +20,7 @@ public:
 		closed
 	};
 
-	CTlsSocket(wxEvtHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner);
+	CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner);
 	virtual ~CTlsSocket();
 
 	bool Init();
@@ -29,14 +28,11 @@ public:
 
 	int Handshake(const CTlsSocket* pPrimarySocket = 0, bool try_resume = false);
 
-	virtual void Read(void *buffer, unsigned int len);
-	virtual void Write(const void *buffer, unsigned int len);
-	virtual bool Error() const { return !m_lastSuccessful; }
-	virtual unsigned int LastCount() const { return m_lastCount; }
-	virtual int LastError() const { return m_lastError; }
-	virtual void Peek(void *buffer, unsigned int len);
+	virtual int Read(void *buffer, unsigned int size, int& error);
+	virtual int Peek(void *buffer, unsigned int size, int& error);
+	virtual int Write(const void *buffer, unsigned int size, int& error);
 
-	void Shutdown();
+	int Shutdown();
 
 	void TrustCurrentCert(bool trusted);
 
@@ -47,6 +43,8 @@ public:
 
 	bool ResumedSession() const;
 	
+	// PEM formatted
+	bool AddTrustedRootCertificate(const wxString& cert);
 protected:
 
 	bool CopySessionData(const CTlsSocket* pPrimarySocket);
@@ -79,7 +77,6 @@ protected:
 
 	void TriggerEvents();
 
-	DECLARE_EVENT_TABLE();
 	void OnSocketEvent(CSocketEvent& event);
 	void OnRead();
 	void OnSend();
@@ -96,10 +93,6 @@ protected:
 	CSocketBackend* m_pSocketBackend;
 	CSocket* m_pSocket;
 
-	// Used by LastError() and LastCount()
-	int m_lastError;
-	unsigned int m_lastCount;
-	bool m_lastSuccessful;
 	bool m_shutdown_requested;
 
 	// Due to the strange gnutls_record_send semantics, call it again
@@ -120,6 +113,8 @@ protected:
 	gnutls_datum_t m_implicitTrustedCert;
 
 	bool m_socket_eof;
+
+	bool m_require_root_trust;
 };
 
 #endif //__TLSSOCKET_H__
