@@ -14,8 +14,14 @@ public:
 	bool link;
 	wxString target; // Set to linktarget it link is true
 
-	bool hasDate;
-	bool hasTime;
+	enum _timestamp
+	{
+		timestamp_none,
+		timestamp_date,
+		timestamp_time,
+		timestamp_seconds
+	} hasTimestamp;
+
 	wxDateTime time;
 
 	bool unsure; // May be true on cached items if any changes were made to the file
@@ -74,6 +80,9 @@ public:
 	bool m_failed;
 	bool m_hasDirs;
 
+	bool m_has_perms;
+	bool m_has_usergroup;
+
 	CTimeEx m_firstListTime;
 
 	void Assign(const std::list<CDirentry> &entries);
