@@ -1,10 +1,14 @@
 #ifndef __MISC_H__
 #define __MISC_H__
 
+#include "socket.h"
+
 bool VerifySetDate(wxDateTime& date, int year, wxDateTime::Month month, int day, int hour = 0, int minute = 0, int second = 0);
 
+// Also verifies that it is a correct IPv6 address
 wxString GetIPV6LongForm(wxString short_address);
-bool IsRoutableAddress(const wxString& address, int family);
+
+bool IsRoutableAddress(const wxString& address, enum CSocket::address_family family);
 
 bool IsIpAddress(const wxString& address);
 
@@ -19,4 +23,11 @@ int GetRandomNumber(int low, int high);
 // and locale-independently
 void MakeLowerAscii(wxString& str);
 
+enum _dependency
+{
+	dependency_gnutls
+};
+
+wxString GetDependencyVersion(enum _dependency dependency);
+
 #endif //__MISC_H__
