@@ -63,6 +63,11 @@ enum engineOptions
 	OPTION_PROXY_USER,
 	OPTION_PROXY_PASS,
 
+	OPTION_LOGGING_FILE,
+	OPTION_LOGGING_FILE_SIZELIMIT,
+
+	OPTION_INTERNAL_ROOTCERT,
+
 	OPTIONS_ENGINE_NUM
 };
 
