@@ -13,6 +13,7 @@
 #ifdef __WXMSW__
 #include "lm.h"
 #include <wx/msw/registry.h>
+#include "volume_enumerator.h"
 #else
 #include <langinfo.h>
 #endif
@@ -243,6 +244,10 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_OPEN"), CLocalListView::OnMenuOpen)
 	EVT_MENU(XRCID("ID_EDIT"), CLocalListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CLocalListView::OnMenuEnter)
+#ifdef __WXMSW__
+	EVT_COMMAND(-1, fzEVT_VOLUMESENUMERATED, CLocalListView::OnVolumesEnumerated)
+	EVT_COMMAND(-1, fzEVT_VOLUMEENUMERATED, CLocalListView::OnVolumesEnumerated)
+#endif
 END_EVENT_TABLE()
 
 CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQueue)
@@ -268,6 +273,8 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
 #ifdef __WXMSW__
+	m_pVolumeEnumeratorThread = 0;
+
 	InitHeaderImageList();
 #endif
 
@@ -282,10 +289,19 @@ CLocalListView::~CLocalListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
 	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str);
+
+#ifdef __WXMSW__
+	delete m_pVolumeEnumeratorThread;
+#endif
 }
 
 bool CLocalListView::DisplayDir(wxString dirname)
 {
+#ifdef __WXMSW__
+	if (GetEditControl())
+		ListView_CancelEditLabel((HWND)GetHandle());
+#endif
+
 	wxString focused;
 	std::list<wxString> selectedNames;
 	if (m_dir != dirname)
@@ -716,9 +732,6 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		{
 			// Enter action
 
-			if (IsComparing())
-				ExitComparisonMode();
-
 			wxString error;
 			if (!m_pState->SetLocalDir(data->name, &error))
 			{
@@ -801,9 +814,6 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &event)
 		return;
 	}
 
-	if (IsComparing())
-		ExitComparisonMode();
-
 	wxString error;
 	if (!m_pState->SetLocalDir(data->name, &error))
 	{
@@ -842,7 +852,7 @@ void CLocalListView::DisplayDrives()
 
 	int count = m_fileData.size();
 	int drive_count = 0;
-	while(*pDrive)
+	while (*pDrive)
 	{
 		// Check if drive should be hidden by default
 		if (pDrive[0] != 0 && pDrive[1] == ':')
@@ -870,18 +880,7 @@ void CLocalListView::DisplayDrives()
 		CLocalFileData data;
 		data.flags = normal;
 		data.name = path;
-
-		// Get the label of the drive
-		wxChar* pVolumeName = new wxChar[501];
-		int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-		BOOL res = GetVolumeInformation(pDrive, pVolumeName, 500, 0, 0, 0, 0, 0);
-		SetErrorMode(oldErrorMode);
-		if (res && pVolumeName[0])
-			data.label = data.name + _T(" (") + pVolumeName + _T(")");
-		else
-			data.label = data.name;
-		delete [] pVolumeName;
-
+		data.label = data.name;
 		data.dir = true;
 		data.icon = -2;
 		data.size = -1;
@@ -897,6 +896,16 @@ void CLocalListView::DisplayDrives()
 
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->SetDirectoryContents(0, drive_count, 0, false, 0);
+
+	if (!m_pVolumeEnumeratorThread)
+	{
+		m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
+		if (m_pVolumeEnumeratorThread->Failed())
+		{
+			delete m_pVolumeEnumeratorThread;
+			m_pVolumeEnumeratorThread = 0;
+		}
+	}
 }
 
 void CLocalListView::DisplayShares(wxString computer)
@@ -1258,7 +1267,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	const bool connected = m_pState->IsRemoteConnected();
 	if (!connected)
 	{
-		pMenu->Enable(XRCID("ID_EDIT"), false);
+		pMenu->Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
 		pMenu->Enable(XRCID("ID_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -1299,19 +1308,15 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	{
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_RENAME"), false);
-		pMenu->Enable(XRCID("ID_OPEN"), false);
-		pMenu->Enable(XRCID("ID_EDIT"), false);
 	}
 	else
 	{
 		// Exactly one item selected
-		if (selectedDir)
-			pMenu->Enable(XRCID("ID_EDIT"), false);
-		else
+		if (!selectedDir)
 			pMenu->Delete(XRCID("ID_ENTER"));
 	}
-	if (fillCount)
-		pMenu->Enable(XRCID("ID_OPEN"), false);
+	if (selectedDir)
+		pMenu->Enable(XRCID("ID_EDIT"), false);
 
 	PopupMenu(pMenu);
 	delete pMenu;
@@ -1493,9 +1498,6 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 			return;
 		}
 
-		if (IsComparing())
-			ExitComparisonMode();
-
 		wxString error;
 		if (!m_pState->SetLocalDir(_T(".."), &error))
 		{
@@ -1821,7 +1823,7 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	}
 }
 
-void CLocalListView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		DisplayDir(m_pState->GetLocalDir().GetPath());
@@ -2204,114 +2206,138 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 
 void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 {
-	const CServer* const pServer = m_pState->GetServer();
-	if (!pServer)
+	CServer server;
+	CServerPath path;
+
+	if (!m_pState->GetServer())
 	{
-		wxMessageBox(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
-		return;
-	}
-
-	const CServerPath& path = m_pState->GetRemotePath();
-	if (path.IsEmpty())
-	{
-		wxMessageBox(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
-		return;
-	}
-
-	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	if (item < 1)
-	{
-		wxBell();
-		return;
-	}
-
-	if (GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1)
-	{
-		wxBell();
-		return;
-	}
-
-	if (!item && m_hasParent)
-	{
-		wxBell();
-		return;
-	}
-
-	CEditHandler* pEditHandler = CEditHandler::Get();
-
-	CLocalFileData *data = GetData(item);
-	if (!data)
-	{
-		wxBell();
-		return;
-	}
-
-	if (data->dir || data->flags == fill)
-	{
-		wxBell();
-		return;
-	}
-
-	wxFileName fn(m_dir, data->name);
-
-	bool dangerous = false;
-	bool program_exists = false;
-	wxString cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
-	if (cmd.empty())
-	{
-		CNewAssociationDialog dlg(this);
-		if (!dlg.Show(fn.GetFullName()))
-			return;
-		cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
-		if (cmd.empty())
+		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
 		{
-			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+			wxMessageBox(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
-	if (!program_exists)
+	else
 	{
-		wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
-		wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
-		return;
+		server = *m_pState->GetServer();
+
+		path = m_pState->GetRemotePath();
+		if (path.IsEmpty())
+		{
+			wxMessageBox(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
+			return;
+		}
 	}
-	if (dangerous)
+
+	std::list<CLocalFileData> selected_item_list;
+
+	long item = -1;
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		int res = wxMessageBox(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
-		if (res != wxYES)
+		if (!item && m_hasParent)
 		{
 			wxBell();
 			return;
 		}
+
+		const CLocalFileData *data = GetData(item);
+		if (!data)
+			continue;
+
+		if (data->dir)
+		{
+			wxBell();
+			return;
+		}
+
+		if (data->flags == fill)
+			continue;
+
+		selected_item_list.push_back(*data);
 	}
 
-	CEditHandler::fileState state = pEditHandler->GetFileState(fn.GetFullPath());
-	switch (state)
+	CEditHandler* pEditHandler = CEditHandler::Get();
+
+	if (selected_item_list.empty())
 	{
-	case CEditHandler::upload:
-	case CEditHandler::upload_and_remove:
-		wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+		wxBell();
 		return;
-	case CEditHandler::edit:
+	}
+
+	if (selected_item_list.size() > 10)
+	{
+
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return;
+	}
+
+	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); data++)
+	{
+		wxFileName fn(m_dir, data->name);
+
+		bool dangerous = false;
+		bool program_exists = false;
+		wxString cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
+		if (cmd.empty())
 		{
-			int res = wxMessageBox(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath().c_str()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
+			CNewAssociationDialog dlg(this);
+			if (!dlg.Show(fn.GetFullName()))
+				continue;
+			cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
+			if (cmd.empty())
+			{
+				wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+				continue;
+			}
+		}
+		if (!program_exists)
+		{
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
+			wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			continue;
+		}
+		if (dangerous)
+		{
+			int res = wxMessageBox(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
 			if (res != wxYES)
 			{
 				wxBell();
-				return;
+				continue;
 			}
-			pEditHandler->StartEditing(fn.GetFullPath());
-			return;
 		}
-	default:
-		break;
-	}
 
-	wxString file = fn.GetFullPath();
-	if (!pEditHandler->AddFile(CEditHandler::local, file, path, *pServer))
-	{
-		wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
-		return;
+		CEditHandler::fileState state = pEditHandler->GetFileState(fn.GetFullPath());
+		switch (state)
+		{
+		case CEditHandler::upload:
+		case CEditHandler::upload_and_remove:
+			wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+			continue;
+		case CEditHandler::edit:
+			{
+				int res = wxMessageBox(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath().c_str()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
+				if (res != wxYES)
+				{
+					wxBell();
+					continue;
+				}
+				pEditHandler->StartEditing(fn.GetFullPath());
+				continue;
+			}
+		default:
+			break;
+		}
+
+		wxString file = fn.GetFullPath();
+		if (!pEditHandler->AddFile(CEditHandler::local, file, path, server))
+		{
+			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+			continue;
+		}
 	}
 }
 
@@ -2323,78 +2349,94 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 		wxLaunchDefaultBrowser(CState::GetAsURL(m_dir));
 		return;
 	}
-	else if (item < (m_hasParent ? 1 : 0))
-	{
-		wxBell();
-		return;
-	}
 
-	if (GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1)
-	{
-		wxBell();
-		return;
-	}
+	std::list<CLocalFileData> selected_item_list;
 
-	if (!item && m_hasParent)
+	item = -1;
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		wxBell();
-		return;
-	}
-
-	CEditHandler* pEditHandler = CEditHandler::Get();
-
-	CLocalFileData *data = GetData(item);
-	if (!data)
-	{
-		wxBell();
-		return;
-	}
-
-	if (data->flags == fill)
-	{
-		wxBell();
-		return;
-	}
-
-	if (data->dir)
-	{
-		CLocalPath path(m_dir);
-		if (!path.ChangePath(data->name))
+		if (!item && m_hasParent)
 		{
 			wxBell();
 			return;
 		}
 
-		wxLaunchDefaultBrowser(CState::GetAsURL(path.GetPath()));
+		const CLocalFileData *data = GetData(item);
+		if (!data)
+			continue;
+
+		if (data->flags == fill)
+			continue;
+
+		selected_item_list.push_back(*data);
+	}
+
+	CEditHandler* pEditHandler = CEditHandler::Get();
+	if (!pEditHandler)
+	{
+		wxBell();
 		return;
 	}
 
-	wxFileName fn(m_dir, data->name);
+	if (selected_item_list.empty())
+	{
+		wxBell();
+		return;
+	}
 
-	bool program_exists = false;
-	wxString cmd = pEditHandler->GetSystemOpenCommand(fn.GetFullPath(), program_exists);
-	if (cmd == _T(""))
+	if (selected_item_list.size() > 10)
 	{
-		int pos = data->name.Find('.') == -1;
-		if (pos == -1 || (pos == 0 && data->name.Mid(1).Find('.') == -1))
-			cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
-	}
-	if (cmd == _T(""))
-	{
-		wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
-		return;
-	}
-	if (!program_exists)
-	{
-		wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
-		wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
-		return;
-	}
-	
-	if (wxExecute(cmd))
-		return;
 
-	wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return;
+	}
+
+	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); data++)
+	{
+		if (data->dir)
+		{
+			CLocalPath path(m_dir);
+			if (!path.ChangePath(data->name))
+			{
+				wxBell();
+				continue;
+			}
+
+			wxLaunchDefaultBrowser(CState::GetAsURL(path.GetPath()));
+			continue;
+		}
+
+		wxFileName fn(m_dir, data->name);
+
+		bool program_exists = false;
+		wxString cmd = pEditHandler->GetSystemOpenCommand(fn.GetFullPath(), program_exists);
+		if (cmd == _T(""))
+		{
+			int pos = data->name.Find('.') == -1;
+			if (pos == -1 || (pos == 0 && data->name.Mid(1).Find('.') == -1))
+				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
+		}
+		if (cmd == _T(""))
+		{
+			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+			continue;
+		}
+		if (!program_exists)
+		{
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
+			wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			continue;
+		}
+
+		if (wxExecute(cmd))
+			continue;
+
+		wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+	}
 }
 
 bool CLocalListView::ItemIsDir(int index) const
@@ -2406,3 +2448,44 @@ wxLongLong CLocalListView::ItemGetSize(int index) const
 {
 	return m_fileData[index].size;
 }
+
+#ifdef __WXMSW__
+
+void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
+{
+	if (!m_pVolumeEnumeratorThread)
+		return;
+
+	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
+	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
+
+	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED)
+	{
+		delete m_pVolumeEnumeratorThread;
+		m_pVolumeEnumeratorThread = 0;
+	}
+
+	if (m_dir != _T("\\"))
+		return;
+
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); iter++)
+	{
+		wxString drive = iter->volume;
+
+		unsigned int item, index;
+		for (item = 1; item < m_indexMapping.size(); item++)
+		{
+			index = m_indexMapping[item];
+			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" "))
+				break;
+		}
+		if (item == m_indexMapping.size())
+			continue;
+
+		m_fileData[index].label = drive + _T(" (") + iter->volumeName + _T(")");
+		
+		RefreshItem(item);
+	}
+}
+
+#endif
