@@ -6,6 +6,9 @@
 
 class CQueueView;
 class CLocalListViewDropTarget;
+#ifdef __WXMSW__
+class CVolumeDescriptionEnumeratorThread;
+#endif
 
 class CLocalFileData : public CGenericFileData
 {
@@ -31,7 +34,7 @@ public:
 	virtual ~CLocalListView();
 
 protected:
-	void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 	bool DisplayDir(wxString dirname);
 	void ApplyCurrentFilter();
 
@@ -98,6 +101,10 @@ protected:
 	void OnMenuOpen(wxCommandEvent& event);
 	void OnMenuEdit(wxCommandEvent& event);
 	void OnMenuEnter(wxCommandEvent& event);
+#ifdef __WXMSW__
+	void OnVolumesEnumerated(wxCommandEvent& event);
+	CVolumeDescriptionEnumeratorThread* m_pVolumeEnumeratorThread;
+#endif
 };
 
 #endif
