@@ -17,6 +17,7 @@
 #include <wx/msw/registry.h>
 #include <shlobj.h>
 #include <dbt.h>
+#include "volume_enumerator.h"
 #endif
 
 class CTreeItemData : public wxTreeItemData
@@ -201,12 +202,16 @@ IMPLEMENT_CLASS(CLocalTreeView, wxTreeCtrl)
 
 BEGIN_EVENT_TABLE(CLocalTreeView, wxTreeCtrl)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CLocalTreeView::OnItemExpanding)
+#ifdef __WXMSW__
+EVT_TREE_SEL_CHANGING(wxID_ANY, CLocalTreeView::OnSelectionChanging)
+#endif
 EVT_TREE_SEL_CHANGED(wxID_ANY, CLocalTreeView::OnSelectionChanged)
 EVT_TREE_BEGIN_DRAG(wxID_ANY, CLocalTreeView::OnBeginDrag)
 #ifndef __WXMSW__
 EVT_KEY_DOWN(CLocalTreeView::OnKeyDown)
 #else
 EVT_COMMAND(-1, fzEVT_VOLUMESENUMERATED, CLocalTreeView::OnVolumesEnumerated)
+EVT_COMMAND(-1, fzEVT_VOLUMEENUMERATED, CLocalTreeView::OnVolumesEnumerated)
 #endif //__WXMSW__
 EVT_TREE_ITEM_MENU(wxID_ANY, CLocalTreeView::OnContextMenu)
 EVT_MENU(XRCID("ID_UPLOAD"), CLocalTreeView::OnMenuUpload)
@@ -298,8 +303,10 @@ void CLocalTreeView::SetDir(wxString localDir)
 	{
 		wxTreeItemIdValue value;
 		wxTreeItemId child = GetFirstChild(parent, value);
-		if (child && GetItemText(child) == _T(""))
-			DisplayDir(parent, localDir);
+
+		//Not needed, item stays unexpanded
+		//if (child && GetItemText(child) == _T(""))
+		//	DisplayDir(parent, localDir);
 
 		m_setSelection = true;
 		SelectItem(parent);
@@ -477,7 +484,13 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 					return;
 
 				const wxString fullName = dirname + knownSubdir;
-				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+						-1
+#else
+						GetIconIndex(opened_dir, fullName)
+#endif
+					);
 				CheckSubdirStatus(item, fullName);
 			}
 			else
@@ -526,14 +539,28 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+				-1
+#else
+				GetIconIndex(opened_dir, fullName)
+#endif
+			);
+
 		CheckSubdirStatus(item, fullName);
 	}
 
 	if (!matchedKnown && knownSubdir != _T(""))
 	{
 		const wxString fullName = dirname + knownSubdir;
-		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName), GetIconIndex(opened_dir, fullName));
+		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+#ifdef __WXMSW__
+				-1
+#else
+				GetIconIndex(opened_dir, fullName)
+#endif
+			);
+
 		CheckSubdirStatus(item, fullName);
 	}
 
@@ -602,7 +629,8 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 		dirname += segment + separator;
 	}
 
-	DisplayDir(parent, dirname);
+	// Not needed, stays unexpanded by default
+	// DisplayDir(parent, dirname);
 	return parent;
 }
 
@@ -824,7 +852,14 @@ void CLocalTreeView::Refresh()
 			else if (cmp < 0)
 			{
 				wxString fullname = dir.dir + *iter + separator;
-				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
+				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+#ifdef __WXMSW__
+						-1
+#else
+						GetIconIndex(opened_dir, fullname)
+#endif
+					);
+
 				CheckSubdirStatus(newItem, fullname);
 				iter++;
 				inserted = true;
@@ -843,7 +878,14 @@ void CLocalTreeView::Refresh()
 		while (iter != dirs.rend())
 		{
 			wxString fullname = dir.dir + *iter + separator;
-			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname), GetIconIndex(opened_dir, fullname));
+			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+#ifdef __WXMSW__
+					-1
+#else
+					GetIconIndex(opened_dir, fullname)
+#endif
+				);
+
 			CheckSubdirStatus(newItem, fullname);
 			iter++;
 			inserted = true;
@@ -895,7 +937,7 @@ int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId
 #endif
 }
 
-void CLocalTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		SetDir(m_pState->GetLocalDir().GetPath());
@@ -1057,8 +1099,11 @@ void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
 
-	delete m_pVolumeEnumeratorThread;
-	m_pVolumeEnumeratorThread = 0;
+	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED)
+	{
+		delete m_pVolumeEnumeratorThread;
+		m_pVolumeEnumeratorThread = 0;
+	}
 
 	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); iter++)
 	{
@@ -1079,158 +1124,6 @@ void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 	}
 }
 
-DEFINE_EVENT_TYPE(fzEVT_VOLUMESENUMERATED)
-
-CVolumeDescriptionEnumeratorThread::CVolumeDescriptionEnumeratorThread(wxEvtHandler* pEvtHandler)
-	: wxThreadEx(wxTHREAD_JOINABLE),
-	  m_pEvtHandler(pEvtHandler)
-{
-	m_failure = false;
-	m_stop = false;
-	m_running = true;
-
-	if (Create() != wxTHREAD_NO_ERROR)
-	{
-		m_running = false;
-		m_failure = true;
-	}
-	Run();
-}
-
-CVolumeDescriptionEnumeratorThread::~CVolumeDescriptionEnumeratorThread()
-{
-	m_stop = true;
-	Wait();
-
-	for (std::list<t_VolumeInfoInternal>::const_iterator iter = m_volumeInfo.begin(); iter != m_volumeInfo.end(); iter++)
-	{
-		delete [] iter->pVolume;
-		delete [] iter->pVolumeName;
-	}
-	m_volumeInfo.clear();
-}
-
-wxThread::ExitCode CVolumeDescriptionEnumeratorThread::Entry()
-{
-	if (!GetDrives())
-		m_failure = true;
-
-	m_running = false;
-
-	wxCommandEvent evt(fzEVT_VOLUMESENUMERATED);
-	m_pEvtHandler->AddPendingEvent(evt);
-
-	return 0;
-}
-
-bool CVolumeDescriptionEnumeratorThread::GetDrives()
-{
-	long drivesToHide = 0;
-	// Adhere to the NODRIVES group policy
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
-
-	int len = GetLogicalDriveStrings(0, 0);
-	if (!len)
-		return false;
-
-	wxChar* drives = new wxChar[len + 1];
-
-	if (!GetLogicalDriveStrings(len, drives))
-	{
-		delete [] drives;
-		return false;
-	}
-
-	const wxChar* pDrive = drives;
-	while (*pDrive)
-	{
-		if (m_stop)
-		{
-			delete [] drives;
-			return false;
-		}
-
-		// Check if drive should be hidden by default
-		if (pDrive[0] != 0 && pDrive[1] == ':')
-		{
-			int bit = 0;
-			char letter = pDrive[0];
-			if (letter >= 'A' && letter <= 'Z')
-				bit = 1 << (letter - 'A');
-			if (letter >= 'a' && letter <= 'z')
-				bit = 1 << (letter - 'a');
-
-			if (drivesToHide & bit)
-			{
-				pDrive += wxStrlen(pDrive) + 1;
-				continue;
-			}
-		}
-
-		int len = wxStrlen(pDrive);
-		wxChar* pVolume = new wxChar[len + 1];
-		wxStrcpy(pVolume, pDrive);
-		if (pVolume[len - 1] == '\\')
-			pVolume[len - 1] = 0;
-		if (!*pVolume)
-		{
-			delete [] pVolume;
-			pDrive += len + 1;
-			continue;
-		}
-
-		// Get the label of the drive
-		wxChar* pVolumeName = new wxChar[501];
-		int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-		BOOL res = GetVolumeInformation(pDrive, pVolumeName, 500, 0, 0, 0, 0, 0);
-		SetErrorMode(oldErrorMode);
-		if (res && pVolumeName[0])
-		{
-			t_VolumeInfoInternal volumeInfo;
-			volumeInfo.pVolume = pVolume;
-			volumeInfo.pVolumeName = pVolumeName;
-			m_volumeInfo.push_back(volumeInfo);
-		}
-		else
-		{
-			delete [] pVolumeName;
-			delete [] pVolume;
-		}
-
-		pDrive += wxStrlen(pDrive) + 1;
-	}
-
-	delete [] drives;
-
-	return true;
-}
-
-std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> CVolumeDescriptionEnumeratorThread::GetVolumes()
-{
-	std::list<t_VolumeInfo> volumeInfo;
-
-	if (m_running)
-		return volumeInfo;
-
-	for (std::list<t_VolumeInfoInternal>::const_iterator iter = m_volumeInfo.begin(); iter != m_volumeInfo.end(); iter++)
-	{
-		t_VolumeInfo info;
-		info.volume = iter->pVolume;
-		delete [] iter->pVolume;
-		info.volumeName = iter->pVolumeName;
-		delete [] iter->pVolumeName;
-		volumeInfo.push_back(info);
-	}
-	m_volumeInfo.clear();
-
-	return volumeInfo;
-}
-
 #endif
 
 void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
@@ -1714,4 +1607,16 @@ void CLocalTreeView::RemoveDrive(wxChar drive)
 	Delete(driveItem);
 }
 
+void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
+{
+	// On-demand open icon for selected items
+	wxTreeItemId item = event.GetItem();
+	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
+	{
+		int icon = GetIconIndex(opened_dir, GetDirFromItem(item));
+		SetItemImage(item, icon, wxTreeItemIcon_Selected);
+		SetItemImage(item, icon, wxTreeItemIcon_SelectedExpanded);
+	}
+}
+
 #endif
