@@ -26,7 +26,7 @@ public:
 #endif
 
 protected:
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	void SetDir(wxString localDir);
 	void Refresh();
@@ -54,6 +54,9 @@ protected:
 
 	DECLARE_EVENT_TABLE()
 	void OnItemExpanding(wxTreeEvent& event);
+#ifdef __WXMSW__
+	void OnSelectionChanging(wxTreeEvent& event);
+#endif
 	void OnSelectionChanged(wxTreeEvent& event);
 	void OnBeginDrag(wxTreeEvent& event);
 #ifndef __WXMSW__
@@ -87,53 +90,4 @@ protected:
 	wxTreeItemId m_dropHighlight;
 };
 
-#ifdef __WXMSW__
-
-DECLARE_EVENT_TYPE(fzEVT_VOLUMESENUMERATED, -1)
-
-// Windows has this very strange concept of drive letters (nowadays called
-// volumes), even if the drive isn't mounted (in the sense of no media
-// inserted).
-// This can result in a long seek time if trying to enumerate the volume
-// labels, especially with legacy floppy drives (why are people still using
-// them?)
-// Since the local directory including the drives is populated at startup,
-// use a background thread to obtain the labels.
-class CVolumeDescriptionEnumeratorThread : protected wxThreadEx
-{
-public:
-	CVolumeDescriptionEnumeratorThread(wxEvtHandler* pEvtHandler);
-	virtual ~CVolumeDescriptionEnumeratorThread();
-
-	bool Failed() const { return m_failure; }
-
-	struct t_VolumeInfo
-	{
-		wxString volume;
-		wxString volumeName;
-	};
-
-	std::list<t_VolumeInfo> GetVolumes();
-
-protected:
-	bool GetDrives();
-	virtual ExitCode Entry();
-
-	wxEvtHandler* m_pEvtHandler;
-
-	bool m_failure;
-	bool m_stop;
-	bool m_running;
-
-	struct t_VolumeInfoInternal
-	{
-		wxChar* pVolume;
-		wxChar* pVolumeName;
-	};
-
-	std::list<t_VolumeInfoInternal> m_volumeInfo;
-};
-
-#endif
-
 #endif
