@@ -12,7 +12,7 @@
 #include "asyncrequestqueue.h"
 #include "led.h"
 #include "sitemanager.h"
-#include "settingsdialog.h"
+#include "settings/settingsdialog.h"
 #include "themeprovider.h"
 #include "filezillaapp.h"
 #include "view.h"
@@ -42,13 +42,16 @@
 #include "auto_ascii_files.h"
 #include "splitter.h"
 #include "bookmarks_dialog.h"
+#ifndef __WXMAC__
+#include <wx/taskbar.h>
+#endif
+#include "search.h"
+#include "power_management.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #endif
 
-#define TRANSFERSTATUS_TIMER_ID wxID_HIGHEST + 3
-
 BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
 	EVT_SIZE(CMainFrame::OnSize)
 	EVT_MENU(wxID_ANY, CMainFrame::OnMenuHandler)
@@ -99,7 +102,13 @@ BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
 #endif
 	EVT_MENU(XRCID("ID_COMPARE_SIZE"), CMainFrame::OnDropdownComparisonMode)
 	EVT_MENU(XRCID("ID_COMPARE_DATE"), CMainFrame::OnDropdownComparisonMode)
+	EVT_MENU(XRCID("ID_COMPARE_HIDEIDENTICAL"), CMainFrame::OnDropdownComparisonHide)
 	EVT_TOOL(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), CMainFrame::OnSyncBrowse)
+#ifndef __WXMAC__
+	EVT_ICONIZE(CMainFrame::OnIconize)
+#endif
+	EVT_TOOL(XRCID("ID_TOOLBAR_FIND"), CMainFrame::OnSearch)
+	EVT_MENU(XRCID("ID_MENU_SERVER_SEARCH"), CMainFrame::OnSearch)
 END_EVENT_TABLE()
 
 class CMainFrameStateEventHandler : public CStateEventHandler
@@ -114,10 +123,11 @@ public:
 		pState->RegisterHandler(this, STATECHANGE_SERVER);
 		pState->RegisterHandler(this, STATECHANGE_QUEUEPROCESSING);
 		pState->RegisterHandler(this, STATECHANGE_SYNC_BROWSE);
+		pState->RegisterHandler(this, STATECHANGE_COMPARISON);
 	}
 
 protected:
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 	{
 		if (notification == STATECHANGE_QUEUEPROCESSING)
 		{
@@ -132,7 +142,7 @@ protected:
 				pMenuBar->Check(XRCID("ID_MENU_TRANSFER_PROCESSQUEUE"), check);
 			return;
 		}
-		if (notification == STATECHANGE_SERVER)
+		else if (notification == STATECHANGE_SERVER)
 		{
 			const CServer* pServer = m_pState->GetServer();
 
@@ -141,8 +151,19 @@ protected:
 				// Can only happen through quickconnect bar
 				m_pMainFrame->ClearBookmarks();
 			}
+
+			CStatusBar* const pStatusBar = m_pMainFrame->GetStatusBar();
+			if (pStatusBar)
+			{
+				pStatusBar->DisplayDataType(pServer);
+				pStatusBar->DisplayEncrypted(pServer);
+			}
+
+			if (!pServer)
+				m_pMainFrame->SetTitle(_T("FileZilla"));
+			// Don't return here, need to update toolbar state
 		}
-		if (notification == STATECHANGE_SYNC_BROWSE)
+		else if (notification == STATECHANGE_SYNC_BROWSE)
 		{
 			if (m_pMainFrame->GetToolBar())
 				m_pMainFrame->GetToolBar()->ToggleTool(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), m_pState->GetSyncBrowse());
@@ -150,6 +171,18 @@ protected:
 				m_pMainFrame->GetMenuBar()->Check(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), m_pState->GetSyncBrowse());
 			return;
 		}
+		else if (notification == STATECHANGE_COMPARISON)
+		{
+			bool is_comparing = pState->GetComparisonManager()->IsComparing();
+			wxToolBar* pToolBar = m_pMainFrame->GetToolBar();
+			if (pToolBar)
+				pToolBar->ToggleTool(XRCID("ID_TOOLBAR_COMPARISON"), is_comparing);
+
+			wxMenuBar* pMenuBar = m_pMainFrame->GetMenuBar();
+			if (pMenuBar)
+				pMenuBar->Check(XRCID("ID_TOOLBAR_COMPARISON"), is_comparing);
+			return;
+		}
 
 		m_pMainFrame->UpdateMenubarState();
 		m_pMainFrame->UpdateToolbarState();
@@ -166,9 +199,17 @@ CMainFrame::CMainFrame()
 	initial_size.x = wxMin(900, screen_size.GetWidth() - 10);
 	initial_size.y = wxMin(750, screen_size.GetHeight() - 50);
 
+	// Needed because on MSW we can get WM_DEVICECHANGE prior to creation of
+	// m_pLocalTreeView
+	m_pLocalTreeView = 0;
+
 	Create(NULL, -1, _T("FileZilla"), wxDefaultPosition, initial_size);
 	SetSizeHints(250, 250);
 
+#ifndef __WXMAC__
+	m_taskBarIcon = 0;
+#endif
+
 #ifdef __WXMSW__
 	// In order for the --close commandline argument to work,
 	// there has to be a way to find other instances.
@@ -205,27 +246,29 @@ CMainFrame::CMainFrame()
 	m_pStatusView = 0;
 
 	m_pThemeProvider = new CThemeProvider();
-	m_pState = new CState(this);
+	m_pState = CContextManager::Get()->CreateState(this);
 	m_pStateEventHandler = new CMainFrameStateEventHandler(m_pState, this);
 
+	CPowerManagement::Create(this);
+
 	m_pStatusBar = new CStatusBar(this);
 	if (m_pStatusBar)
 	{
-		m_pRecvLed = new CLed(m_pStatusBar, 1, m_pState);
-		m_pSendLed = new CLed(m_pStatusBar, 0, m_pState);
+		m_pActivityLed[0] = new CLed(m_pStatusBar, 0);
+		m_pActivityLed[1] = new CLed(m_pStatusBar, 1);
 
-		m_pStatusBar->AddChild(-1, m_pRecvLed, 2);
-		m_pStatusBar->AddChild(-1, m_pSendLed, 16);
+		m_pStatusBar->AddChild(-1, m_pActivityLed[1], 2);
+		m_pStatusBar->AddChild(-1, m_pActivityLed[0], 16);
 
 		SetStatusBar(m_pStatusBar);
 	}
 	else
 	{
-		m_pRecvLed = 0;
-		m_pSendLed = 0;
+		m_pActivityLed[0] = 0;
+		m_pActivityLed[1] = 0;
 	}
 
-	m_transferStatusTimer.SetOwner(this, TRANSFERSTATUS_TIMER_ID);
+	m_transferStatusTimer.SetOwner(this);
 	m_closeEventTimer.SetOwner(this);
 
 	if (CFilterManager::HasActiveFilters(true))
@@ -249,6 +292,12 @@ CMainFrame::CMainFrame()
 		wxMessageBox(_("Failed to initialize FTP engine"));
 	}
 
+	if (m_pActivityLed[0])
+	{
+		m_pActivityLed[0]->SetEngine(m_pState->m_pEngine);
+		m_pActivityLed[1]->SetEngine(m_pState->m_pEngine);
+	}
+
 #ifdef __WXMSW__
 	long style = wxSP_NOBORDER | wxSP_LIVE_UPDATE;
 #else
@@ -457,8 +506,9 @@ CMainFrame::CMainFrame()
 		m_pState->SetLocalDir(_T("/"));
 
 	wxAcceleratorEntry entries[1];
-	entries[0].Set(wxACCEL_NORMAL, WXK_F5, XRCID("ID_TOOLBAR_REFRESH"));
-	wxAcceleratorTable accel(1, entries);
+	entries[0].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
+
+	wxAcceleratorTable accel(sizeof(entries) / sizeof(wxAcceleratorEntry), entries);
 	SetAcceleratorTable(accel);
 
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
@@ -488,23 +538,25 @@ CMainFrame::CMainFrame()
 
 	CEditHandler::Create()->SetQueue(m_pQueueView);
 
-	m_pComparisonManager = 0;
-
 	InitMenubarState();
 	InitToolbarState();
 
 	CAutoAsciiFiles::SettingsChanged();
+
+	m_pState->GetComparisonManager()->SetListings(m_pLocalListView, m_pRemoteListView);
 }
 
 CMainFrame::~CMainFrame()
 {
+	CPowerManagement::Destroy();
+
 	delete m_pStateEventHandler;
-	delete m_pState;
+	CContextManager::Get()->DestroyState(m_pState);
+	m_pState = 0;
 	delete m_pAsyncRequestQueue;
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 	delete m_pUpdateWizard;
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	delete m_pComparisonManager;
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
 	if (pEditHandler)
@@ -513,11 +565,18 @@ CMainFrame::~CMainFrame()
 		// edit handler should clean them on next startup
 		pEditHandler->Release();
 	}
+
+#ifndef __WXMAC__
+	delete m_taskBarIcon;
+#endif
 }
 
 void CMainFrame::HandleResize()
 {
 	wxSize clientSize = GetClientSize();
+	if (clientSize.y <= 0) // Can happen if restoring from tray on XP if using ugly XP themes
+		return;
+
 	if (m_pQuickconnectBar)
 		m_pQuickconnectBar->SetSize(0, 0, clientSize.GetWidth(), -1, wxSIZE_USE_EXISTING);
 	if (m_pTopSplitter)
@@ -612,17 +671,6 @@ bool CMainFrame::CreateMenus()
 		m_pUpdateWizard->DisplayUpdateAvailability(false, true);
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 
-	if (m_pMenuBar)
-	{
-		m_pMenuBar->FindItem(XRCID("ID_MENU_SERVER_VIEWHIDDEN"), 0)->Check(COptions::Get()->GetOptionVal(OPTION_VIEW_HIDDEN_FILES) ? true : false);
-
-		int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
-		if (mode != 1)
-			m_pMenuBar->Check(XRCID("ID_COMPARE_SIZE"), true);
-		else
-			m_pMenuBar->Check(XRCID("ID_COMPARE_DATE"), true);
-	}
-
 	UpdateBookmarkMenu();
 
 	InitMenubarState();
@@ -980,7 +1028,15 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 					m_pState->ChangeRemoteDir(pData->m_remoteDir);
 			}
 			if (!pData->m_localDir.empty())
-				m_pState->SetLocalDir(pData->m_localDir);
+			{
+				bool set = m_pState->SetLocalDir(pData->m_localDir);
+
+				if (set && pData->m_sync)
+				{
+					wxASSERT(!pData->m_remoteDir.IsEmpty());
+					m_pState->SetSyncBrowse(true, pData->m_remoteDir);
+				}
+			}
 
 			delete pData;
 
@@ -993,7 +1049,8 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			// We hit a global bookmark
 			wxString local_dir;
 			CServerPath remote_dir;
-			if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir))
+			bool sync;
+			if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync))
 				return;
 
 			m_pState->SetSyncBrowse(false);
@@ -1012,7 +1069,15 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 				}
 			}
 			if (!local_dir.empty())
-				m_pState->SetLocalDir(local_dir);
+			{
+				bool set = m_pState->SetLocalDir(local_dir);
+
+				if (set && sync)
+				{
+					wxASSERT(!remote_dir.IsEmpty());
+					m_pState->SetSyncBrowse(true, remote_dir);
+				}
+			}
 		}
 
 		wxString path;
@@ -1134,10 +1199,7 @@ void CMainFrame::OnEngineEvent(wxEvent &event)
 		case nId_active:
 			{
 				CActiveNotification *pActiveNotification = reinterpret_cast<CActiveNotification *>(pNotification);
-				if (pActiveNotification->IsRecv())
-					UpdateRecvLed();
-				else
-					UpdateSendLed();
+				UpdateActivityLed(pActiveNotification->GetDirection());
 				delete pNotification;
 			}
 			break;
@@ -1428,8 +1490,14 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	Show(false);
 
 	// Getting deleted by wxWidgets
-	m_pSendLed = 0;
-	m_pRecvLed = 0;
+	for (int i = 0; i < 2; i++)
+	{
+		if (!m_pActivityLed[i])
+			continue;
+
+		m_pActivityLed[i]->SetEngine(0);
+		m_pActivityLed[i] = 0;
+	}
 	m_pStatusBar = 0;
 	m_pMenuBar = 0;
 	m_pToolBar = 0;
@@ -1514,7 +1582,7 @@ void CMainFrame::OnRefresh(wxCommandEvent &event)
 
 void CMainFrame::OnTimer(wxTimerEvent& event)
 {
-	if (event.GetId() == TRANSFERSTATUS_TIMER_ID && m_transferStatusTimer.IsRunning())
+	if (m_transferStatusTimer.IsRunning() && event.GetId() == m_transferStatusTimer.GetId())
 	{
 		if (!m_pState->m_pEngine)
 		{
@@ -1628,16 +1696,10 @@ void CMainFrame::OnSiteManager(wxCommandEvent& event)
 	OpenSiteManager();
 }
 
-void CMainFrame::UpdateSendLed()
+void CMainFrame::UpdateActivityLed(int direction)
 {
-	if (m_pSendLed)
-		m_pSendLed->Ping();
-}
-
-void CMainFrame::UpdateRecvLed()
-{
-	if (m_pRecvLed)
-		m_pRecvLed->Ping();
+	if (m_pActivityLed[direction])
+		m_pActivityLed[direction]->Ping();
 }
 
 void CMainFrame::AddToRequestQueue(CFileZillaEngine *pEngine, CAsyncRequestNotification *pNotification)
@@ -1921,6 +1983,12 @@ void CMainFrame::OnMenuHelpAbout(wxCommandEvent& event)
 
 void CMainFrame::OnFilter(wxCommandEvent& event)
 {
+	if (wxGetKeyState(WXK_SHIFT))
+	{
+		OnFilterRightclicked(event);
+		return;
+	}
+
 	CFilterDialog dlg;
 	if (m_pToolBar)
 		m_pToolBar->ToggleTool(XRCID("ID_TOOLBAR_FILTER"), dlg.HasActiveFilters());
@@ -2169,7 +2237,16 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site* const pData)
 		return false;
 
 	if (pData->m_localDir != _T(""))
-		m_pState->SetLocalDir(pData->m_localDir);
+	{
+		bool set = m_pState->SetLocalDir(pData->m_localDir);
+
+		if (set && pData->m_sync)
+		{
+			wxASSERT(!pData->m_remoteDir.IsEmpty());
+
+			m_pState->SetSyncBrowse(true, pData->m_remoteDir);
+		}
+	}
 
 	return true;
 }
@@ -2439,24 +2516,21 @@ void CMainFrame::OnActivate(wxActivateEvent& event)
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
 	if (pEditHandler)
-		pEditHandler->CheckForModifications(
-#ifdef __WXMAC__
-				true
-#endif
-			);
+		pEditHandler->CheckForModifications(true);
+
+	if (m_pAsyncRequestQueue)
+		m_pAsyncRequestQueue->TriggerProcessing();
 }
 
 void CMainFrame::OnToolbarComparison(wxCommandEvent& event)
 {
-	if (m_pComparisonManager && m_pComparisonManager->IsComparing())
+	CComparisonManager* pComparisonManager = m_pState->GetComparisonManager();
+	if (pComparisonManager->IsComparing())
 	{
-		m_pComparisonManager->ExitComparisonMode();
+		pComparisonManager->ExitComparisonMode();
 		return;
 	}
 
-	if (!m_pComparisonManager)
-		m_pComparisonManager = new CComparisonManager(this, m_pLocalListView, m_pRemoteListView);
-
 	if (!COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT))
 	{
 		if ((m_pLocalSplitter->IsSplit() && !m_pRemoteSplitter->IsSplit()) ||
@@ -2469,7 +2543,8 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent& event)
 			dlg.AddText(_("Show both directory trees and continue comparing?"));
 			if (!dlg.Run())
 			{
-				m_pComparisonManager->UpdateToolState();
+				// Needed to restore non-toggle state of button
+				m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
 				return;
 			}
 
@@ -2481,7 +2556,7 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent& event)
 		m_pRemoteSplitter->SetSashPosition(pos);
 	}
 
-	m_pComparisonManager->CompareListings();
+	pComparisonManager->CompareListings();
 }
 
 void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
@@ -2490,7 +2565,8 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 	if (!pMenu)
 		return;
 
-	pMenu->FindItem(XRCID("ID_TOOLBAR_COMPARISON"))->Check(m_pComparisonManager && m_pComparisonManager->IsComparing());
+	CComparisonManager* pComparisonManager = m_pState->GetComparisonManager();
+	pMenu->FindItem(XRCID("ID_TOOLBAR_COMPARISON"))->Check(pComparisonManager->IsComparing());
 
 	const int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
 	if (mode == 0)
@@ -2498,6 +2574,8 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 	else
 		pMenu->FindItem(XRCID("ID_COMPARE_DATE"))->Check();
 
+	pMenu->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
+
 	ShowDropdownMenu(pMenu, m_pToolBar, event);
 }
 
@@ -2538,8 +2616,26 @@ void CMainFrame::OnDropdownComparisonMode(wxCommandEvent& event)
 			m_pMenuBar->Check(XRCID("ID_COMPARE_DATE"), true);
 	}
 
-	if (old_mode != new_mode && m_pComparisonManager && m_pComparisonManager->IsComparing())
-		m_pComparisonManager->CompareListings();
+	CComparisonManager* pComparisonManager = m_pState->GetComparisonManager();
+	if (old_mode != new_mode && pComparisonManager && pComparisonManager->IsComparing())
+		pComparisonManager->CompareListings();
+}
+
+void CMainFrame::OnDropdownComparisonHide(wxCommandEvent& event)
+{
+	bool old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0;
+	bool new_mode = event.IsChecked();
+
+	COptions::Get()->SetOption(OPTION_COMPARE_HIDEIDENTICAL, new_mode ? 1 : 0);
+
+	if (m_pMenuBar)
+	{
+		m_pMenuBar->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), new_mode);
+	}
+
+	CComparisonManager* pComparisonManager = m_pState->GetComparisonManager();
+	if (old_mode != new_mode && pComparisonManager && pComparisonManager->IsComparing())
+		pComparisonManager->CompareListings();
 }
 
 void CMainFrame::InitToolbarState()
@@ -2565,6 +2661,7 @@ void CMainFrame::UpdateToolbarState()
 	m_pToolBar->EnableTool(XRCID("ID_TOOLBAR_CANCEL"), pServer && !idle);
 	m_pToolBar->EnableTool(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	m_pToolBar->EnableTool(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
+	m_pToolBar->EnableTool(XRCID("ID_TOOLBAR_FIND"), pServer && idle);
 
 	bool canReconnect;
 	if (pServer || !idle)
@@ -2581,6 +2678,17 @@ void CMainFrame::InitMenubarState()
 {
 	if (!m_pMenuBar)
 		return;
+
+	m_pMenuBar->Check(XRCID("ID_MENU_SERVER_VIEWHIDDEN"), COptions::Get()->GetOptionVal(OPTION_VIEW_HIDDEN_FILES) ? true : false);
+
+	int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
+	if (mode != 1)
+		m_pMenuBar->Check(XRCID("ID_COMPARE_SIZE"), true);
+	else
+		m_pMenuBar->Check(XRCID("ID_COMPARE_DATE"), true);
+
+	m_pMenuBar->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
+
 	m_pMenuBar->Check(XRCID("ID_VIEW_QUICKCONNECT"), m_pQuickconnectBar != 0);
 	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) != 2)
 		m_pMenuBar->Check(XRCID("ID_VIEW_MESSAGELOG"), m_pStatusView && m_pStatusView->IsShown());
@@ -2605,6 +2713,7 @@ void CMainFrame::UpdateMenubarState()
 	m_pMenuBar->Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), pServer != 0);
 	m_pMenuBar->Enable(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	m_pMenuBar->Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
+	m_pMenuBar->Enable(XRCID("ID_MENU_SERVER_SEARCH"), pServer && idle);
 
 	bool canReconnect;
 	if (pServer || !idle)
@@ -2652,6 +2761,13 @@ void CMainFrame::ProcessCommandLine()
 		wxString error;
 
 		CServer server;
+
+		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
+		if (logontype == _T("ask"))
+			server.SetLogonType(ASK);
+		else if (logontype == _T("interactive"))
+			server.SetLogonType(INTERACTIVE);
+
 		CServerPath path;
 		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path))
 		{
@@ -2660,6 +2776,13 @@ void CMainFrame::ProcessCommandLine()
 			wxMessageBox(error, _("Syntax error in command line"));
 		}
 
+		if (server.GetLogonType() == ASK ||
+			(server.GetLogonType() == INTERACTIVE && server.GetUser() == _T("")))
+		{
+			if (!CLoginManager::Get().GetPassword(server, false))
+				return;
+		}
+
 		m_pState->Connect(server, true, path);
 	}
 }
@@ -2753,6 +2876,8 @@ void CMainFrame::UpdateBookmarkMenu()
 	{
 		pMenu->AppendSeparator();
 
+		m_bookmark_menu_id_map_global.clear();
+
 		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); iter++)
 		{
 			int id;
@@ -2814,3 +2939,63 @@ void CMainFrame::OnSyncBrowse(wxCommandEvent& event)
 	if (m_pToolBar)
 		m_pToolBar->ToggleTool(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), m_pState->GetSyncBrowse());
 }
+
+#ifndef __WXMAC__
+void CMainFrame::OnIconize(wxIconizeEvent& event)
+{
+	if (!event.Iconized())
+	{
+		if (m_taskBarIcon)
+			m_taskBarIcon->RemoveIcon();
+		Show(true);
+
+		if (m_pAsyncRequestQueue)
+			m_pAsyncRequestQueue->TriggerProcessing();
+
+		return;
+	}
+
+
+	if (!COptions::Get()->GetOptionVal(OPTION_MINIMIZE_TRAY))
+		return;
+
+	if (!m_taskBarIcon)
+	{
+		m_taskBarIcon = new wxTaskBarIcon();
+		m_taskBarIcon->Connect(wxEVT_TASKBAR_LEFT_DCLICK, wxTaskBarIconEventHandler(CMainFrame::OnTaskBarClick), 0, this);
+		m_taskBarIcon->Connect(wxEVT_TASKBAR_LEFT_UP, wxTaskBarIconEventHandler(CMainFrame::OnTaskBarClick), 0, this);
+		m_taskBarIcon->Connect(wxEVT_TASKBAR_RIGHT_UP, wxTaskBarIconEventHandler(CMainFrame::OnTaskBarClick), 0, this);
+	}
+
+	bool installed;
+	if (!m_taskBarIcon->IsIconInstalled())
+		installed = m_taskBarIcon->SetIcon(CThemeProvider::GetIcon(_T("ART_FILEZILLA")), GetTitle());
+	else
+		installed = true;
+
+	if (installed)
+		Show(false);
+}
+
+void CMainFrame::OnTaskBarClick(wxTaskBarIconEvent& event)
+{
+	if (m_taskBarIcon)
+		m_taskBarIcon->RemoveIcon();
+
+	Show(true);
+	Iconize(false);
+
+	if (m_pAsyncRequestQueue)
+		m_pAsyncRequestQueue->TriggerProcessing();
+}
+
+#endif
+
+void CMainFrame::OnSearch(wxCommandEvent& event)
+{
+	CSearchDialog dlg(this, m_pState, m_pQueueView);
+	if (!dlg.Load())
+		return;
+
+	dlg.Run();
+}
