@@ -1,6 +1,10 @@
 #ifndef __MAINFRM_H__
 #define __MAINFRM_H__
 
+#ifndef __WXMAC__
+#include <wx/taskbar.h>
+#endif
+
 class CStatusView;
 class CQueueView;
 class CLocalTreeView;
@@ -19,7 +23,6 @@ class CUpdateWizard;
 class CSiteManagerItemData_Site;
 class CQueue;
 class CViewHeader;
-class CComparisonManager;
 class CWindowStateManager;
 class CStatusBar;
 class CMainFrameStateEventHandler;
@@ -31,8 +34,7 @@ public:
 	CMainFrame();
 	virtual ~CMainFrame();
 
-	void UpdateSendLed();
-	void UpdateRecvLed();
+	void UpdateActivityLed(int direction);
 
 	void AddToRequestQueue(CFileZillaEngine* pEngine, CAsyncRequestNotification* pNotification);
 	CState* GetState() { return m_pState; }
@@ -61,7 +63,7 @@ public:
 	void ProcessCommandLine();
 
 	void ClearBookmarks();
-
+	
 protected:
 	bool CreateMenus();
 	bool CreateQuickconnectBar();
@@ -106,14 +108,12 @@ protected:
 	CRemoteListView* m_pRemoteListView;
 	CViewHeader* m_pLocalViewHeader;
 	CViewHeader* m_pRemoteViewHeader;
-	CLed* m_pRecvLed;
-	CLed* m_pSendLed;
+	CLed* m_pActivityLed[2];
 	wxTimer m_transferStatusTimer;
 	CThemeProvider* m_pThemeProvider;
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 	CUpdateWizard* m_pUpdateWizard;
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	CComparisonManager* m_pComparisonManager;
 
 	void ShowLocalTree();
 	void ShowRemoteTree();
@@ -162,7 +162,13 @@ protected:
 	void OnToolbarComparison(wxCommandEvent& event);
 	void OnToolbarComparisonDropdown(wxCommandEvent& event);
 	void OnDropdownComparisonMode(wxCommandEvent& event);
+	void OnDropdownComparisonHide(wxCommandEvent& event);
 	void OnSyncBrowse(wxCommandEvent& event);
+#ifndef __WXMAC__
+	void OnIconize(wxIconizeEvent& event);
+	void OnTaskBarClick(wxTaskBarIconEvent& event);
+#endif
+	void OnSearch(wxCommandEvent& event);
 
 	bool m_bInitDone;
 	bool m_bQuit;
@@ -176,6 +182,10 @@ protected:
 	CWindowStateManager* m_pWindowStateManager;
 
 	CQueue* m_pQueuePane;
+
+#ifndef __WXMAC__
+	wxTaskBarIcon* m_taskBarIcon;
+#endif
 };
 
 #endif
