@@ -189,6 +189,7 @@ EVT_MENU(XRCID("ID_PRIORITY_LOWEST"), CQueueView::OnSetPriority)
 
 EVT_COMMAND(wxID_ANY, fzEVT_GRANTEXCLUSIVEENGINEACCESS, CQueueView::OnExclusiveEngineRequestGranted)
 
+EVT_SIZE(CQueueView::OnSize)
 END_EVENT_TABLE()
 
 class CFolderProcessingThread : public wxThread
@@ -306,7 +307,7 @@ protected:
 				m_condition.Wait();
 			}
 		}
-		else if (m_entryList.size() < 10)
+		else if (m_entryList.size() < 20)
 			send = false;
 		else
 			send = true;
@@ -476,6 +477,15 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	SettingsChanged();
 
 	SetDropTarget(new CQueueViewDropTarget(this));
+
+	m_folderscan_item_refresh_timer.SetOwner(this);
+
+	m_line_height = -1;
+#ifdef __WXMSW__
+	m_header_height = -1;
+#endif
+
+	m_resize_timer.SetOwner(this);
 }
 
 CQueueView::~CQueueView()
@@ -488,6 +498,8 @@ CQueueView::~CQueueView()
 	}
 
 	DeleteEngines();
+
+	m_resize_timer.Stop();
 }
 
 bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxString& localFile,
@@ -513,6 +525,8 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 		else
 			fileItem->m_transferSettings.binary = !CAutoAsciiFiles::TransferLocalAsAscii(localFile, remotePath.GetType());
 		fileItem->m_edit = edit;
+		if (edit != CEditHandler::none)
+			fileItem->m_onetime_action = CFileExistsNotification::overwrite;
 	}
 
 	InsertItem(pServerItem, fileItem);
@@ -522,6 +536,9 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 
 void CQueueView::QueueFile_Finish(const bool start)
 {
+	bool need_refresh = false;
+	if (m_insertionStart <= GetTopItem() + GetCountPerPage() + 1)
+		need_refresh = true;
 	CommitChanges();
 
 	if (!m_activeMode && start)
@@ -538,7 +555,9 @@ void CQueueView::QueueFile_Finish(const bool start)
 	}
 
 	UpdateStatusLinePositions();
-	RefreshListOnly(false);
+
+	if (need_refresh)
+		RefreshListOnly(false);
 }
 
 bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, const CRemoteDataObject& dataObject)
@@ -631,18 +650,30 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 		{
 		case reqId_fileexists:
 			{
-				CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
-				if (pFileExistsNotification->canResume &&
-					pEngineData->pItem->GetType() == QueueItemType_File &&
-					((CFileItem*)pEngineData->pItem)->m_autoResume &&
-					((CFileItem*)pEngineData->pItem)->m_transferSettings.binary)
+				CFileExistsNotification* pFileExistsNotification = (CFileExistsNotification *)pNotification;
+				pFileExistsNotification->overwriteAction = pEngineData->pItem->m_defaultFileExistsAction;
+
+				if (pEngineData->pItem->GetType() == QueueItemType_File)
 				{
-					pFileExistsNotification->overwriteAction = CFileExistsNotification::resume;
-				}
-				else
-				{
-					// Set default file exists action
-					pFileExistsNotification->overwriteAction = pEngineData->pItem->m_defaultFileExistsAction;
+					CFileItem* pFileItem = (CFileItem*)pEngineData->pItem;
+
+					switch (pFileItem->m_onetime_action)
+					{
+					case CFileExistsNotification::resume:
+						if (pFileExistsNotification->canResume &&
+							pFileItem->m_transferSettings.binary)
+						{
+							pFileExistsNotification->overwriteAction = CFileExistsNotification::resume;
+						}
+						break;
+					case CFileExistsNotification::overwrite:
+						pFileExistsNotification->overwriteAction = CFileExistsNotification::overwrite;
+						break;
+					default:
+						// Others are unused
+						break;
+					}
+					pFileItem->m_onetime_action = CFileExistsNotification::unknown;
 				}
 			}
 			break;
@@ -655,10 +686,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 	case nId_active:
 		{
 			CActiveNotification *pActiveNotification = reinterpret_cast<CActiveNotification *>(pNotification);
-			if (pActiveNotification->IsRecv())
-				m_pMainFrame->UpdateRecvLed();
-			else
-				m_pMainFrame->UpdateSendLed();
+			m_pMainFrame->UpdateActivityLed(pActiveNotification->GetDirection());
 			delete pNotification;
 		}
 		break;
@@ -876,11 +904,12 @@ bool CQueueView::TryStartNextTransfer()
 		int lineIndex = GetItemIndex(bestMatch.fileItem);
 		UpdateSelections_ItemAdded(lineIndex + 1);
 
-		wxRect rect;
-		GetItemRect(lineIndex + 1, rect);
-#ifndef __WXMSW__
-		rect.y -= GetMainWindow()->GetPosition().y;
+		wxRect rect = GetClientRect();
+		rect.y = GetLineHeight() * (lineIndex + 1 - GetTopItem());
+#ifdef __WXMSW__
+		rect.y += m_header_height;
 #endif
+		rect.SetHeight(GetLineHeight());
 		m_allowBackgroundErase = false;
 		if (!pEngineData->pStatusLineCtrl)
 			pEngineData->pStatusLineCtrl = new CStatusLineCtrl(this, pEngineData, rect);
@@ -928,7 +957,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 				{
 					CFileItem* pItem = (CFileItem*)pEngineData->pItem;
 					pItem->m_madeProgress = false;
-					pItem->m_autoResume = true;
+					pItem->m_onetime_action = CFileExistsNotification::resume;
 				}
 				reason = reset;
 			}
@@ -1014,7 +1043,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 			// Don't increase error count if there has been progress
 			CFileItem* pItem = (CFileItem*)pEngineData->pItem;
 			pItem->m_madeProgress = false;
-			pItem->m_autoResume = true;
+			pItem->m_onetime_action = CFileExistsNotification::resume;
 		}
 		else
 		{
@@ -1166,7 +1195,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 
 			if (reason == failure)
 			{
-				pFileItem->m_autoResume = false;
+				pFileItem->m_onetime_action = CFileExistsNotification::unknown;
 				pFileItem->m_madeProgress = false;
 			}
 		}
@@ -1448,8 +1477,13 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		if (pState)
 		{
 			CRecursiveOperation* pRecursiveOperation = pState->GetRecursiveOperationHandler();
-			if (pRecursiveOperation && pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
-				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+			if (pRecursiveOperation)
+			{
+				if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
+					pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+				if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
+					pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
+			}
 		}
 
 		UpdateStatusLinePositions();
@@ -1540,6 +1574,8 @@ bool CQueueView::Quit()
 
 	SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
 
+	m_resize_timer.Stop();
+
 	return true;
 }
 
@@ -1603,10 +1639,6 @@ void CQueueView::UpdateStatusLinePositions()
 	m_lastTopItem = GetTopItem();
 	int bottomItem = m_lastTopItem + GetCountPerPage();
 
-#ifndef __WXMSW__
-	int header_offset = GetMainWindow()->GetPosition().y;
-#endif
-
 	for (std::list<CStatusLineCtrl*>::iterator iter = m_statusLineList.begin(); iter != m_statusLineList.end(); iter++)
 	{
 		CStatusLineCtrl *pCtrl = *iter;
@@ -1617,15 +1649,13 @@ void CQueueView::UpdateStatusLinePositions()
 			continue;
 		}
 
-		wxRect rect;
-		if (!GetItemRect(index, rect))
-		{
-			pCtrl->Show(false);
-			continue;
-		}
-#ifndef __WXMSW__
-		rect.y -= header_offset;
+		wxRect rect = GetClientRect();
+		rect.y = GetLineHeight() * (index - m_lastTopItem);
+#ifdef __WXMSW__
+		rect.y += m_header_height;
 #endif
+		rect.SetHeight(GetLineHeight());
+
 		m_allowBackgroundErase = bottomItem + 1 >= m_itemCount;
 		pCtrl->SetSize(rect);
 		m_allowBackgroundErase = false;
@@ -1722,6 +1752,8 @@ void CQueueView::OnFolderThreadComplete(wxCommandEvent& event)
 	if (!m_pFolderProcessingThread)
 		return;
 
+	m_folderscan_item_refresh_timer.Stop();
+
 	wxASSERT(!m_queuedFolders[1].empty());
 	CFolderScanItem* pItem = m_queuedFolders[1].front();
 	if (pItem->m_dir_is_empty)
@@ -1812,6 +1844,10 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 
 void CQueueView::SaveQueue()
 {
+	// Kiosk mode doesn't save queue
+	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		return;
+
 	// We have to synchronize access to queue.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
@@ -1862,6 +1898,9 @@ void CQueueView::LoadQueue()
 
 	pDocument->RemoveChild(pQueue);
 
+	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		return;
+
 	wxString error;
 	if (!xml.Save(&error))
 	{
@@ -1892,6 +1931,7 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 				unsigned int errorCount = GetTextElementInt(pFile, "ErrorCount");
 				unsigned int priority = GetTextElementInt(pFile, "Priority", priority_normal);
 				bool binary = GetTextElementInt(pFile, "TransferMode", 1) != 0;
+				int overwrite_action = GetTextElementInt(pFile, "OverwriteAction", CFileExistsNotification::unknown);
 
 				CServerPath remotePath;
 				if (localFile != _T("") && remoteFile != _T("") && remotePath.SetSafePath(safeRemotePath) &&
@@ -1902,6 +1942,9 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 					fileItem->SetPriorityRaw((enum QueuePriority)priority);
 					fileItem->m_errorCount = errorCount;
 					InsertItem(pServerItem, fileItem);
+
+					if (overwrite_action > 0 && overwrite_action < CFileExistsNotification::ACTION_COUNT)
+						fileItem->m_defaultFileExistsAction = (CFileExistsNotification::OverwriteAction)overwrite_action;
 				}
 			}
 			for (TiXmlElement* pFolder = pServer->FirstChildElement("Folder"); pFolder; pFolder = pFolder->NextSiblingElement("Folder"))
@@ -2312,8 +2355,9 @@ void CQueueView::OnFolderThreadFiles(wxCommandEvent& event)
 	m_pFolderProcessingThread->CheckFinished();
 
 	pItem->m_count += added;
-	pItem->m_statusMessage = wxString::Format(_("%d files added to queue"), pItem->GetCount());
-	RefreshItem(pItem);
+
+	if (!m_folderscan_item_refresh_timer.IsRunning())
+		m_folderscan_item_refresh_timer.Start(200, true);
 }
 
 void CQueueView::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction)
@@ -2489,7 +2533,7 @@ void CQueueView::TryRefreshListings()
 	if (!pServer)
 		return;
 
-	const CDirectoryListing* const pListing = pState->GetRemoteDir();
+	const CDirectoryListing* const pListing = pState->GetRemoteDir().Value();
 	if (!pListing)
 		return;
 
@@ -2559,7 +2603,6 @@ void CQueueView::OnAskPassword(wxCommandEvent& event)
 			continue;
 		}
 
-		static std::list<void*> AskPasswordBusyList;
 		if (CLoginManager::Get().GetPassword(pEngineData->lastServer, false))
 		{
 			pEngineData->state = t_EngineData::connect;
@@ -2684,6 +2727,23 @@ void CQueueView::OnTimer(wxTimerEvent& event)
 	}
 #endif
 
+	if (id == m_resize_timer.GetId())
+	{
+		UpdateStatusLinePositions();
+		return;
+	}
+
+	if (id == m_folderscan_item_refresh_timer.GetId())
+	{
+		if (m_queuedFolders[1].empty())
+			return;
+
+		CFolderScanItem* pItem = m_queuedFolders[1].front();
+		pItem->m_statusMessage = wxString::Format(_("%d files added to queue"), pItem->GetCount());
+		RefreshItem(pItem);
+		return;
+	}
+
 	for (unsigned int i = 1; i < m_engineData.size(); i++)
 	{
 		t_EngineData* pData = m_engineData[i];
@@ -3067,3 +3127,63 @@ void CQueueView::OnChar(wxKeyEvent& event)
 	else
 		event.Skip();
 }
+
+int CQueueView::GetLineHeight()
+{
+	if (m_line_height != -1)
+		return m_line_height;
+
+	if (!GetItemCount())
+		return 20;
+
+	wxRect rect;
+	if (!GetItemRect(0, rect))
+		return 20;
+
+	m_line_height = rect.GetHeight();
+
+#ifdef __WXMSW__
+	m_header_height = rect.y;
+#endif
+
+	return m_line_height;
+}
+
+void CQueueView::OnSize(wxSizeEvent& event)
+{
+	if (!m_resize_timer.IsRunning())
+		m_resize_timer.Start(250);
+
+	event.Skip();
+}
+
+void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString& newName, bool local)
+{
+	std::vector<t_EngineData*>::iterator iter;
+	for (iter = m_engineData.begin(); iter != m_engineData.end(); iter++)
+	{
+		if ((*iter)->pEngine == pEngine)
+			break;
+	}
+	if (iter == m_engineData.end())
+		return;
+
+	t_EngineData* const pEngineData = *iter;
+	if (!pEngineData->pItem)
+		return;
+	
+	if (pEngineData->pItem->GetType() != QueueItemType_File)
+		return;
+
+	CFileItem* pFile = (CFileItem*)pEngineData->pItem;
+	if (local)
+	{
+		wxFileName fn(pFile->GetLocalFile());
+		fn.SetFullName(newName);
+		pFile->SetLocalFile(fn.GetFullPath());
+	}
+	else
+		pFile->SetRemoteFile(newName);
+
+	RefreshItem(pFile);
+}
