@@ -15,6 +15,7 @@
 #include "recursive_operation.h"
 #include "edithandler.h"
 #include "dragdropmanager.h"
+#include <wx/clipbrd.h>
 
 #ifdef __WXMSW__
 #include "shellapi.h"
@@ -335,6 +336,7 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_LIST_BEGIN_DRAG(wxID_ANY, CRemoteListView::OnBeginDrag)
 	EVT_MENU(XRCID("ID_EDIT"), CRemoteListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CRemoteListView::OnMenuEnter)
+	EVT_MENU(XRCID("ID_GETURL"), CRemoteListView::OnMenuGeturl)
 END_EVENT_TABLE()
 
 CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
@@ -345,6 +347,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
 
 	m_dropTarget = -1;
 
@@ -430,7 +433,7 @@ bool CRemoteListView::IsItemValid(unsigned int item) const
 	return true;
 }
 
-void CRemoteListView::UpdateDirectoryListing_Added(const CDirectoryListing *pDirectoryListing)
+void CRemoteListView::UpdateDirectoryListing_Added(const CSharedPointer<const CDirectoryListing> &pDirectoryListing)
 {
 	const unsigned int to_add = pDirectoryListing->GetCount() - m_pDirectoryListing->GetCount();
 	m_pDirectoryListing = pDirectoryListing;
@@ -515,7 +518,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(const CDirectoryListing *pDir
 	wxASSERT(m_indexMapping.size() == pDirectoryListing->GetCount() + 1);
 }
 
-void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pDirectoryListing)
+void CRemoteListView::UpdateDirectoryListing_Removed(const CSharedPointer<const CDirectoryListing> &pDirectoryListing)
 {
 	const unsigned int removed = m_pDirectoryListing->GetCount() - pDirectoryListing->GetCount();
 	if (!removed)
@@ -650,7 +653,7 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 	SaveSetItemCount(m_indexMapping.size());
 }
 
-bool CRemoteListView::UpdateDirectoryListing(const CDirectoryListing *pDirectoryListing)
+bool CRemoteListView::UpdateDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing)
 {
 	wxASSERT(!IsComparing());
 
@@ -697,10 +700,11 @@ bool CRemoteListView::UpdateDirectoryListing(const CDirectoryListing *pDirectory
 	return true;
 }
 
-void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryListing, bool modified /*=false*/)
+void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing, bool modified /*=false*/)
 {
 #ifdef __WXMSW__
-	EndEditLabel(true);
+	if (GetEditControl())
+		ListView_CancelEditLabel((HWND)GetHandle());
 #endif
 
 	bool reset = false;
@@ -955,16 +959,16 @@ public:
 
 	inline int CmpTime(const CDirentry &data1, const CDirentry &data2) const
 	{
-		if (!data1.hasDate)
+		if (data1.hasTimestamp == CDirentry::timestamp_none)
 		{
-			if (data2.hasDate)
+			if (data2.hasTimestamp != CDirentry::timestamp_none)
 				return -1;
 			else
 				return 0;
 		}
 		else
 		{
-			if (!data2.hasDate)
+			if (data2.hasTimestamp == CDirentry::timestamp_none)
 				return 1;
 
 			if (data1.time < data2.time)
@@ -1214,8 +1218,6 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 			if (!action)
 			{
-				if (IsComparing())
-					ExitComparisonMode();
 				if (entry.link)
 				{
 					delete m_pLinkResolveState;
@@ -1266,9 +1268,6 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 	}
 	else
 	{
-		if (IsComparing())
-			ExitComparisonMode();
-
 		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
@@ -1319,8 +1318,6 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &event)
 			return;
 		}
 
-		if (IsComparing())
-			ExitComparisonMode();
 		if (entry.link)
 		{
 			delete m_pLinkResolveState;
@@ -1334,9 +1331,6 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &event)
 	}
 	else
 	{
-		if (IsComparing())
-			ExitComparisonMode();
-
 		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
@@ -1363,6 +1357,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
+		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
 	else if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED)))
 	{
@@ -1372,6 +1367,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
+		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
 	else if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
 	{
@@ -1382,13 +1378,14 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
+		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
 	else
 	{
 		int count = 0;
 		int fillCount = 0;
 		bool selectedDir = false;
-		int item = -1;;
+		int item = -1;
 		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 		{
 			int index = GetItemIndex(item);
@@ -1412,6 +1409,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
+			pMenu->Enable(XRCID("ID_GETURL"), false);
 		}
 		else
 		{
@@ -1421,7 +1419,8 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 				pMenu->Delete(XRCID("ID_ENTER"));
 			if (count > 1)
 			{
-				pMenu->Delete(XRCID("ID_ENTER"));
+				if (selectedDir)
+					pMenu->Delete(XRCID("ID_ENTER"));
 				pMenu->Enable(XRCID("ID_RENAME"), false);
 			}
 
@@ -1537,7 +1536,8 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 	{
 		if (IsComparing())
 			ExitComparisonMode();
-		pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, m_pDirectoryListing->path);
+		CFilterManager filter;
+		pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, m_pDirectoryListing->path, filter.GetActiveFilters(false));
 	}
 }
 
@@ -1600,6 +1600,10 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 		return;
 	}
 
+	int count_dirs = 0;
+	int count_files = 0;
+	bool selected_link = false;
+
 	long item = -1;
 	while (true)
 	{
@@ -1612,11 +1616,54 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 			wxBell();
 			return;
 		}
+
+		int index = GetItemIndex(item);
+		if (index == -1)
+			continue;
+		if (m_fileData[index].flags == fill)
+			continue;
+
+		const CDirentry& entry = (*m_pDirectoryListing)[index];
+		if (entry.dir)
+		{
+			count_dirs++;
+			if (entry.link)
+				selected_link = true;
+		}
+		else
+			count_files++;
 	}
 
-	if (wxMessageBox(_("Really delete all selected files and/or directories?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+	wxString question;
+	if (!count_dirs)
+		question.Printf(wxPLURAL("Really delete %d file?", "Really delete %d files?", count_files), count_files);
+	else if (!count_files)
+		question.Printf(wxPLURAL("Really delete %d directory with its contents?", "Really delete %d directories with their contents?", count_dirs), count_dirs);
+	else
+	{
+		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
+		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
+		question.Printf(_("Really delete %s and %s?"), files.c_str(), dirs.c_str());
+	}
+
+	if (wxMessageBox(question, _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
 		return;
 
+	bool follow_symlink = false;
+	if (selected_link)
+	{
+		wxDialogEx dlg;
+		if (!dlg.Load(this, _T("ID_DELETE_SYMLINK")))
+		{
+			wxBell();
+			return;
+		}
+		if (dlg.ShowModal() != wxID_OK)
+			return;
+
+		follow_symlink = XRCCTRL(dlg, "ID_RECURSE", wxRadioButton)->GetValue();
+	}
+
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
 	wxASSERT(pRecursiveOperation);
 
@@ -1639,12 +1686,12 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		if (entry.dir)
+		if (entry.dir && (follow_symlink || !entry.link))
 		{
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name))
 			{
-				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name);
+				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, _T(""), true);
 				startRecursive = true;
 			}
 		}
@@ -1659,7 +1706,8 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 	{
 		if (IsComparing())
 			ExitComparisonMode();
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, m_pDirectoryListing->path);
+		CFilterManager filter;
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, m_pDirectoryListing->path, filter.GetActiveFilters(false));
 	}
 }
 
@@ -1728,9 +1776,6 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 			return;
 		}
 
-		if (IsComparing())
-			ExitComparisonMode();
-
 		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 	else
@@ -1867,7 +1912,6 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 	wxString name;
 
 	char permissions[9] = {0};
-	const unsigned char permchars[3] = {'r', 'w', 'x'};
 
 	long item = -1;
 	while (true)
@@ -1893,21 +1937,16 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 			fileCount++;
 		name = entry.name;
 
-		if (entry.permissions.Length() == 10)
+		char file_perms[9];
+		if (CChmodDialog::ConvertPermissions(entry.permissions, file_perms))
 		{
 			for (int i = 0; i < 9; i++)
 			{
-				bool set = entry.permissions[i + 1] == permchars[i % 3];
-				if (!permissions[i] || permissions[i] == (set ? 2 : 1))
-					permissions[i] = set ? 2 : 1;
+				if (!permissions[i] || permissions[i] == file_perms[i])
+					permissions[i] = file_perms[i];
 				else
 					permissions[i] = -1;
 			}
-			if (entry.permissions[6] == 's')
-				permissions[5] = 2;
-			if (entry.permissions[9] == 't')
-				permissions[8] = 2;
-
 		}
 	}
 	if (!dirCount && !fileCount)
@@ -1981,7 +2020,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 		{
 			char permissions[9];
 			bool res = pChmodDlg->ConvertPermissions(entry.permissions, permissions);
-			wxString newPerms = pChmodDlg->GetPermissions(res ? permissions : 0);
+			wxString newPerms = pChmodDlg->GetPermissions(res ? permissions : 0, entry.dir);
 
 			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
 		}
@@ -1996,7 +2035,8 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 			ExitComparisonMode();
 
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, m_pDirectoryListing->path);
+		CFilterManager filter;
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, m_pDirectoryListing->path, filter.GetActiveFilters(false));
 
 		// Refresh listing. This gets done implicitely by the recursive operation, so
 		// only it if not recursing.
@@ -2006,7 +2046,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 	else
 	{
 		pChmodDlg->Destroy();
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path);
+		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(""), 0, true);
 	}
 
 }
@@ -2260,13 +2300,18 @@ void CRemoteListView::RepositionInfoText()
 
 }
 
-void CRemoteListView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CRemoteListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
-	wxASSERT(m_pState);
+	wxASSERT(pState);
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(m_pState->GetRemoteDir(), false);
+		SetDirectoryListing(pState->GetRemoteDir(), false);
 	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(m_pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), true);
+	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR)
+	{
+		wxASSERT(data2);
+		LinkIsNotDir(*(CServerPath*)data2, data);
+	}
 	else
 	{
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -2496,33 +2541,6 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 #endif
 }
 
-bool CRemoteListView::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, const CLocalPath& path, bool queueOnly)
-{
-	if (!m_pState->IsRemoteIdle())
-		return false;
-
-	wxASSERT(!path.empty());
-
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	wxASSERT(pRecursiveOperation);
-
-	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
-	{
-		if (!iter->dir)
-			continue;
-
-		pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + iter->name, iter->link);
-	}
-
-	if (IsComparing())
-		ExitComparisonMode();
-
-	pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, pRemoteDataObject->GetServerPath());
-
-	return true;
-}
-
 void CRemoteListView::InitDateFormat()
 {
 	const wxString& dateFormat = COptions::Get()->GetOption(OPTION_DATE_FORMAT);
@@ -2551,31 +2569,29 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 		return;
 	}
 
-	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	if (item < 1)
+	long item = -1;
+	
+	std::list<CDirentry> selected_item_list;
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		wxBell();
-		return;
-	}
+		if (!item)
+		{
+			wxBell();
+			return;
+		}
 
-	if (GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1)
-	{
-		wxBell();
-		return;
-	}
+		int index = GetItemIndex(item);
+		if (index == -1 || m_fileData[index].flags == fill)
+			continue;
 
-	int index = GetItemIndex(item);
-	if (index == -1 || m_fileData[index].flags == fill)
-	{
-		wxBell();
-		return;
-	}
+		const CDirentry& entry = (*m_pDirectoryListing)[index];
+		if (entry.dir)
+		{
+			wxBell();
+			return;
+		}
 
-	const CDirentry entry = (*m_pDirectoryListing)[index];
-	if (entry.dir)
-	{
-		wxBell();
-		return;
+		selected_item_list.push_back(entry);
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
@@ -2585,6 +2601,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 		return;
 	}
 
+
 	const wxString& localDir = pEditHandler->GetLocalDirectory();
 	if (localDir == _T(""))
 	{
@@ -2595,97 +2612,119 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 	const CServerPath path = m_pDirectoryListing->path;
 	const CServer server = *m_pState->GetServer();
 
-	bool dangerous = false;
-	bool program_exists = false;
-	wxString cmd = pEditHandler->CanOpen(CEditHandler::remote, entry.name, dangerous, program_exists);
-	if (cmd.empty())
+	if (selected_item_list.empty())
 	{
-		CNewAssociationDialog dlg(this);
-		if (!dlg.Show(entry.name))
-			return;
-		cmd = pEditHandler->CanOpen(CEditHandler::remote, entry.name, dangerous, program_exists);
-		if (cmd.empty())
-		{
-			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), entry.name.c_str()), _("Opening failed"), wxICON_EXCLAMATION);
-			return;
-		}
-	}
-	if (!program_exists)
-	{
-		wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), entry.name.c_str(), cmd.c_str());
-		wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
-		return;
-	}
-	if (dangerous)
-	{
-		int res = wxMessageBox(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
-		if (res != wxYES)
-		{
-			wxBell();
-			return;
-		}
-	}
-
-	CEditHandler::fileState state = pEditHandler->GetFileState(entry.name, path, server);
-	switch (state)
-	{
-	case CEditHandler::download:
-	case CEditHandler::upload:
-	case CEditHandler::upload_and_remove:
-	case CEditHandler::upload_and_remove_failed:
-		wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
-		return;
-	case CEditHandler::removing:
-		if (!pEditHandler)
-		{
-			wxMessageBox(_("A file with that name is still being edited. Please close it and try again."), _("Selected file already opened."), wxICON_EXCLAMATION);
-			return;
-		}
-		break;
-	case CEditHandler::edit:
-		{
-			wxDialogEx dlg;
-			if (!dlg.Load(this, _T("ID_EDITEXISTING")))
-			{
-				wxBell();
-				return;
-			}
-			dlg.SetLabel(XRCID("ID_FILENAME"), entry.name);
-			if (dlg.ShowModal() != wxID_OK)
-			{
-				wxBell();
-				return;
-			}
-
-			if (XRCCTRL(dlg, "ID_REOPEN", wxRadioButton)->GetValue())
-			{
-				pEditHandler->StartEditing(entry.name, path, server);
-				return;
-			}
-			else
-			{
-				if (!pEditHandler->Remove(entry.name, path, server))
-				{
-					wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file still being edited"), wxICON_EXCLAMATION);
-					return;
-				}
-			}
-		}
-		break;
-	default:
-		break;
-	}
-
-	wxString file = entry.name;
-	if (!pEditHandler->AddFile(CEditHandler::remote, file, path, server))
-	{
-		wxFAIL;
 		wxBell();
 		return;
 	}
 
-	m_pQueue->QueueFile(false, true, file, entry.name, path, server, entry.size, CEditHandler::remote);
-	m_pQueue->QueueFile_Finish(true);
+	if (selected_item_list.size() > 10)
+	{
+
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return;
+	}
+
+	for (std::list<CDirentry>::const_iterator iter = selected_item_list.begin(); iter != selected_item_list.end(); iter++)
+	{
+		const CDirentry& entry = *iter;
+
+		bool dangerous = false;
+		bool program_exists = false;
+		wxString cmd = pEditHandler->CanOpen(CEditHandler::remote, entry.name, dangerous, program_exists);
+		if (cmd.empty())
+		{
+			CNewAssociationDialog dlg(this);
+			if (!dlg.Show(entry.name))
+				continue;
+			cmd = pEditHandler->CanOpen(CEditHandler::remote, entry.name, dangerous, program_exists);
+			if (cmd.empty())
+			{
+				wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), entry.name.c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+				continue;
+			}
+		}
+		if (!program_exists)
+		{
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), entry.name.c_str(), cmd.c_str());
+			wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			continue;
+		}
+		if (dangerous)
+		{
+			int res = wxMessageBox(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
+			if (res != wxYES)
+			{
+				wxBell();
+				continue;
+			}
+		}
+
+		CEditHandler::fileState state = pEditHandler->GetFileState(entry.name, path, server);
+		switch (state)
+		{
+		case CEditHandler::download:
+		case CEditHandler::upload:
+		case CEditHandler::upload_and_remove:
+		case CEditHandler::upload_and_remove_failed:
+			wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+			continue;
+		case CEditHandler::removing:
+			if (!pEditHandler)
+			{
+				wxMessageBox(_("A file with that name is still being edited. Please close it and try again."), _("Selected file already opened."), wxICON_EXCLAMATION);
+				continue;
+			}
+			break;
+		case CEditHandler::edit:
+			{
+				wxDialogEx dlg;
+				if (!dlg.Load(this, _T("ID_EDITEXISTING")))
+				{
+					wxBell();
+					continue;
+				}
+				dlg.SetLabel(XRCID("ID_FILENAME"), entry.name);
+				if (dlg.ShowModal() != wxID_OK)
+				{
+					wxBell();
+					continue;
+				}
+
+				if (XRCCTRL(dlg, "ID_REOPEN", wxRadioButton)->GetValue())
+				{
+					pEditHandler->StartEditing(entry.name, path, server);
+					continue;
+				}
+				else
+				{
+					if (!pEditHandler->Remove(entry.name, path, server))
+					{
+						wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file still being edited"), wxICON_EXCLAMATION);
+						continue;
+					}
+				}
+			}
+			break;
+		default:
+			break;
+		}
+
+		wxString file = entry.name;
+		if (!pEditHandler->AddFile(CEditHandler::remote, file, path, server))
+		{
+			wxFAIL;
+			wxBell();
+			continue;
+		}
+
+		m_pQueue->QueueFile(false, true, file, entry.name, path, server, entry.size, CEditHandler::remote);
+		m_pQueue->QueueFile_Finish(true);
+	}
 }
 
 #ifdef __WXDEBUG__
@@ -2788,10 +2827,10 @@ bool CRemoteListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, w
 	name = entry.name;
 	dir = entry.dir;
 	size = entry.size;
-	if (entry.hasDate)
+	if (entry.hasTimestamp != CDirentry::timestamp_none)
 	{
 		date = entry.time;
-		hasTime = entry.hasTime;
+		hasTime = entry.hasTimestamp >= CDirentry::timestamp_time;
 	}
 	else
 	{
@@ -2917,10 +2956,10 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 	else if (column == 3)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (!entry.hasDate)
+		if (entry.hasTimestamp == CDirentry::timestamp_none)
 			return _T("");
 
-		if (entry.hasTime)
+		if (entry.hasTimestamp >= CDirentry::timestamp_time)
 			return entry.time.Format(m_timeFormat);
 		else
 			return entry.time.Format(m_dateFormat);
@@ -2936,35 +2975,36 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 {
 	CRemoteListViewSort::DirSortMode dirSortMode = GetDirSortMode();
 
+	const CDirectoryListing* pDirectoryListing = m_pDirectoryListing.Value();
 	if (!m_sortDirection)
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortSize(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortSize(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortType(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortType(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup(pDirectoryListing, m_fileData, dirSortMode, this));
 		else
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName(pDirectoryListing, m_fileData, dirSortMode, this));
 	}
 	else
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortSize_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortSize_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortType_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortType_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 		else
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName_Reverse(pDirectoryListing, m_fileData, dirSortMode, this));
 	}
 }
 
@@ -2999,3 +3039,79 @@ void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link
 	delete m_pLinkResolveState;
 	m_pLinkResolveState = 0;
 }
+
+void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
+{
+	if (!m_pDirectoryListing)
+		return;
+
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer)
+		return;
+
+	long item = -1;
+	
+	std::list<CDirentry> selected_item_list;
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+	{
+		if (!item)
+		{
+			wxBell();
+			return;
+		}
+
+		int index = GetItemIndex(item);
+		if (index == -1 || m_fileData[index].flags == fill)
+			continue;
+
+		const CDirentry& entry = (*m_pDirectoryListing)[index];
+
+		selected_item_list.push_back(entry);
+	}
+	if (selected_item_list.empty())
+	{
+		wxBell();
+		return;
+	}
+
+	if (!wxTheClipboard->Open())
+	{
+		wxMessageBox(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	const CServerPath& path = m_pDirectoryListing->path;
+	const wxString server = pServer->FormatServer(true);
+	if (selected_item_list.size() == 1)
+	{
+		wxString url = server;
+		url += path.FormatFilename(selected_item_list.front().name, false);
+
+		// Poor mans URLencode
+		url.Replace(_T(" "), _T("%20"));
+
+		wxTheClipboard->SetData(new wxURLDataObject(url));
+	}
+	else
+	{
+		wxString urls;
+		for (std::list<CDirentry>::const_iterator iter = selected_item_list.begin(); iter != selected_item_list.end(); iter++)
+		{
+			urls += server;
+			urls += path.FormatFilename(iter->name, false);
+#ifdef __WXMSW__
+			urls += _T("\r\n");
+#else
+			urls += _T("\n");
+#endif
+		}
+
+		// Poor mans URLencode
+		urls.Replace(_T(" "), _T("%20"));
+
+		wxTheClipboard->SetData(new wxTextDataObject(urls));
+	}
+
+	wxTheClipboard->Flush();
+	wxTheClipboard->Close();
+}
