@@ -7,6 +7,7 @@
 #include "recursive_operation.h"
 #include "inputdialog.h"
 #include "dragdropmanager.h"
+#include <wx/clipbrd.h>
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -227,6 +228,7 @@ EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnEndLabelEdit)
 EVT_MENU(XRCID("ID_MKDIR"), CRemoteTreeView::OnMkdir)
 EVT_CHAR(CRemoteTreeView::OnChar)
+EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
@@ -259,20 +261,20 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(m_pState->GetRemoteDir(), false);
+		SetDirectoryListing(pState->GetRemoteDir(), false);
 	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(m_pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), true);
+	else if (notification == STATECHANGE_APPLYFILTER)
+		ApplyFilters();
 }
 
-void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, bool modified)
+void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pListing, bool modified)
 {
 	m_busy = true;
 
-	m_pDirectoryListing = pListing;
-
 	if (!pListing)
 	{
 		m_ExpandAfterList = wxTreeItemId();
@@ -489,6 +491,9 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 	if (!listing.m_hasDirs)
 		return false;
 
+	if (!filter.HasActiveFilters())
+		return true;
+
 	const wxString path = listing.path.GetPath();
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
@@ -926,7 +931,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	if (!pMenu)
 		return;
 
-	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
+	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_pState->IsRemoteIdle() || path.IsEmpty())
 	{
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
@@ -936,6 +941,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
 		pMenu->Enable(XRCID("ID_RENAME"), false);
+		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
 	else if (!path.HasParent())
 		pMenu->Enable(XRCID("ID_RENAME"), false);
@@ -1022,7 +1028,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		// Change directory permissions
 		if (!applyType || applyType == 2)
 		{
-			wxString newPerms = pChmodDlg->GetPermissions(permissions);
+			wxString newPerms = pChmodDlg->GetPermissions(permissions, true);
 
 			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
 		}
@@ -1047,7 +1053,8 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		const wxTreeItemId selected = GetSelection();
 		if (selected)
 			currentPath = GetPathFromItem(selected);
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, hasParent ? path.GetParent() : path, !cached, currentPath);
+		CFilterManager filter;
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, hasParent ? path.GetParent() : path, filter.GetActiveFilters(false), !cached, currentPath);
 	}
 	else
 	{
@@ -1093,7 +1100,8 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 		currentPath = GetPathFromItem(selected);
 
 	const bool addOnly = event.GetId() == XRCID("ID_ADDTOQUEUE");
-	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, path, true, currentPath);
+	CFilterManager filter;
+	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, path, filter.GetActiveFilters(false), true, currentPath);
 }
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
@@ -1135,7 +1143,8 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 	if (!currentPath.IsEmpty() && (path == currentPath || path.IsParentOf(currentPath, false)))
 		currentPath = startDir;
 
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, !hasParent, currentPath);
+	CFilterManager filter;
+	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, filter.GetActiveFilters(false), !hasParent, currentPath);
 }
 
 void CRemoteTreeView::OnMenuRename(wxCommandEvent& event)
@@ -1338,3 +1347,116 @@ void CRemoteTreeView::OnChar(wxKeyEvent& event)
 	else
 		event.Skip();
 }
+
+struct _parents
+{
+	wxTreeItemId item;
+	CServerPath path;
+};
+
+void CRemoteTreeView::ApplyFilters()
+{
+	std::list<struct _parents> parents;
+
+	const wxTreeItemId root = GetRootItem();
+	wxTreeItemIdValue cookie;
+	for (wxTreeItemId child = GetFirstChild(root, cookie); child; child = GetNextSibling(child))
+	{
+		CServerPath path = GetPathFromItem(child);
+		if (path.IsEmpty())
+			continue;
+
+		struct _parents dir;
+		dir.item = child;
+		dir.path = path;
+		parents.push_back(dir);
+	}
+	
+	CFilterManager filter;
+	while (!parents.empty())
+	{
+		struct _parents parent = parents.back();
+		parents.pop_back();
+
+		CDirectoryListing listing;
+		if (m_pState->m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
+			RefreshItem(parent.item, listing, false);
+		else if (filter.HasActiveFilters())
+		{
+			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
+			{
+				CServerPath path = GetPathFromItem(child);
+				if (path.IsEmpty())
+					continue;
+
+				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0))
+				{
+					wxTreeItemId sel = GetSelection();
+					while (sel && sel != child)
+						sel = GetItemParent(sel);
+					if (!sel)
+					{
+						Delete(child);
+						continue;
+					}
+				}
+				
+				struct _parents dir;
+				dir.item = child;
+				dir.path = path;
+				parents.push_back(dir);
+			}
+
+			// The stuff below has already been done above in this one case
+			continue;
+		}
+		for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
+		{
+			CServerPath path = GetPathFromItem(child);
+			if (path.IsEmpty())
+				continue;
+
+			struct _parents dir;
+			dir.item = child;
+			dir.path = path;
+			parents.push_back(dir);
+		}
+	}
+}
+
+void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
+{
+	if (!m_contextMenuItem)
+		return;
+
+	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
+	if (path.IsEmpty())
+	{
+		wxBell();
+		return;
+	}
+
+	const CServer *pServer = m_pState->GetServer();
+	if (!pServer)
+	{
+		wxBell();
+		return;
+	}
+
+	if (!wxTheClipboard->Open())
+	{
+		wxMessageBox(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	wxString url = pServer->FormatServer(true);
+	url += path.GetPath();
+
+	// Poor mans URLencode
+	url.Replace(_T(" "), _T("%20"));
+
+	wxTheClipboard->SetData(new wxURLDataObject(url));
+
+	wxTheClipboard->Flush();
+	wxTheClipboard->Close();
+}
