@@ -12,7 +12,7 @@ public:
 	bool Create(wxWindow* parent, int fileCount, int dirCount, 
 				const wxString& name, const char permissions[9]);
 
-	wxString GetPermissions(const char* previousPermissions);
+	wxString GetPermissions(const char* previousPermissions, bool dir);
 
 	bool Recursive() const ;
 	int GetApplyType() const { return m_applyType; }
@@ -22,7 +22,9 @@ public:
 	// 9 characters.
 	// Example:
 	//   drwxr--r-- gets converted into 222211211
-	bool ConvertPermissions(const wxString rwx, char* permissions);
+	//   0644 gets converted into 221211211
+	//   foo (0273) gets converted into 121222122
+	static bool ConvertPermissions(wxString rwx, char* permissions);
 
 protected:
 
