@@ -33,16 +33,12 @@ void CChangedFileDialog::OnNo(wxCommandEvent& event)
 
 //-------------
 
-#ifdef __WXMAC__
 DECLARE_EVENT_TYPE(fzEDIT_CHANGEDFILE, -1)
 DEFINE_EVENT_TYPE(fzEDIT_CHANGEDFILE)
-#endif
 
 BEGIN_EVENT_TABLE(CEditHandler, wxEvtHandler)
 EVT_TIMER(wxID_ANY, CEditHandler::OnTimerEvent)
-#ifdef __WXMAC__
 EVT_COMMAND(wxID_ANY, fzEDIT_CHANGEDFILE, CEditHandler::OnChangedFileEvent)
-#endif
 END_EVENT_TABLE()
 
 CEditHandler* CEditHandler::m_pEditHandler = 0;
@@ -302,6 +298,9 @@ bool CEditHandler::AddFile(enum CEditHandler::fileType type, wxString& fileName,
 	data.remotePath = remotePath;
 	data.server = server;
 
+	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
+		return StartEditing(local, data);
+
 	if (type == remote || StartEditing(type, data))
 		m_fileDataList[type].push_back(data);
 
@@ -607,24 +606,18 @@ bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& da
 	return true;
 }
 
-void CEditHandler::CheckForModifications(
-#ifdef __WXMAC__
-		bool emitEvent /*=false*/
-#endif
-	)
+void CEditHandler::CheckForModifications(bool emitEvent)
 {
 	static bool insideCheckForModifications = false;
 	if (insideCheckForModifications)
 		return;
 
-#ifdef __WXMAC__
 	if (emitEvent)
 	{
 		wxCommandEvent evt(fzEDIT_CHANGEDFILE);
 		AddPendingEvent(evt);
 		return;
 	}
-#endif
 
 	insideCheckForModifications = true;
 
@@ -643,7 +636,13 @@ checkmodifications_remote:
 			goto checkmodifications_remote;
 		}
 
-		wxDateTime mtime = fn.GetModificationTime();
+		wxDateTime mtime;
+
+		{
+			wxLogNull log; // If GetModificationTime fails wx spams error messages
+			mtime = fn.GetModificationTime();
+		}
+
 		if (!mtime.IsValid())
 			continue;
 
@@ -704,7 +703,13 @@ checkmodifications_local:
 			goto checkmodifications_local;
 		}
 
-		wxDateTime mtime = fn.GetModificationTime();
+		wxDateTime mtime;
+
+		{
+			wxLogNull log; // If GetModificationTime fails wx spams error messages
+			mtime = fn.GetModificationTime();
+		}
+
 		if (!mtime.IsValid())
 			continue;
 
@@ -936,11 +941,20 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	return command + _T(" \"") + file + _T("\"");
 }
 
-#ifdef __WXGTK__
+
 static bool PathExpand(wxString& cmd)
 {
+#ifndef __WXMSW__
 	if (cmd[0] == '/')
 		return true;
+#else
+	if (cmd[0] == '\\')
+		// UNC or root of current working dir, whatever that is
+		return true;
+	if (cmd.Len() > 2 && cmd[1] == ':')
+		// Absolute path
+		return true;
+#endif
 
 	// Need to search for program in $PATH
 	wxString path;
@@ -948,13 +962,21 @@ static bool PathExpand(wxString& cmd)
 		return false;
 
 	wxString full_cmd;
-	if (!wxFindFileInPath(&full_cmd, path, cmd))
+	bool found = wxFindFileInPath(&full_cmd, path, cmd);
+#ifdef __WXMSW__
+	if (!found && cmd.Right(4).Lower() != _T(".exe"))
+	{
+		cmd += _T(".exe");
+		found = wxFindFileInPath(&full_cmd, path, cmd);
+	}
+#endif
+
+	if (!found)
 		return false;
 
 	cmd = full_cmd;
 	return true;
 }
-#endif
 
 wxString CEditHandler::GetSystemOpenCommand(const wxString& file, bool &program_exists)
 {
@@ -999,10 +1021,9 @@ wxString CEditHandler::GetSystemOpenCommand(const wxString& file, bool &program_
 	if (!UnquoteCommand(editor, args) || editor.empty())
 		return cmd;
 
-#ifdef __WXGTK__
 	if (!PathExpand(editor))
 		return cmd;
-#endif
+
 	if (ProgramExists(editor))
 		program_exists = true;
 	return cmd;
@@ -1061,12 +1082,10 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 	return _T("");
 }
 
-#ifdef __WXMAC__
 void CEditHandler::OnChangedFileEvent(wxCommandEvent& event)
 {
 	CheckForModifications();
 }
-#endif
 
 wxString CEditHandler::GetTemporaryFile(wxString name)
 {
@@ -1144,7 +1163,11 @@ bool CEditHandler::FilenameExists(const wxString& file)
 {
 	for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); iter++)
 	{
+#ifdef __WXMSW__
+		if (!iter->file.CmpNoCase(file))
+#else
 		if (iter->file == file)
+#endif
 			return true;
 	}
 
@@ -1352,9 +1375,9 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 		
 		if (type == CEditHandler::local)
 		{
+			pEditHandler->Remove(pData->file);
 			delete pData;
 			pListCtrl->DeleteItem(i);
-			pEditHandler->Remove(pData->file);
 		}
 		else
 		{
@@ -1491,7 +1514,12 @@ CEditHandler::t_fileData* CEditHandlerStatusDialog::GetDataFromItem(int item, CE
 	CEditHandler::t_fileData* pData = (CEditHandler::t_fileData*)pListCtrl->GetItemData(item);
 	wxASSERT(pData);
 
-	if (pListCtrl->GetItemText(item) == _("Local"))
+	wxListItem info;
+	info.SetMask(wxLIST_MASK_TEXT);
+	info.SetId(item);
+	info.SetColumn(1);
+	pListCtrl->GetItem(info);
+	if (info.GetText() == _("Local"))
 		type = CEditHandler::local;
 	else
 		type = CEditHandler::remote;
@@ -1538,10 +1566,9 @@ bool CNewAssociationDialog::Show(const wxString &file)
 		if (!UnquoteCommand(cmd, args))
 			cmd.clear();
 	}
-#ifdef __WXGTK__
+
 	if (!PathExpand(cmd))
 		cmd.clear();
-#endif
 
 	pDesc = XRCCTRL(*this, "ID_EDITOR_DESC", wxStaticText);
 	if (cmd.empty())
