@@ -38,7 +38,7 @@ public:
 template<class CFileData> class CFileListCtrl : public wxListCtrlEx, public CComparableListing
 {
 public:
-	CFileListCtrl(wxWindow* pParent, CState *pState, CQueueView *pQueue);
+	CFileListCtrl(wxWindow* pParent, CState *pState, CQueueView *pQueue, bool border = false);
 	virtual ~CFileListCtrl();
 
 	class CSortComparisonObject : public std::binary_function<int,int,bool>
@@ -63,6 +63,7 @@ public:
 	};
 
 	void SetFilelistStatusBar(CFilelistStatusBar* pFilelistStatusBar) { m_pFilelistStatusBar = pFilelistStatusBar; }
+	CFilelistStatusBar* GetFilelistStatusBar() { return m_pFilelistStatusBar; }
 
 	void ClearSelection();
 
