@@ -34,37 +34,12 @@ END_EVENT_TABLE();
 
 CFilterCondition::CFilterCondition()
 {
-	type = name;
+	type = filter_name;
 	condition = 0;
-	pRegEx = 0;
 	matchCase = true;
 	value = 0;
 }
 
-CFilterCondition::~CFilterCondition()
-{
-	delete pRegEx;
-}
-
-CFilterCondition::CFilterCondition(const CFilterCondition& cond)
-{
-	pRegEx = 0;
-	*this = cond;
-}
-
-CFilterCondition& CFilterCondition::operator=(const CFilterCondition& cond)
-{
-	type = cond.type;
-	condition = cond.condition;
-	strValue = cond.strValue;
-	value = cond.value;
-	matchCase = cond.matchCase;
-	delete pRegEx;
-	pRegEx = 0;
-
-	return *this;
-}
-
 CFilter::CFilter()
 {
 	matchType = all;
@@ -90,6 +65,11 @@ bool CFilter::HasConditionOfType(enum t_filterType type) const
 	return false;
 }
 
+bool CFilter::IsLocalFilter() const
+{
+	 return HasConditionOfType(filter_attributes) || HasConditionOfType(filter_permissions);
+}
+
 CFilterDialog::CFilterDialog()
 {
 	m_pMainFrame = 0;
@@ -200,7 +180,29 @@ void CFilterDialog::SaveFilters()
 			const CFilterCondition& condition = *conditionIter;
 			TiXmlElement* pCondition = pConditions->InsertEndChild(TiXmlElement("Condition"))->ToElement();
 
-			AddTextElement(pCondition, "Type", condition.type);
+			int type;
+			switch (condition.type)
+			{
+			case filter_name:
+				type = 0;
+				break;
+			case filter_size:
+				type = 1;
+				break;
+			case filter_attributes:
+				type = 2;
+				break;
+			case filter_permissions:
+				type = 3;
+				break;
+			case filter_path:
+				type = 4;
+				break;
+			default:
+				wxFAIL_MSG(_T("Unhandled filter type"));
+				break;
+			}
+			AddTextElement(pCondition, "Type", type);
 			AddTextElement(pCondition, "Condition", condition.condition);
 			AddTextElement(pCondition, "Value", condition.strValue);
 		}
@@ -249,7 +251,7 @@ void CFilterDialog::DisplayFilters()
 	{
 		const CFilter& filter = m_filters[i];
 
-		const bool localOnly = filter.HasConditionOfType(attributes) || filter.HasConditionOfType(permissions);
+		const bool localOnly = filter.IsLocalFilter();
 
 		pLocalFilters->Append(filter.name);
 		pRemoteFilters->Append(filter.name);
@@ -279,7 +281,7 @@ void CFilterDialog::OnFilterSelect(wxCommandEvent& event)
 	int item = event.GetSelection();
 
 	const CFilter& filter = m_filters[item];
-	const bool localOnly = filter.HasConditionOfType(attributes) || filter.HasConditionOfType(permissions);
+	const bool localOnly = filter.IsLocalFilter();
 	if (localOnly && event.GetEventObject() != pLocal)
 	{
 		pRemote->Check(item, false);
@@ -490,7 +492,7 @@ void CFilterDialog::OnChangeAll(wxCommandEvent& event)
 
 	for (size_t i = 0; i < pListBox->GetCount(); i++)
 	{
-		if (!local && (m_filters[i].HasConditionOfType(attributes) || m_filters[i].HasConditionOfType(permissions)))
+		if (!local && (m_filters[i].IsLocalFilter()))
 		{
 			pListBox->Check(i, false);
 			(*pValues)[i] = false;
@@ -595,13 +597,13 @@ bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path
 		if (local)
 		{
 			if (set.local[i])
-				if (FilenameFilteredByFilter(name, path, dir, size, i, attributes))
+				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes))
 					return true;
 		}
 		else
 		{
 			if (set.remote[i])
-				if (FilenameFilteredByFilter(name, path, dir, size, i, attributes))
+				if (FilenameFilteredByFilter(m_globalFilters[i], name, path, dir, size, attributes))
 					return true;
 		}
 	}
@@ -609,11 +611,19 @@ bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path
 	return false;
 }
 
-bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxString& path, bool dir, wxLongLong size, unsigned int filterIndex, int attributes) const
+bool CFilterManager::FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes) const
 {
-	wxRegEx regex;
-	const CFilter& filter = m_globalFilters[filterIndex];
+	for (std::list<CFilter>::const_iterator iter = filters.begin(); iter != filters.end(); iter++)
+	{
+		if (FilenameFilteredByFilter(*iter, name, path, dir, size, attributes))
+			return true;
+	}
 
+	return false;
+}
+
+bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes)
+{
 	if (dir && !filter.filterDirs)
 		return false;
 	else if (!dir && !filter.filterFiles)
@@ -626,7 +636,7 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 
 		switch (condition.type)
 		{
-		case (enum t_filterType)::name:
+		case filter_name:
 			switch (condition.condition)
 			{
 			case 0:
@@ -689,7 +699,7 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 					match = true;
 			}
 			break;
-		case (enum t_filterType)::path:
+		case filter_path:
 			switch (condition.condition)
 			{
 			case 0:
@@ -752,7 +762,7 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 					match = true;
 			}
 			break;
-		case (enum t_filterType)::size:
+		case filter_size:
 			if (size == -1)
 				continue;
 			switch (condition.condition)
@@ -771,7 +781,7 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 				break;
 			}
 			break;
-		case (enum t_filterType)::attributes:
+		case filter_attributes:
 #ifndef __WXMSW__
 			continue;
 #else
@@ -808,7 +818,7 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 			}
 #endif //__WXMSW__
 			break;
-		case permissions:
+		case filter_permissions:
 #ifdef __WXMSW__
 			continue;
 #else
@@ -878,21 +888,31 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxStri
 	return false;
 }
 
+bool CFilterManager::CompileRegexes(CFilter& filter)
+{
+	for (std::vector<CFilterCondition>::iterator iter = filter.filters.begin(); iter != filter.filters.end(); iter++)
+	{
+		CFilterCondition& condition = *iter;
+		if ((condition.type == filter_name || condition.type == filter_path) && condition.condition == 4)
+		{
+			condition.pRegEx = new wxRegEx(condition.strValue);
+			if (!condition.pRegEx->IsValid())
+			{
+				condition.pRegEx.clear();
+				return false;
+			}
+		}
+		else
+			condition.pRegEx.clear();
+	}
+
+	return true;
+}
+
 bool CFilterManager::CompileRegexes()
 {
 	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
-	{
-		CFilter& filter = m_globalFilters[i];
-		for (std::vector<CFilterCondition>::iterator iter = filter.filters.begin(); iter != filter.filters.end(); iter++)
-		{
-			CFilterCondition& condition = *iter;
-			delete condition.pRegEx;
-			if ((condition.type == name || condition.type == path) && condition.condition == 4)
-				condition.pRegEx = new wxRegEx(condition.strValue);
-			else
-				condition.pRegEx = 0;
-		}
-	}
+		CompileRegexes(m_globalFilters[i]);
 	return true;
 }
 
@@ -970,12 +990,27 @@ void CFilterManager::LoadFilters()
 		{
 			CFilterCondition condition;
 			int type = GetTextElementInt(pCondition, "Type", 0);
-			if (type < 0 || type >= filterType_size)
+			switch (type)
 			{
+			case 0:
+				condition.type = filter_name;
+				break;
+			case 1:
+				condition.type = filter_size;
+				break;
+			case 2:
+				condition.type = filter_attributes;
+				break;
+			case 3:
+				condition.type = filter_permissions;
+				break;
+			case 4:
+				condition.type = filter_path;
+				break;
+			default:
 				pCondition = pCondition->NextSiblingElement("Condition");
 				continue;
 			}
-			condition.type = (enum t_filterType)type;
 			condition.condition = GetTextElementInt(pCondition, "Condition", 0);
 			condition.strValue = GetTextElement(pCondition, "Value");
 			condition.matchCase = filter.matchCase;
@@ -986,13 +1021,13 @@ void CFilterManager::LoadFilters()
 			}
 
 			// TODO: 64bit filesize
-			if (condition.type == size)
+			if (condition.type == filter_size)
 			{
 				unsigned long tmp;
 				condition.strValue.ToULong(&tmp);
 				condition.value = tmp;
 			}
-			else if (condition.type == attributes || condition.type == permissions)
+			else if (condition.type == filter_attributes || condition.type == filter_permissions)
 			{
 				if (condition.strValue == _T("0"))
 					condition.value = 0;
@@ -1062,3 +1097,30 @@ void CFilterManager::ToggleFilters()
 	if (HasActiveFilters(true))
 		m_filters_disabled = true;
 }
+
+std::list<CFilter> CFilterManager::GetActiveFilters(bool local)
+{
+	std::list<CFilter> filters;
+
+	if (m_filters_disabled)
+		return filters;
+
+	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
+
+	// Check active filters
+	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
+	{
+		if (local)
+		{
+			if (set.local[i])
+				filters.push_back(m_globalFilters[i]);
+		}
+		else
+		{
+			if (set.remote[i])
+				filters.push_back(m_globalFilters[i]);
+		}
+	}
+
+	return filters;
+}
