@@ -1,17 +1,14 @@
 #include "FileZilla.h"
 #include "led.h"
 #include "filezillaapp.h"
-#include "state.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #endif
 
-#define TIMER_ID (wxID_HIGHEST + 1)
-
 BEGIN_EVENT_TABLE(CLed, wxWindow)
 	EVT_PAINT(CLed::OnPaint)
-	EVT_TIMER(TIMER_ID, CLed::OnTimer)
+	EVT_TIMER(wxID_ANY, CLed::OnTimer)
 #ifdef __WXMSW__
 	EVT_ERASE_BACKGROUND(CLed::OnEraseBackground)
 #endif
@@ -20,11 +17,9 @@ END_EVENT_TABLE()
 #define LED_OFF 1
 #define LED_ON 0
 
-CLed::CLed(wxWindow *parent, unsigned int index, CState* pState)
+CLed::CLed(wxWindow *parent, unsigned int index)
 	: wxWindow(parent, -1, wxDefaultPosition, wxSize(11, 11))
 {
-	m_pState = pState;
-
 	if (index == 1)
 		m_index = 1;
 	else
@@ -32,10 +27,12 @@ CLed::CLed(wxWindow *parent, unsigned int index, CState* pState)
 
 	m_ledState = LED_OFF;
 
-	m_timer.SetOwner(this, TIMER_ID);
+	m_timer.SetOwner(this);
 
 	m_loaded = false;
 
+	m_pEngine = 0;
+
 	wxImage image;
 	if (!image.LoadFile(wxGetApp().GetResourceDir() + _T("leds.png"), wxBITMAP_TYPE_PNG))
 		return;
@@ -81,22 +78,16 @@ void CLed::Unset()
 
 void CLed::OnTimer(wxTimerEvent& event)
 {
-	if (event.GetId() != TIMER_ID)
+	if (!m_timer.IsRunning())
+		return;
+
+	if (event.GetId() != m_timer.GetId())
 	{
 		event.Skip();
 		return;
 	}
 
-	if (!m_timer.IsRunning())
-		return;
-
-	if (!m_pState->m_pEngine)
-	{
-		m_timer.Stop();
-		return;
-	}
-
-	if (!m_pState->m_pEngine->IsActive(m_index == 0))
+	if (!m_pEngine || !m_pEngine->IsActive((enum CFileZillaEngine::_direction)m_index))
 	{
 		Unset();
 		m_timer.Stop();
@@ -107,9 +98,12 @@ void CLed::OnTimer(wxTimerEvent& event)
 
 void CLed::Ping()
 {
-	if (m_timer.IsRunning())
+	if (!m_loaded)
 		return;
 
+	if (m_timer.IsRunning())
+		return;
+	
 	Set();
 	m_timer.Start(100);
 }
@@ -119,3 +113,8 @@ void CLed::OnEraseBackground(wxEraseEvent& event)
 {
 }
 #endif
+
+void CLed::SetEngine(CFileZillaEngine *pEngine)
+{
+	m_pEngine = pEngine;
+}
