@@ -9,24 +9,31 @@ CQueueItem::CQueueItem()
 	m_visibleOffspring = 0;
 	m_parent = 0;
 	m_maxCachedIndex = -1;
+
+	m_removed_at_front = 0;
 }
 
 CQueueItem::~CQueueItem()
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 		delete *iter;
 }
 
 void CQueueItem::SetPriority(enum QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 		(*iter)->SetPriority(priority);
 }
 
 void CQueueItem::AddChild(CQueueItem* item)
 {
+	if (m_removed_at_front)
+	{
+		m_children.erase(m_children.begin(), m_children.begin() + m_removed_at_front);
+		m_removed_at_front = 0;
+	}
 	item->m_indent = m_indent + _T("  ");
 	m_children.push_back(item);
 
@@ -46,17 +53,17 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 	std::vector<CQueueItem*>::iterator iter;
 	if (!recursive)
 	{
-		if (item >= m_children.size())
+		if (item + m_removed_at_front >= m_children.size())
 			return 0;
 		iter = m_children.begin();
-		iter += item;
+		iter += item + m_removed_at_front;
 		return *iter;
 	}
 
 	if ((int)item <= m_maxCachedIndex)
 	{
 		// Index is cached
-		iter = m_children.begin();
+		iter = m_children.begin() + m_removed_at_front;
 		iter += m_lookupCache[item].child;
 		item -= m_lookupCache[item].index;
 		if (!item)
@@ -67,7 +74,7 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 
 	int index;
 	int child;
-	iter = m_children.begin();
+	iter = m_children.begin() + m_removed_at_front;
 	if (m_maxCachedIndex == -1)
 	{
 		child = 0;
@@ -111,7 +118,7 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 unsigned int CQueueItem::GetChildrenCount(bool recursive)
 {
 	if (!recursive)
-		return m_children.size();
+		return m_children.size() - m_removed_at_front;
 
 	return m_visibleOffspring;
 }
@@ -121,7 +128,7 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 	int oldVisibleOffspring = m_visibleOffspring;
 	std::vector<CQueueItem*>::iterator iter;
 	bool deleted = false;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		if (*iter == pItem)
 		{
@@ -129,7 +136,17 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 			m_visibleOffspring -= pItem->m_visibleOffspring;
 			if (destroy)
 				delete pItem;
-			m_children.erase(iter);
+
+			if (iter - m_children.begin() - m_removed_at_front <= 10)
+			{
+				m_removed_at_front++;
+				unsigned int end = iter - m_children.begin();
+				int c = 0;
+				for (int i = end; i >= m_removed_at_front; i--, c++)
+					m_children[i] = m_children[i - 1];
+			}
+			else
+				m_children.erase(iter);
 
 			deleted = true;
 			break;
@@ -139,17 +156,27 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 		if ((*iter)->RemoveChild(pItem, destroy))
 		{
 			m_visibleOffspring -= visibleOffspring - (*iter)->m_visibleOffspring;
-			if (!(*iter)->m_children.size())
+			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front))
 			{
 				m_visibleOffspring -= 1;
 				delete *iter;
-				m_children.erase(iter);
+
+				if (iter - m_children.begin() - m_removed_at_front <= 10)
+				{
+					m_removed_at_front++;
+					unsigned int end = iter - m_children.begin();
+					for (int i = end; i >= m_removed_at_front; i--)
+						m_children[i] = m_children[i - 1];
+				}
+				else
+					m_children.erase(iter);
 			}
 
 			deleted = true;
 			break;
 		}
 	}
+
 	if (!deleted)
 		return false;
 
@@ -173,7 +200,7 @@ bool CQueueItem::TryRemoveAll()
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
 	m_visibleOffspring = 0;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		CQueueItem* pItem = *iter;
 		if (pItem->TryRemoveAll())
@@ -187,6 +214,7 @@ bool CQueueItem::TryRemoveAll()
 	}
 	m_children = keepChildren;
 	m_maxCachedIndex = -1;
+	m_removed_at_front = 0;
 
 	CQueueItem* parent = GetParent();
 	while (parent)
@@ -238,7 +266,7 @@ int CQueueItem::GetItemIndex() const
 		return 0;
 
 	int index = 1;
-	for (std::vector<CQueueItem*>::const_iterator iter = pParent->m_children.begin(); iter != pParent->m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::const_iterator iter = pParent->m_children.begin() + pParent->m_removed_at_front; iter != pParent->m_children.end(); iter++)
 	{
 		if (*iter == this)
 			break;
@@ -251,15 +279,12 @@ int CQueueItem::GetItemIndex() const
 
 CFileItem::CFileItem(CServerItem* parent, bool queued, bool download, const wxString& localFile,
 					 const wxString& remoteFile, const CServerPath& remotePath, wxLongLong size)
+	: m_localFile(localFile), m_remoteFile(remoteFile), m_remotePath(remotePath), m_size(size)
 {
 	m_parent = parent;
 	m_priority = priority_normal;
 
 	m_download = download;
-	m_localFile = localFile;
-	m_remoteFile = remoteFile;
-	m_remotePath = remotePath;
-	m_size = size;
 	m_queued = queued;
 	m_active = false;
 	m_errorCount = 0;
@@ -268,7 +293,7 @@ CFileItem::CFileItem(CServerItem* parent, bool queued, bool download, const wxSt
 	m_defaultFileExistsAction = CFileExistsNotification::unknown;
 	m_edit = CEditHandler::none;
 	m_madeProgress = false;
-	m_autoResume = false;
+	m_onetime_action = CFileExistsNotification::unknown;
 }
 
 CFileItem::~CFileItem()
@@ -303,7 +328,7 @@ void CFileItem::SetActive(const bool active)
 	}
 	else if (!active && m_active)
 	{
-		CQueueItem* pItem = m_children.front();
+		CQueueItem* pItem = GetChild(0, false);
 		RemoveChild(pItem);
 	}
 	m_active = active;
@@ -314,21 +339,21 @@ void CFileItem::SaveItem(TiXmlElement* pElement) const
 	if (m_edit != CEditHandler::none)
 		return;
 
-	TiXmlElement file("File");
+	TiXmlElement *file = pElement->LinkEndChild(new TiXmlElement("File"))->ToElement();
 
-	AddTextElement(&file, "LocalFile", m_localFile);
-	AddTextElement(&file, "RemoteFile", m_remoteFile);
-	AddTextElement(&file, "RemotePath", m_remotePath.GetSafePath());
-	AddTextElementRaw(&file, "Download", m_download ? "1" : "0");
+	AddTextElement(file, "LocalFile", m_localFile);
+	AddTextElement(file, "RemoteFile", m_remoteFile);
+	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
+	AddTextElementRaw(file, "Download", m_download ? "1" : "0");
 	if (m_size != -1)
-		AddTextElement(&file, "Size", m_size.ToString());
+		AddTextElement(file, "Size", m_size.ToString());
 	if (m_errorCount)
-		AddTextElement(&file, "ErrorCount", m_errorCount);
+		AddTextElement(file, "ErrorCount", m_errorCount);
 	if (m_priority != priority_normal)
-		AddTextElement(&file, "Priority", m_priority);
-	AddTextElementRaw(&file, "TransferMode", m_transferSettings.binary ? "1" : "0");
-
-	pElement->InsertEndChild(file);
+		AddTextElement(file, "Priority", m_priority);
+	AddTextElementRaw(file, "TransferMode", m_transferSettings.binary ? "1" : "0");
+	if (m_defaultFileExistsAction != CFileExistsNotification::unknown)
+		AddTextElement(file, "OverwriteAction", m_defaultFileExistsAction);
 }
 
 bool CFileItem::TryRemoveAll()
@@ -340,11 +365,18 @@ bool CFileItem::TryRemoveAll()
 	return false;
 }
 
-void CFileItem::SetLocalFile(const wxString file)
+void CFileItem::SetLocalFile(const wxString &file)
 {
+	wxASSERT(!file.empty());
 	m_localFile = file;
 }
 
+void CFileItem::SetRemoteFile(const wxString &file)
+{
+	wxASSERT(!file.empty());
+	m_remoteFile = file;
+}
+
 CFolderItem::CFolderItem(CServerItem* parent, bool queued, const wxString& localFile)
 	: CFileItem(parent, queued, true, localFile, _T(""), CServerPath(), -1)
 {
@@ -428,7 +460,7 @@ void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
 
 void CServerItem::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction)
 {
-	for (std::vector<CQueueItem*>::iterator iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		CQueueItem *pItem = *iter;
 		if (pItem->GetType() == QueueItemType_File)
@@ -558,13 +590,13 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 
 void CServerItem::SaveItem(TiXmlElement* pElement) const
 {
-	TiXmlElement server("Server");
-	SetServer(&server, m_server);
+	TiXmlElement *server = new TiXmlElement("Server");
+	SetServer(server, m_server);
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin(); iter != m_children.end(); iter++)
-		(*iter)->SaveItem(&server);
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+		(*iter)->SaveItem(server);
 
-	pElement->InsertEndChild(server);
+	pElement->LinkEndChild(server);
 }
 
 wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
@@ -587,7 +619,7 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 		}
 	}
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		if ((*iter)->GetType() == QueueItemType_File ||
 			(*iter)->GetType() == QueueItemType_Folder)
@@ -605,7 +637,7 @@ bool CServerItem::TryRemoveAll()
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
 	m_visibleOffspring = 0;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		CQueueItem* pItem = *iter;
 		if (pItem->TryRemoveAll())
@@ -625,6 +657,7 @@ bool CServerItem::TryRemoveAll()
 		}
 	}
 	m_children = keepChildren;
+	m_removed_at_front = 0;
 
 	CQueueItem* parent = GetParent();
 	while (parent)
@@ -643,6 +676,7 @@ void CServerItem::DetachChildren()
 	m_children.clear();
 	m_visibleOffspring = 0;
 	m_maxCachedIndex = -1;
+	m_removed_at_front = 0;
 
 	for (int i = 0; i < 2; i++)
 		for (int j = 0; j < PRIORITY_COUNT; j++)
@@ -652,7 +686,7 @@ void CServerItem::DetachChildren()
 void CServerItem::SetPriority(enum QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
 	{
 		if ((*iter)->GetType() == QueueItemType_File)
 			((CFileItem*)(*iter))->SetPriorityRaw(priority);
@@ -718,6 +752,7 @@ EVT_NAVIGATION_KEY(CQueueViewBase::OnNavigationKey)
 EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
 EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
+EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
@@ -1392,12 +1427,14 @@ void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
 {
 	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); i++)
 	{
-		CQueueViewBase* page = (CQueueViewBase*)m_pQueue->GetPage(i);
-		if (!page || page == this)
+		wxWindow *page = m_pQueue->GetPage(i);
+
+		wxListCtrl* queue_page = wxDynamicCast(page, wxListCtrl);
+		if (!queue_page || queue_page == this)
 			continue;
 
-		for (int col = 0; col < wxMin(GetColumnCount(), page->GetColumnCount()); col++)
-			page->SetColumnWidth(col, GetColumnWidth(col));
+		for (int col = 0; col < wxMin(GetColumnCount(), queue_page->GetColumnCount()); col++)
+			queue_page->SetColumnWidth(col, GetColumnWidth(col));
 	}
 }
 
@@ -1413,6 +1450,19 @@ void CQueueViewBase::OnTimer(wxTimerEvent& event)
 		DisplayNumberQueuedFiles();
 }
 
+void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
+{
+	const int code = event.GetKeyCode();
+	const int mods = event.GetModifiers();
+	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META)))
+	{
+		for (unsigned int i = 0; i < (unsigned int)GetItemCount(); i++)
+			SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+	}
+	else
+		event.Skip();
+}
+
 // ------
 // CQueue
 // ------
