@@ -347,6 +347,29 @@ bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path,
 	}
 	int margin = m_pNotebook_Site->GetSize().x - m_pNotebook_Site->GetPage(0)->GetSize().x;
 	m_pNotebook_Site->GetPage(0)->GetSizer()->SetMinSize(wxSize(width - margin, 0));
+#else
+	// Make pages at least wide enough to fit all tabs
+	int width = 10; // Guessed
+	wxClientDC dc(m_pNotebook_Site);
+	for (unsigned int i = 0; i < m_pNotebook_Site->GetPageCount(); i++)
+	{
+		wxCoord w, h;
+		dc.GetTextExtent(m_pNotebook_Site->GetPageText(i), &w, &h);
+		
+		width += w;
+#ifdef __WXMAC__
+		width += 20; // Guessed
+#else
+		width += 10;
+#endif
+	}
+	
+	wxSize page_min_size = m_pNotebook_Site->GetPage(0)->GetSizer()->GetMinSize();
+	if (page_min_size.x < width)
+	{
+		page_min_size.x = width;
+		m_pNotebook_Site->GetPage(0)->GetSizer()->SetMinSize(page_min_size);
+	}
 #endif
 
 	Layout();
@@ -753,7 +776,7 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 
 					TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
 					if (localDir)
-						data->m_localDir = ConvLocal(localDir->Value());
+						data->m_localDir = GetTextElement(pBookmark, "LocalDir");
 
 					TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
 					if (remoteDir)
@@ -765,6 +788,9 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 						continue;
 					}
 
+					if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+						data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+
 					pHandler->AddBookmark(name, data);
 				}
 
@@ -801,6 +827,9 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 	if (remoteDir)
 		data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
 
+	if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+		data->m_sync = GetTextElementBool(pElement, "SyncBrowsing", false);
+
 	return data;
 }
 
@@ -847,6 +876,8 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		wxString error;
 		if (!xml.Save(&error))
 		{
+			if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+				return res;
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFullPath().c_str(), error.c_str());
 			wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 		}
@@ -902,6 +933,8 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 		// Save remote dir
 		AddTextElement(pNode, "RemoteDir", data->m_remoteDir.GetSafePath());
 
+		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
+
 		pNode->InsertEndChild(TiXmlText(utf8));
 
 		Save(pNode, child);
@@ -923,6 +956,8 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 		
 		// Save remote dir
 		AddTextElement(pNode, "RemoteDir", data->m_remoteDir.GetSafePath());
+
+		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
 	}
 	
 	delete [] utf8;
@@ -1091,6 +1126,17 @@ bool CSiteManager::Verify()
 				return false;
 			}
 		}
+
+		const wxString localPath = XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->GetValue();
+		if (XRCCTRL(*this, "ID_SYNC", wxCheckBox)->GetValue())
+		{
+			if (remotePathRaw.empty() || localPath.empty())
+			{
+				XRCCTRL(*this, "ID_SYNC", wxCheckBox)->SetFocus();
+				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this site."));
+				return false;
+			}
+		}
 	}
 	else
 	{
@@ -1125,6 +1171,16 @@ bool CSiteManager::Verify()
 			wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."));
 			return false;
 		}
+
+		if (XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->GetValue())
+		{
+			if (remotePathRaw.empty() || localPath.empty())
+			{
+				XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetFocus();
+				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."));
+				return false;
+			}
+		}
 	}
 
 	return true;
@@ -1341,6 +1397,7 @@ bool CSiteManager::UpdateBookmark(CSiteManagerItemData &bookmark, const CServer&
 	bookmark.m_remoteDir = CServerPath();
 	bookmark.m_remoteDir.SetType(server.GetType());
 	bookmark.m_remoteDir.SetPath(XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue());
+	bookmark.m_sync = XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->GetValue();
 
 	return true;
 }
@@ -1381,6 +1438,8 @@ bool CSiteManager::UpdateServer(CSiteManagerItemData_Site &server, const wxStrin
 	server.m_remoteDir = CServerPath();
 	server.m_remoteDir.SetType(server.m_server.GetType());
 	server.m_remoteDir.SetPath(XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->GetValue());
+	server.m_sync = XRCCTRL(*this, "ID_SYNC", wxCheckBox)->GetValue();
+
 	int hours, minutes;
 	hours = XRCCTRL(*this, "ID_TIMEZONE_HOURS", wxSpinCtrl)->GetValue();
 	minutes = XRCCTRL(*this, "ID_TIMEZONE_MINUTES", wxSpinCtrl)->GetValue();
@@ -1445,6 +1504,10 @@ bool CSiteManager::GetServer(CSiteManagerItemData_Site& data)
 			data.m_localDir = pData->m_localDir;
 		if (!pData->m_remoteDir.IsEmpty())
 			data.m_remoteDir = pData->m_remoteDir;
+		if (!data.m_localDir.empty() && !data.m_remoteDir.IsEmpty())
+			data.m_sync = true;
+		else
+			data.m_sync = false;
 	}
 	else
 		data = *(CSiteManagerItemData_Site *)pData;
@@ -1545,6 +1608,7 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetSelection(0);
 		XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetValue(_T(""));
+		XRCCTRL(*this, "ID_SYNC", wxCheckBox)->SetValue(false);
 		XRCCTRL(*this, "ID_TIMEZONE_HOURS", wxSpinCtrl)->SetValue(0);
 		XRCCTRL(*this, "ID_TIMEZONE_MINUTES", wxSpinCtrl)->SetValue(0);
 
@@ -1613,6 +1677,8 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_LOCALDIR", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetValue(site_data->m_remoteDir.GetPath());
 		XRCCTRL(*this, "ID_REMOTEDIR", wxWindow)->Enable(!predefined);
+		XRCCTRL(*this, "ID_SYNC", wxCheckBox)->Enable(!predefined);
+		XRCCTRL(*this, "ID_SYNC", wxCheckBox)->SetValue(site_data->m_sync);
 		XRCCTRL(*this, "ID_TIMEZONE_HOURS", wxSpinCtrl)->SetValue(site_data->m_server.GetTimezoneOffset() / 60);
 		XRCCTRL(*this, "ID_TIMEZONE_HOURS", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_TIMEZONE_MINUTES", wxSpinCtrl)->SetValue(site_data->m_server.GetTimezoneOffset() % 60);
@@ -1684,6 +1750,9 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->SetValue(data->m_remoteDir.GetPath());
 		XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxWindow)->Enable(!predefined);
+
+		XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->Enable(true);
+		XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetValue(data->m_sync);
 	}
 #ifdef __WXGTK__
 	if (pFocus && !pFocus->IsEnabled())
@@ -2547,6 +2616,12 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
 		if (remoteDir)
 			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
+		if (!localPath.empty() && !remotePath.IsEmpty())
+		{
+			data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+		}
+		else
+			data->m_sync = false;
 
 		data->m_localDir = localPath;
 		data->m_remoteDir = remotePath;
@@ -2819,6 +2894,9 @@ wxString CSiteManager::AddServer(CServer server)
 	wxString error;
 	if (!file.Save(&error))
 	{
+		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+			return _T("");
+
 		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 		return _T("");
@@ -2863,7 +2941,7 @@ TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxSt
 	return pNode;
 }
 
-bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir)
+bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
 {
 	if (local_dir.empty() && remote_dir.IsEmpty())
 		return false;
@@ -2909,6 +2987,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	}
 
 	// Bookmarks
+	TiXmlElement *pInsertBefore = 0;
 	TiXmlElement* pBookmark;
 	for (pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
 	{
@@ -2923,18 +3002,28 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 			wxMessageBox(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
+		if (name < old_name && !pInsertBefore)
+			pInsertBefore = pBookmark;
 	}
 
-	pBookmark = pChild->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
+	if (pInsertBefore)
+		pBookmark = pChild->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
+	else
+		pBookmark = pChild->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
 	if (!remote_dir.IsEmpty())
 		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
+	if (sync)
+		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
 
 	wxString error;
 	if (!file.Save(&error))
 	{
+		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+			return true;
+
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
@@ -2994,6 +3083,9 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	wxString error;
 	if (!file.Save(&error))
 	{
+		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+			return true;
+
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
