@@ -6,13 +6,123 @@
 #include "Mainfrm.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "RemoteListView.h"
 #include "recursive_operation.h"
 #include "statusbar.h"
 #include "local_filesys.h"
+#include "listingcomparison.h"
+
+CContextManager the_context_manager;
+
+CContextManager* CContextManager::Get()
+{
+	return &the_context_manager;
+}
+
+CState* CContextManager::CreateState(CMainFrame* pMainFrame)
+{
+	wxASSERT(pMainFrame);
+
+	CState* pState = new CState(pMainFrame);
+
+	m_contexts.push_back(pState);
+
+	return pState;
+}
+
+void CContextManager::DestroyState(CState* pState)
+{
+	for (unsigned int i = 0; i < m_contexts.size(); i++)
+	{
+		if (m_contexts[i] != pState)
+			continue;
+
+		m_contexts.erase(m_contexts.begin() + i);
+		if ((int)i < m_current_context)
+			m_current_context--;
+		else if ((int)i == m_current_context)
+		{
+			m_current_context--;
+			// TODO context change
+		}
+
+		break;
+	}
+
+	delete pState;
+}
+
+void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable)
+{
+	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
+
+	std::list<t_handler> &handlers = m_handlers[notification];
+	std::list<t_handler>::const_iterator iter;
+	for (iter = handlers.begin(); iter != handlers.end(); iter++)
+	{
+		if (iter->pHandler == pHandler)
+			return;
+	}
+
+	t_handler handler;
+	handler.pHandler = pHandler;
+	handler.blockable = blockable;
+	handler.current_only = current_only;
+	handlers.push_back(handler);
+}
+
+void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+{
+	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX);
+
+	if (notification == STATECHANGE_NONE)
+	{
+		for (int i = 0; i < STATECHANGE_MAX; i++)
+		{
+			std::list<t_handler> &handlers = m_handlers[i];
+			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			{
+				if (iter->pHandler == pHandler)
+				{
+					handlers.erase(iter);
+					break;
+				}
+			}
+		}
+	}
+	else
+	{
+		std::list<t_handler> &handlers = m_handlers[notification];
+		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		{
+			if (iter->pHandler == pHandler)
+			{
+				handlers.erase(iter);
+				return;
+			}
+		}
+	}
+}
+
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked)
+{
+	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
+
+	const std::list<t_handler> &handlers = m_handlers[notification];
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	{
+		if (blocked && iter->blockable)
+			continue;
+
+		iter->pHandler->OnStateChange(pState, notification, data, data2);
+	}
+}
 
 CState::CState(CMainFrame* pMainFrame)
 {
+	memset(m_blocked, 0, sizeof(m_blocked));
+
 	m_pMainFrame = pMainFrame;
 
 	m_pDirectoryListing = 0;
@@ -21,25 +131,29 @@ CState::CState(CMainFrame* pMainFrame)
 
 	m_pEngine = 0;
 	m_pCommandQueue = 0;
+	m_pComparisonManager = new CComparisonManager(this);
 
 	m_pRecursiveOperation = new CRecursiveOperation(this);
 
 	m_sync_browse.is_changing = false;
+	m_sync_browse.compare = false;
 }
 
 CState::~CState()
 {
-	delete m_pDirectoryListing;
 	delete m_pServer;
 
+	delete m_pComparisonManager;
 	delete m_pCommandQueue;
 	delete m_pEngine;
 
 	// Unregister all handlers
 	for (int i = 0; i < STATECHANGE_MAX; i++)
 	{
-		for (std::list<CStateEventHandler*>::iterator iter = m_handlers[i].begin(); iter != m_handlers[i].end(); iter++)
-			(*iter)->m_pState = 0;
+		for (std::list<t_handler>::iterator iter = m_handlers[i].begin(); iter != m_handlers[i].end(); iter++)
+		{
+			iter->pHandler->m_pState = 0;
+		}
 	}
 
 	delete m_pRecursiveOperation;
@@ -107,6 +221,7 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 			}
 
 			m_sync_browse.is_changing = true;
+			m_sync_browse.compare = m_pComparisonManager->IsComparing();
 			CListCommand *pCommand = new CListCommand(remote_path);
 			m_pCommandQueue->ProcessCommand(pCommand);
 
@@ -133,10 +248,8 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 
 		if (m_pDirectoryListing)
 		{
-			const CDirectoryListing* pOldListing = m_pDirectoryListing;
 			m_pDirectoryListing = 0;
 			NotifyHandlers(STATECHANGE_REMOTE_DIR);
-			delete pOldListing;
 		}
 		return true;
 	}
@@ -163,7 +276,6 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 		return true;
 	}
 
-	const CDirectoryListing *pOldListing = m_pDirectoryListing;
 	m_pDirectoryListing = pDirectoryListing;
 
 	if (!modified)
@@ -171,8 +283,6 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 	else
 		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
 
-	delete pOldListing;
-
 	if (m_sync_browse.is_changing)
 	{
 		m_sync_browse.is_changing = false;
@@ -210,12 +320,15 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 			COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
 
 			NotifyHandlers(STATECHANGE_LOCAL_DIR);
+
+			if (m_sync_browse.compare)
+				m_pComparisonManager->CompareListings();
 		}
 	}
 	return true;
 }
 
-const CDirectoryListing *CState::GetRemoteDir() const
+CSharedPointer<const CDirectoryListing> CState::GetRemoteDir() const
 {
 	return m_pDirectoryListing;
 }
@@ -287,24 +400,12 @@ void CState::SetServer(const CServer* server)
 		SetRemoteDir(0);
 		delete m_pServer;
 	}
-
-	m_successful_connect = false;
-
-	CStatusBar* const pStatusBar = m_pMainFrame->GetStatusBar();
-	if (pStatusBar)
-	{
-		pStatusBar->DisplayDataType(server);
-		pStatusBar->DisplayEncrypted(server);
-	}
-
 	if (server)
 		m_pServer = new CServer(*server);
 	else
-	{
-		if (m_pServer)
-			m_pMainFrame->SetTitle(_T("FileZilla"));
 		m_pServer = 0;
-	}
+
+	m_successful_connect = false;
 
 	NotifyHandlers(STATECHANGE_SERVER);
 }
@@ -363,20 +464,24 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool blockable /*=true*/)
 {
 	wxASSERT(pHandler);
+	wxASSERT(pHandler->m_pState == this);
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 
-	std::list<CStateEventHandler*> &handlers = m_handlers[notification];
-	std::list<CStateEventHandler*>::const_iterator iter;
+	std::list<t_handler> &handlers = m_handlers[notification];
+	std::list<t_handler>::const_iterator iter;
 	for (iter = handlers.begin(); iter != handlers.end(); iter++)
 	{
-		if (*iter == pHandler)
+		if (iter->pHandler == pHandler)
 			return;
 	}
 
-	handlers.push_back(pHandler);
+	t_handler handler;
+	handler.pHandler = pHandler;
+	handler.blockable = blockable;
+	handlers.push_back(handler);
 }
 
 void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
@@ -388,10 +493,10 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	{
 		for (int i = 0; i < STATECHANGE_MAX; i++)
 		{
-			std::list<CStateEventHandler*> &handlers = m_handlers[i];
-			for (std::list<CStateEventHandler*>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			std::list<t_handler> &handlers = m_handlers[i];
+			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
 			{
-				if (*iter == pHandler)
+				if (iter->pHandler == pHandler)
 				{
 					handlers.erase(iter);
 					break;
@@ -401,10 +506,10 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 	else
 	{
-		std::list<CStateEventHandler*> &handlers = m_handlers[notification];
-		for (std::list<CStateEventHandler*>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		std::list<t_handler> &handlers = m_handlers[notification];
+		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
 		{
-			if (*iter == pHandler)
+			if (iter->pHandler == pHandler)
 			{
 				handlers.erase(iter);
 				return;
@@ -413,21 +518,51 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 }
 
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/)
+void CState::BlockHandlers(enum t_statechange_notifications notification)
+{
+	wxASSERT(notification != STATECHANGE_MAX);
+
+	if (notification == STATECHANGE_NONE)
+	{
+		for (int i = 0; i < STATECHANGE_MAX; i++)
+			m_blocked[i] = true;
+	}
+	else
+		m_blocked[notification] = true;
+}
+
+void CState::UnblockHandlers(enum t_statechange_notifications notification)
+{
+	wxASSERT(notification != STATECHANGE_MAX);
+
+	if (notification == STATECHANGE_NONE)
+	{
+		for (int i = 0; i < STATECHANGE_MAX; i++)
+			m_blocked[i] = false;
+	}
+	else
+		m_blocked[notification] = false;
+
+}
+
+void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
-	const std::list<CStateEventHandler*> &handlers = m_handlers[notification];
-	for (std::list<CStateEventHandler*>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	const std::list<t_handler> &handlers = m_handlers[notification];
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
 	{
-		(*iter)->OnStateChange(notification, data);
+		if (m_blocked[notification] && iter->blockable)
+			continue;
+
+		iter->pHandler->OnStateChange(this, notification, data, data2);
 	}
+
+	CContextManager::Get()->NotifyHandlers(this, notification, data, data2, m_blocked[notification]);
 }
 
 CStateEventHandler::CStateEventHandler(CState* pState)
 {
-	wxASSERT(pState);
-
 	if (!pState)
 		return;
 
@@ -436,9 +571,11 @@ CStateEventHandler::CStateEventHandler(CState* pState)
 
 CStateEventHandler::~CStateEventHandler()
 {
-	if (!m_pState)
-		return;
-	m_pState->UnregisterHandler(this, STATECHANGE_NONE);
+	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
+
+	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); iter++)
+		(*iter)->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
@@ -631,7 +768,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 
 	if (hasDirs)
 	{
-		if (!m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle())
+		if (!IsRemoteConnected() || !IsRemoteIdle())
 			return false;
 	}
 
@@ -641,7 +778,21 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	if (!hasDirs)
 		return true;
 
-	return m_pMainFrame->GetRemoteListView()->DownloadDroppedFiles(pRemoteDataObject, path, queueOnly);
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	{
+		if (!iter->dir)
+			continue;
+
+		m_pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + iter->name, iter->link);
+	}
+
+	if (m_pComparisonManager->IsComparing())
+		m_pComparisonManager->ExitComparisonMode();
+
+	CFilterManager filter;
+	m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, pRemoteDataObject->GetServerPath(), filter.GetActiveFilters(false));
+
+	return true;
 }
 
 bool CState::IsRemoteConnected() const
@@ -734,13 +885,10 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 {
 	m_sync_browse.is_changing = false;
 
-	if (m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
-		m_pRecursiveOperation->LinkIsNotDir();
-	else
-		m_pMainFrame->GetRemoteListView()->LinkIsNotDir(path, subdir);
+	NotifyHandlers(STATECHANGE_REMOTE_LINKNOTDIR, subdir, &path);
 }
 
-bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=_T("")*/, int flags /*=0*/)
+bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=_T("")*/, int flags /*=0*/, bool ignore_busy /*=false*/)
 {
 	if (!m_pServer || !m_pCommandQueue)
 		return false;
@@ -764,7 +912,7 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle())
+		else if (!IsRemoteIdle() && !ignore_busy)
 		{
 			wxString msg(_("Another remote operation is already in progress, cannot change directory now."));
 			wxMessageBox(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
@@ -790,7 +938,10 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 				SetSyncBrowse(false);
 			}
 			else
+			{
 				m_sync_browse.is_changing = true;
+				m_sync_browse.compare = m_pComparisonManager->IsComparing();
+			}
 		}
 	}
 
@@ -800,13 +951,14 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 	return true;
 }
 
-bool CState::SetSyncBrowse(bool enable)
+bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /*=CServerPath()*/)
 {
 	if (enable != m_sync_browse.local_root.empty())
 		return enable;
 
 	if (!enable)
 	{
+		wxASSERT(assumed_remote_root.IsEmpty());
 		m_sync_browse.local_root.clear();
 		m_sync_browse.remote_root.Clear();
 		m_sync_browse.is_changing = false;
@@ -815,12 +967,20 @@ bool CState::SetSyncBrowse(bool enable)
 		return false;
 	}
 
-	if (!m_pDirectoryListing)
+	if (!m_pDirectoryListing && assumed_remote_root.IsEmpty())
 		return false;
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.local_root = m_localDir;
-	m_sync_browse.remote_root = m_pDirectoryListing->path;
+
+	if (assumed_remote_root.IsEmpty())
+		m_sync_browse.remote_root = m_pDirectoryListing->path;
+	else
+	{
+		m_sync_browse.remote_root = assumed_remote_root;
+		m_sync_browse.is_changing = true;
+		m_sync_browse.compare = false;
+	}
 
 	while (m_sync_browse.local_root.HasParent() && m_sync_browse.remote_root.HasParent() &&
 		m_sync_browse.local_root.GetLastSegment() == m_sync_browse.remote_root.GetLastSegment())
