@@ -11,6 +11,7 @@ enum t_statechange_notifications
 	STATECHANGE_REMOTE_DIR_MODIFIED,
 	STATECHANGE_REMOTE_RECV,
 	STATECHANGE_REMOTE_SEND,
+	STATECHANGE_REMOTE_LINKNOTDIR,
 	STATECHANGE_LOCAL_DIR,
 
 	// data contains name (excluding path) of file to refresh
@@ -23,6 +24,7 @@ enum t_statechange_notifications
 
 	STATECHANGE_QUEUEPROCESSING,
 	STATECHANGE_SYNC_BROWSE,
+	STATECHANGE_COMPARISON,
 
 	STATECHANGE_MAX
 };
@@ -34,6 +36,39 @@ class CMainFrame;
 class CStateEventHandler;
 class CRemoteDataObject;
 class CRecursiveOperation;
+class CComparisonManager;
+
+class CState;
+class CContextManager
+{
+	friend class CState;
+public:
+	// If current_only is set, only notifications from the current (at the time
+	// of notification emission) context is dispatched to the handler.
+	void RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable);
+	void UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification);
+
+	CState* CreateState(CMainFrame* pMainFrame);
+	void DestroyState(CState* pState);
+
+	CState* GetCurrentContext();
+	const std::vector<CState*>* GetAllStates() { return &m_contexts; }
+
+	static CContextManager* Get();
+protected:
+	std::vector<CState*> m_contexts;
+	int m_current_context;
+
+	struct t_handler
+	{
+		CStateEventHandler* pHandler;
+		bool blockable;
+		bool current_only;
+	};
+	std::list<t_handler> m_handlers[STATECHANGE_MAX];
+
+	void NotifyHandlers(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked);
+};
 
 class CState
 {
@@ -53,9 +88,9 @@ public:
 
 	bool Connect(const CServer& server, bool askBreak, const CServerPath& path = CServerPath());
 
-	bool ChangeRemoteDir(const CServerPath& path, const wxString& subdir = _T(""), int flags = 0);
+	bool ChangeRemoteDir(const CServerPath& path, const wxString& subdir = _T(""), int flags = 0, bool ignore_busy = false);
 	bool SetRemoteDir(const CDirectoryListing *m_pDirectoryListing, bool modified = false);
-	const CDirectoryListing *GetRemoteDir() const;
+	CSharedPointer<const CDirectoryListing> GetRemoteDir() const;
 	const CServerPath GetRemotePath() const;
 
 	const CServer* GetServer() const;
@@ -64,11 +99,15 @@ public:
 	void RefreshLocalFile(wxString file);
 	void LocalDirCreated(const CLocalPath& path);
 
-	void RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification);
+	void RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool blockable = true);
 	void UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification);
 
+	void BlockHandlers(enum t_statechange_notifications notification);
+	void UnblockHandlers(enum t_statechange_notifications notification);
+
 	CFileZillaEngine* m_pEngine;
 	CCommandQueue* m_pCommandQueue;
+	CComparisonManager* GetComparisonManager() { return m_pComparisonManager; }
 
 	void UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly);
 	void UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly);
@@ -82,7 +121,7 @@ public:
 
 	CRecursiveOperation* GetRecursiveOperationHandler() { return m_pRecursiveOperation; }
 
-	void NotifyHandlers(enum t_statechange_notifications notification, const wxString& data = _T(""));
+	void NotifyHandlers(enum t_statechange_notifications notification, const wxString& data = _T(""), const void* data2 = 0);
 
 	bool SuccessfulConnect() const { return m_successful_connect; }
 	void SetSuccessfulConnect() { m_successful_connect = true; }
@@ -90,14 +129,14 @@ public:
 	void ListingFailed(int error);
 	void LinkIsNotDir(const CServerPath& path, const wxString& subdir);
 
-	bool SetSyncBrowse(bool enable);
+	bool SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root = CServerPath());
 	bool GetSyncBrowse() const { return !m_sync_browse.local_root.empty(); }
 
 protected:
 	void SetServer(const CServer* server);
 
 	CLocalPath m_localDir;
-	const CDirectoryListing *m_pDirectoryListing;
+	CSharedPointer<const CDirectoryListing> m_pDirectoryListing;
 
 	CServer* m_pServer;
 	bool m_successful_connect;
@@ -106,7 +145,15 @@ protected:
 
 	CRecursiveOperation* m_pRecursiveOperation;
 
-	std::list<CStateEventHandler*> m_handlers[STATECHANGE_MAX];
+	CComparisonManager* m_pComparisonManager;
+
+	struct t_handler
+	{
+		CStateEventHandler* pHandler;
+		bool blockable;
+	};
+	std::list<t_handler> m_handlers[STATECHANGE_MAX];
+	bool m_blocked[STATECHANGE_MAX];
 
 	CLocalPath GetSynchronizedDirectory(CServerPath remote_path);
 	CServerPath GetSynchronizedDirectory(CLocalPath local_path);
@@ -116,6 +163,7 @@ protected:
 		CLocalPath local_root;
 		CServerPath remote_root;
 		bool is_changing;
+		bool compare;
 	} m_sync_browse;
 };
 
@@ -127,7 +175,7 @@ public:
 
 	CState* m_pState;
 
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data) = 0;
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
 };
 
 #endif
