@@ -12,7 +12,13 @@
 #include <wx/dynlib.h> // Used by GetDownloadDir
 #endif //__WXMSW__
 
-#define MAXCHECKPROGRESS 10 // Maximum value of progress bar
+// This is ugly but does the job
+#define SHA512_STANDALONE
+typedef unsigned int uint32;
+#include "../putty/int64.h"
+#include "../putty/sshsh512.c"
+
+#define MAXCHECKPROGRESS 11 // Maximum value of progress bar
 
 BEGIN_EVENT_TABLE(CUpdateWizard, wxWizard)
 EVT_CHECKBOX(wxID_ANY, CUpdateWizard::OnCheck)
@@ -21,14 +27,80 @@ EVT_WIZARD_PAGE_CHANGED(wxID_ANY, CUpdateWizard::OnPageChanged)
 EVT_FZ_NOTIFICATION(wxID_ANY, CUpdateWizard::OnEngineEvent)
 EVT_TIMER(wxID_ANY, CUpdateWizard::OnTimer)
 EVT_WIZARD_FINISHED(wxID_ANY, CUpdateWizard::OnFinish)
+EVT_WIZARD_CANCEL(wxID_ANY, CUpdateWizard::OnCancel)
 END_EVENT_TABLE()
 
+static wxChar s_update_cert[] = _T("-----BEGIN CERTIFICATE-----\n\
+MIIFsTCCA5ugAwIBAgIESnXLbzALBgkqhkiG9w0BAQ0wSTELMAkGA1UEBhMCREUx\n\
+GjAYBgNVBAoTEUZpbGVaaWxsYSBQcm9qZWN0MR4wHAYDVQQDExVmaWxlemlsbGEt\n\
+cHJvamVjdC5vcmcwHhcNMDkwODAyMTcyMjU2WhcNMzEwNjI4MTcyMjU4WjBJMQsw\n\
+CQYDVQQGEwJERTEaMBgGA1UEChMRRmlsZVppbGxhIFByb2plY3QxHjAcBgNVBAMT\n\
+FWZpbGV6aWxsYS1wcm9qZWN0Lm9yZzCCAh8wCwYJKoZIhvcNAQEBA4ICDgAwggIJ\n\
+AoICAJqWXy7YzVP5pOk8VB9bd/ROC9SVbAxJiFHh0I0/JmyW+jSfzFCYWr1DKGVv\n\
+Oui+qiUsaSgjWTh/UusnVu4Q4Lb00k7INRF6MFcGFkGNmOZPk4Qt0uuWMtsxiFek\n\
+9QMPWSYs+bxk+M0u0rNOdAblsIzeV16yhfUQDtrJxPWbRpuLgp9/4/oNbixet7YM\n\
+pvwlns2o1KXcsNcBcXraux5QmnD4oJVYbTY2qxdMVyreA7dxd40c55F6FvA+L36L\n\
+Nv54VwRFSqY12KBG4I9Up+c9OQ9HMN0zm0FhYtYeKWzdMIRk06EKAxO7MUIcip3q\n\
+7v9eROPnKM8Zh4dzkWnCleirW8EKFEm+4+A8pDqirMooiQqkkMesaJDV361UCoVo\n\
+fRhqfK+Prx0BaJK/5ZHN4tmgU5Tmq+z2m7aIKwOImj6VF3somVvmh0G/othnU2MH\n\
+GB7qFrIUMZc5VhrAwmmSA2Z/w4+0ToiR+IrdGmDKz3cVany3EZAzWRJUARaId9FH\n\
+v/ymA1xcFAKmfxsjGNlNpXd7b8UElS8+ccKL9m207k++IIjc0jUPgrM70rU3cv5M\n\
+Kevp971eHLhpWa9vrjbz/urDzBg3Dm8XEN09qwmABfIEnhm6f7oz2bYXjz73ImYj\n\
+rZsogz+Jsx3NWhHFUD42iA4ZnxHIEgchD/TAihpbdrEhgmdvAgMBAAGjgacwgaQw\n\
+EgYDVR0TAQH/BAgwBgEB/wIBAjAmBgNVHREEHzAdgRthZG1pbkBmaWxlemlsbGEt\n\
+cHJvamVjdC5vcmcwDwYDVR0PAQH/BAUDAwcGADAdBgNVHQ4EFgQUd4w2verFjXAn\n\
+CrNLor39nFtemNswNgYDVR0fBC8wLTAroCmgJ4YlaHR0cHM6Ly9jcmwuZmlsZXpp\n\
+bGxhLXByb2plY3Qub3JnL2NybDALBgkqhkiG9w0BAQ0DggIBAF3fmV/Bs4amV78d\n\
+uhe5PkW7yTO6iCfKJVDB22kXPvL0rzZn4SkIZNoac8Xl5vOoRd6k+06i3aJ78w+W\n\
+9Z0HK1jUdjW7taYo4bU58nAp3Li+JwjE/lUBNqSKSescPjdZW0KzIIZls91W30yt\n\
+tGq85oWAuyVprHPlr2uWLg1q4eUdF6ZAz4cZ0+9divoMuk1HiWxi1Y/1fqPRzUFf\n\
+UGK0K36iPPz2ktzT7qJYXRfC5QDoX7tCuoDcO5nccVjDypRKxy45O5Ucm/fywiQW\n\
+NQfz/yQAmarQSCfDjNcHD1rdJ0lx9VWP6xi+Z8PGSlR9eDuMaqPVAE1DLHwMMTTZ\n\
+93PbfP2nvgbElgEki28LUalyVuzvrKcu/rL1LnCJA4jStgE/xjDofpYwgtG4ZSnE\n\
+KgNy48eStvNZbGhwn2YvrxyKmw58WSQG9ArOCHoLcWnpedSZuTrPTLfgNUx7DNbo\n\
+qJU36tgxiO0XLRRSetl7jkSIO6U1okVH0/tvstrXEWp4XwdlmoZf92VVBrkg3San\n\
+fA5hBaI2gpQwtpyOJzwLzsd43n4b1YcPiyzhifJGcqRCBZA1uArNsH5iG6z/qHXp\n\
+KjuMxZu8aM8W2gp8Yg8QZfh5St/nut6hnXb5A8Qr+Ixp97t34t264TBRQD6MuZc3\n\
+PqQuF7sJR6POArUVYkRD/2LIWsB7\n\
+-----END CERTIFICATE-----\n\
+");
+
+
+class CUpdateWizardOptions : public COptionsBase
+{
+public:
+	virtual int GetOptionVal(unsigned int nID)
+	{
+		return COptions::Get()->GetOptionVal(nID);
+	}
+
+	virtual wxString GetOption(unsigned int nID)
+	{
+		if (nID == OPTION_INTERNAL_ROOTCERT)
+			return s_update_cert;
+
+		return COptions::Get()->GetOption(nID);
+	}
+
+	virtual bool SetOption(unsigned int nID, int value)
+	{
+		return COptions::Get()->SetOption(nID, value);
+	}
+
+	virtual bool SetOption(unsigned int nID, wxString value)
+	{
+		return COptions::Get()->SetOption(nID, value);
+	}
+};
+
 CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 	: m_parent(pParent)
 {
 	m_pEngine = new CFileZillaEngine;
 
-	m_pEngine->Init(this, COptions::Get());
+	m_update_options = new CUpdateWizardOptions();
+
+	m_pEngine->Init(this, m_update_options);
 
 	m_inTransfer = false;
 	m_skipPageChanging = false;
@@ -40,6 +112,7 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 
 	wxString version(PACKAGE_VERSION, wxConvLocal);
 	version.Replace(_T(" "), _T("%20"));
+	m_urlProtocol = HTTPS;
 	m_urlServer = _T("update.filezilla-project.org");
 	m_urlFile = wxString::Format(_T("/updatecheck.php?platform=%s&version=%s"), host.c_str(), version.c_str());
 #if defined(__WXMSW__) || defined(__WXMAC__)
@@ -48,8 +121,8 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 	m_urlFile += osVersion;
 #endif
 
-	m_statusTimer.SetOwner(this, 0);
-	m_autoCheckTimer.SetOwner(this, 1);
+	m_statusTimer.SetOwner(this);
+	m_autoCheckTimer.SetOwner(this);
 
 	m_autoUpdateCheckRunning = false;
 
@@ -61,6 +134,7 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 CUpdateWizard::~CUpdateWizard()
 {
 	delete m_pEngine;
+	delete m_update_options;
 }
 
 bool CUpdateWizard::Load()
@@ -247,7 +321,29 @@ void CUpdateWizard::OnPageChanging(wxWizardEvent& event)
 			m_skipPageChanging = false;
 			return;
 		}
-		m_localFile = dialog.GetPath();
+
+		{
+			wxLogNull log;
+			wxRemoveFile(dialog.GetPath());
+		}
+		
+		if (wxFileName::FileExists(dialog.GetPath()))
+		{
+			wxMessageBox(_("Error, local file exists but cannot be removed"));
+			event.Veto();
+			m_skipPageChanging = false;
+			return;
+		}
+
+		const wxString file = dialog.GetPath() + _T(".tmp");
+		m_localFile = file;
+
+		int i = 1;
+		while (wxFileName::FileExists(m_localFile))
+		{
+			i++;
+			m_localFile = file + wxString::Format(_T("%d"), i);
+		}
 	}
 	
 	m_skipPageChanging = false;
@@ -273,11 +369,14 @@ void CUpdateWizard::OnPageChanged(wxWizardEvent& event)
 
 	m_currentPage = 2;
 
-	XRCCTRL(*this, "ID_DOWNLOADTEXT", wxStaticText)->SetLabel(wxString::Format(_("Downloading %s"), wxString(_T("http://") + m_urlServer + m_urlFile).c_str()));
-	
+	wxStaticText *pText = XRCCTRL(*this, "ID_DOWNLOADTEXT", wxStaticText);
+	wxString text = wxString::Format(_("Downloading %s"), (CServer::GetPrefixFromProtocol(m_urlProtocol) + _T("://") + m_urlServer + m_urlFile).c_str());
+	text.Replace(_T("&"), _T("&&"));
+	pText->SetLabel(text);
+
 	m_inTransfer = false;
 
-	int res = m_pEngine->Command(CConnectCommand(CServer(HTTP, DEFAULT, m_urlServer, 80)));
+	int res = m_pEngine->Command(CConnectCommand(CServer(m_urlProtocol, DEFAULT, m_urlServer, (m_urlProtocol == HTTPS) ? 443 : 80)));
 	if (res == FZ_REPLY_OK)
 	{
 		XRCCTRL(*this, "ID_DOWNLOADPROGRESSTEXT", wxStaticText)->SetLabel(_("Connecting to server"));
@@ -302,6 +401,12 @@ void CUpdateWizard::FailedTransfer()
 	if (!m_loaded)
 		return;
 
+	if (m_localFile != _T(""))
+	{
+		wxLogNull log;
+		wxRemoveFile(m_localFile);
+	}
+
 	if (!m_currentPage)
 		XRCCTRL(*this, "ID_FAILURE", wxStaticText)->SetLabel(_("Failed to check for newer version of FileZilla."));
 	else
@@ -319,6 +424,8 @@ void CUpdateWizard::FailedTransfer()
 	pNext->Enable();
 	wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
 	pPrev->Disable();
+
+	XRCCTRL(*this, "ID_LOG", wxTextCtrl)->ChangeValue(m_update_log);
 }
 
 void CUpdateWizard::OnEngineEvent(wxEvent& event)
@@ -353,7 +460,11 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 					if (!m_currentPage)
 					{
 						wxStaticText *pText = XRCCTRL(*this, "ID_CHECKINGTEXTPROGRESS", wxStaticText);
-						pText->SetLabel(pLogMsg->msg);
+						wxString text = pLogMsg->msg;
+						text.Replace(_T("&"), _T("&&"));
+						WrapText(pText, text, m_pages[0]->GetClientSize().x);
+						pText->SetLabel(text);
+
 						m_pages[0]->GetSizer()->Layout();
 						wxGauge* pProgress = XRCCTRL(*this, "ID_CHECKINGPROGRESS", wxGauge);
 						int value = pProgress->GetValue();
@@ -365,10 +476,36 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 					}
 					else if (m_currentPage == 2)
 					{
-						XRCCTRL(*this, "ID_DOWNLOADPROGRESSTEXT", wxStaticText)->SetLabel(pLogMsg->msg);
+						wxStaticText *pText = XRCCTRL(*this, "ID_DOWNLOADPROGRESSTEXT", wxStaticText);
+						
+						wxString text = pLogMsg->msg;
+						text.Replace(_T("&"), _T("&&"));
+						WrapText(pText, text, m_pages[2]->GetClientSize().x);
+						pText->SetLabel(text);
 						m_pages[2]->GetSizer()->Layout();
 					}
 				}
+
+				wxString label;
+				switch (pLogMsg->msgType)
+				{
+				case Error:
+					label = _("Error:");
+					break;
+				case Status:
+					label = _("Status:");
+					break;
+				case Command:
+					label = _("Command:");
+					break;
+				case Response:
+					label = _("Response:");
+					break;
+				default:
+					break;
+				}
+				if (label != _T(""))
+					m_update_log += label + _T(" ") + pLogMsg->msg + _T("\n");
 			}
 			break;
 		case nId_operation:
@@ -409,6 +546,14 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 				}
 				else if (m_currentPage == 2)
 				{
+					if (!VerifyChecksum())
+						break;
+
+					int pos = m_localFile.Find('.', true);
+					wxASSERT(pos > 0);
+					wxRenameFile(m_localFile, m_localFile.Left(pos));
+					m_localFile = m_localFile.Left(pos);
+
 					wxStaticText* pText = XRCCTRL(*this, "ID_DOWNLOADCOMPLETE", wxStaticText);
 					wxASSERT(pText);
 
@@ -470,6 +615,11 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 				{
 					reinterpret_cast<CFileExistsNotification *>(pData)->overwriteAction = CFileExistsNotification::overwrite;
 				}
+				else if (pData->GetRequestID() == reqId_certificate)
+				{
+					CCertificateNotification* pCertNotification = (CCertificateNotification*)pData;
+					pCertNotification->m_trusted = true;
+				}
 				m_pEngine->SetAsyncRequestReply(pData);
 			}
 			break;
@@ -499,6 +649,7 @@ void CUpdateWizard::ParseData()
 	wxString newVersion;
 	wxULongLong newVersionNumber = 0;
 	wxString newUrl;
+	wxString newChecksum;
 
 	while (m_data != _T(""))
 	{
@@ -544,13 +695,19 @@ void CUpdateWizard::ParseData()
 		line = line.Mid(pos + 1);
 
 		// Extract URL
-		pos = line.Find(' ');
-		if (pos != -1)
-			continue;
 		wxString url = line;
 		if (url == _T("none"))
 			url = _T("");
 
+		pos = url.Find(' ');
+		if (pos < 1)
+			newChecksum.clear();
+		else
+		{
+			newChecksum = url.Mid(pos + 1);
+			url = url.Left(pos);
+		}
+
 		if (type == _T("nightly"))
 		{
 			if (!m_loaded)
@@ -614,7 +771,7 @@ void CUpdateWizard::ParseData()
 	}
 	else
 	{
-		PrepareUpdateAvailablePage(newVersion, newUrl);
+		PrepareUpdateAvailablePage(newVersion, newUrl, newChecksum);
 
 		m_skipPageChanging = true;
 		ShowPage(m_pages[1]);
@@ -630,7 +787,7 @@ void CUpdateWizard::ParseData()
 
 void CUpdateWizard::OnTimer(wxTimerEvent& event)
 {
-	if (!event.GetId())
+	if (event.GetId() == m_statusTimer.GetId())
 	{
 		bool changed;
 		CTransferStatus status;
@@ -782,7 +939,7 @@ void CUpdateWizard::StartUpdateCheck()
 	if (COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0)
 		m_urlFile += _T("&beta=1");
 
-	int res = m_pEngine->Command(CConnectCommand(CServer(HTTP, DEFAULT, m_urlServer, 80)));
+	int res = m_pEngine->Command(CConnectCommand(CServer(m_urlProtocol, DEFAULT, m_urlServer, (m_urlProtocol == HTTPS) ? 443 : 80)));
 	if (res == FZ_REPLY_OK)
 	{
 		if (m_loaded)
@@ -842,7 +999,7 @@ void CUpdateWizard::DisplayUpdateAvailability(bool showDialog, bool forceMenu /*
 	}
 }
 
-void CUpdateWizard::PrepareUpdateAvailablePage(const wxString &newVersion, wxString newUrl)
+void CUpdateWizard::PrepareUpdateAvailablePage(const wxString &newVersion, wxString newUrl, const wxString& newChecksum)
 {
 	XRCCTRL(*this, "ID_VERSION", wxStaticText)->SetLabel(newVersion);
 
@@ -875,8 +1032,10 @@ void CUpdateWizard::PrepareUpdateAvailablePage(const wxString &newVersion, wxStr
 			XRCCTRL(*this, "ID_UPDATEDESC2", wxStaticText)->SetLabel(_("Alternatively, visit http://filezilla-project.org to download the most recent version."));
 			XRCCTRL(*this, "ID_UPDATEDESC2", wxStaticText)->Show();
 
+			m_urlProtocol = HTTP;
 			m_urlServer = newUrl.Left(pos);
 			m_urlFile = newUrl.Mid(pos);
+			m_update_checksum = newChecksum;
 		}
 		if (m_news == _T(""))
 		{
@@ -935,4 +1094,117 @@ void CUpdateWizard::PrepareUpdateCheckPage()
 	pText->SetLabel(_("Resolving hostname"));
 }
 
+bool CUpdateWizard::VerifyChecksum()
+{
+	if (m_localFile == _T("") || m_update_checksum == _T(""))
+		return true;
+
+	if (m_update_checksum.Left(7).CmpNoCase(_T("sha512 ")))
+	{
+		// Unknown hash algorithm? Fail
+		FailedChecksum();
+		return false;
+	}
+	m_update_checksum = m_update_checksum.Mid(7);
+	SHA512_State state;
+	SHA512_Init(&state);
+
+	wxFile f;
+	{
+		wxLogNull null;
+		if (!f.Open(m_localFile))
+		{
+			FailedChecksum();
+			return false;
+		}
+	}
+	char buffer[65536];
+	size_t read;
+	while ((read = f.Read(buffer, sizeof(buffer))) > 0)
+	{
+		SHA512_Bytes(&state, buffer, read);
+	}
+	if (read < 0)
+	{
+		FailedChecksum();
+		return false;
+	}
+	f.Close();
+
+	unsigned char raw_digest[64];
+	SHA512_Final(&state, raw_digest);
+
+	wxString digest;
+
+	for (unsigned int i = 0; i < sizeof(raw_digest); i++)
+	{
+		unsigned char l = raw_digest[i] >> 4;
+		unsigned char r = raw_digest[i] & 0x0F;
+
+		if (l > 9)
+			digest += 'a' + l - 10;
+		else
+			digest += '0' + l;
+
+		if (r > 9)
+			digest += 'a' + r - 10;
+		else
+			digest += '0' + r;
+	}
+
+	if (m_update_checksum.CmpNoCase(digest))
+	{
+		FailedChecksum();
+		return false;
+	}
+
+	return true;
+}
+
+void CUpdateWizard::FailedChecksum()
+{
+	m_inTransfer = false;
+
+	if (m_localFile == _T(""))
+		return;
+	else
+	{
+		wxLogNull log;
+		wxRemoveFile(m_localFile);
+	}
+
+	wxString label = _("Checksum mismatch of downloaded file.");
+	XRCCTRL(*this, "ID_FAILURE", wxStaticText)->SetLabel(label);
+
+	XRCCTRL(*this, "ID_MISMATCH1", wxStaticText)->Show();
+	XRCCTRL(*this, "ID_MISMATCH2", wxStaticText)->Show();
+	XRCCTRL(*this, "ID_MISMATCH3", wxStaticText)->Show();
+	XRCCTRL(*this, "ID_MISMATCH4", wxStaticText)->Show();
+	
+	((wxWizardPageSimple*)GetCurrentPage())->SetNext(m_pages[3]);
+	m_pages[3]->SetPrev((wxWizardPageSimple*)GetCurrentPage());	
+
+	m_skipPageChanging = true;
+	ShowPage(m_pages[3]);
+	m_currentPage = 3;
+	m_skipPageChanging = false;
+
+	wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
+	pNext->Enable();
+	wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
+	pPrev->Disable();
+
+	XRCCTRL(*this, "ID_LOG", wxTextCtrl)->ChangeValue(m_update_log);
+
+	RewrapPage(3);
+}
+
+void CUpdateWizard::OnCancel(wxWizardEvent& event)
+{
+	delete m_pEngine;
+	m_pEngine = 0;
+	if (m_localFile != _T(""))
+		wxRemoveFile(m_localFile);
+}
+
 #endif //FZ_MANUALUPDATECHECK
