@@ -5,6 +5,9 @@
 #include "ipcmutex.h"
 #include "xmlfunctions.h"
 #include "buildinfo.h"
+#include "Options.h"
+
+bool CWrapEngine::m_use_cache = true;
 
 #define WRAPDEBUG 0
 #if wxUSE_UNICODE
@@ -223,100 +226,48 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 	int wrapAfter = -1;
 	int start = 0;
 	unsigned int lineLength = 0;
+
+	bool url = false;
 	for (int i = 0; i <= strLen; i++)
 	{
+		if ((text[i] == ':' && text[i + 1] == '/' && text[i + 2] == '/') || // absolute
+			(text[i] == '/' && (!i || text[i - 1] == ' '))) // relative
+			url = true;
 		if (text[i] != ' ' && text[i] != 0)
+		{
+			// If url, wrap on slashes and ampersands, but not first slash of something://
+			if (!url ||
+				 ((text[i] != '/' || text[i + 1] == '/') && (text[i] != '&' || text[i + 1] == '&') && text[i] != '?'))
 			continue;
+		}
 
 		wxString segment;
 		if (wrapAfter == -1)
 		{
-			segment = text.Mid(start, i - start);
+			if (text[i] == '/' || text[i] == '?' || text[i] == '&')
+				segment = text.Mid(start, i - start + 1);
+			else
+				segment = text.Mid(start, i - start);
 			wrapAfter = i;
 		}
 		else
-			segment = text.Mid(wrapAfter + 1, i - wrapAfter - 1);
+		{
+			if (text[i] == '/' || text[i] == '?' || text[i] == '&')
+				segment = text.Mid(wrapAfter + 1, i - wrapAfter);
+			else
+				segment = text.Mid(wrapAfter + 1, i - wrapAfter - 1);
+		}
 
 		segment = wxStripMenuCodes(segment);
 		parent->GetTextExtent(segment, &width, &height, 0, 0, &m_font);
 
-		if ((unsigned int)width > maxLength)
-		{
-			// Something quite long. Perhaps it's a URL we can wrap at slashes?
-
-			while (segment != _T(""))
-			{
-				// Find longest possible subsegment
-				// which can be appended to current line, if there is a current line
-				unsigned int j;
-				int best = -1;
-
-				for (j = 1; j < segment.Len() - 1; j++)
-				{
-					if (segment[j] == '/' && segment[j + 1] != '/')
-					{
-						wxString left = segment.Left(j + 1);
-						int lwidth;
-						parent->GetTextExtent(left, &lwidth, &height, 0, 0, &m_font);
-						if ((unsigned)lwidth <= maxLength)
-						{
-							if (lineLength && lineLength + spaceWidth + lwidth > maxLength)
-							{
-								if (best == -1)
-								{
-									best = j;
-									width = lwidth;
-								}
-
-								break;
-							}
-
-							best = j;
-							width = lwidth;
-						}
-						else
-							break;
-					}
-				}
-				if (best == -1)
-				{
-					// No suitable slash found
-					return false;
-				}
-
-				if (wrappedText != _T(""))
-					wrappedText += _T("\n");
-
-				if (lineLength)
-					wrappedText += text.Mid(start, wrapAfter - start);
-
-				if (lineLength && lineLength + spaceWidth + width > maxLength)
-					wrappedText += _T("\n");
-				else
-					wrappedText += _T(" ");
-				lineLength = 0;
-
-				wrappedText += segment.Left(best + 1);
-				segment = segment.Mid(best + 1);
-
-				parent->GetTextExtent(wrappedText, &width, &height, 0, 0, &m_font);
-				parent->GetTextExtent(segment, &width, &height);
-				if ((unsigned)width <= maxLength)
-				{
-					break;
-				}
-			}
-
-			start = i - segment.Len();
-			wrapAfter = i;
-			lineLength = 0;
-		}
-
 		if (lineLength + spaceWidth + width > maxLength)
 		{
 			if (wrappedText != _T(""))
 				wrappedText += _T("\n");
 			wrappedText += text.Mid(start, wrapAfter - start);
+			if (text[wrapAfter] != ' ' && text[wrapAfter] != '\0')
+				wrappedText += text[wrapAfter];
 			if (width + spaceWidth >= (int)maxLength)
 			{
 				if (wrappedText != _T(""))
@@ -339,7 +290,7 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 			if (wrappedText != _T(""))
 				wrappedText += _T("\n");
 			wrappedText += text.Mid(start, i - start);
-			if (text[i] != ' ')
+			if (text[i] != ' ' && text[i] != '\0')
 				wrappedText += text[i];
 			start = i + 1;
 			wrapAfter = -1;
@@ -352,6 +303,9 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 			lineLength += width;
 			wrapAfter = i;
 		}
+
+		if (text[i] == ' ')
+			url = false;
 	}
 	if (start < strLen)
 	{
@@ -446,8 +400,8 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				{
 #if WRAPDEBUG >= 3
 					plvl printf("Leave: WrapText failed\n");
-#endif				
-					return wrap_failed;
+#endif		
+					return result | wrap_failed;
 				}
 				text->SetLabel(str);
 
@@ -535,20 +489,20 @@ void CWrapEngine::UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::ve
 		iter != wrapped.end();
 		iter++)
 	{
-		if (!(*iter & wrap_didwrap))
+		UnwrapRecursive(windows[i], windows[i]->GetSizer());
+		windows[i]->GetSizer()->Layout();
+
+		if (!(*iter & wrap_didwrap) && !(*iter & wrap_failed))
 		{
 			if (!(*iter) && remove_fitting)
 			{
 				// Page didn't need to be wrapped with current wrap offset,
-				// remove it since desired with will only be larger in further wrappings.
+				// remove it since desired width will only be larger in further wrappings.
 				windows.erase(windows.begin() + i);
+				continue;
 			}
-			continue;
 		}
-
-		UnwrapRecursive(windows[i], windows[i]->GetSizer());
-		windows[i]->GetSizer()->Layout();
-
+		
 		i++;
 	}
 }
@@ -567,6 +521,8 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			pSizer->Layout();
 			
 #ifdef __WXMAC__
+			const int offset = 6;
+#elif defined(__WXGTK__)
 			const int offset = 4;
 #else
 			const int offset = 0;
@@ -585,6 +541,8 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		return true;
 	}
 
+	std::vector<wxWindow*> all_windows = windows;
+
 	wxSize size = minRequestedSize;
 
 	for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); iter++)
@@ -619,6 +577,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 #if WRAPDEBUG > 0	
 	printf("Target ratio: %f\n", (float)ratio);
 	printf("Canvas: % 4d % 4d\n", canvas.x, canvas.y);
+	printf("Initial min and max: %d %d\n", min, max);
 #endif
 
 	while (true)
@@ -630,6 +589,8 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		{
 			wxSizer* pSizer = (*iter)->GetSizer();
 #ifdef __WXMAC__
+			const int offset = 6;
+#elif defined(__WXGTK__)
 			const int offset = 4;
 #else
 			const int offset = 0;
@@ -644,7 +605,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		}
 
 #if WRAPDEBUG > 0
-	    printf("Current: % 4d % 4d   desiredWidth: %d\n", size.GetWidth(), size.GetHeight(), desiredWidth);
+	    printf("Current: % 4d % 4d   desiredWidth: %d, min: %d, max: %d\n", size.GetWidth(), size.GetHeight(), desiredWidth, min, max);
 #endif
 		if (size.GetWidth() > desiredWidth)
 		{
@@ -658,6 +619,9 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 			desiredWidth = (min + max) / 2;
 
+#if WRAPDEBUG > 0
+			printf("Wrapping failed, new min: %d\n", min);
+#endif
 			continue;
 		}
 		actualWidth = size.GetWidth();
@@ -710,7 +674,10 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 		currentRatio = newRatio;
 	}
-	for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); iter++)
+#if WRAPDEBUG > 0		
+		printf("Performing final wrap with bestwidth %d\n", bestWidth);
+#endif		
+	for (std::vector<wxWindow*>::iterator iter = all_windows.begin(); iter != all_windows.end(); iter++)
 	{
 		wxSizer *pSizer = (*iter)->GetSizer();
 
@@ -820,6 +787,9 @@ bool CWrapEngine::UnwrapRecursive(wxWindow* wnd, wxSizer* sizer)
 
 int CWrapEngine::GetWidthFromCache(const char* name)
 {
+	if (!m_use_cache)
+		return 0;
+
 	if (!name || !*name)
 		return 0;
 
@@ -867,6 +837,9 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 
 void CWrapEngine::SetWidthToCache(const char* name, int width)
 {
+	if (!m_use_cache)
+		return;
+
 	if (!name || !*name)
 		return;
 
@@ -963,6 +936,7 @@ bool CWrapEngine::LoadCache()
 
 	if (!pDocument)
 	{
+		m_use_cache = false;
 		wxMessageBox(xml.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
@@ -1110,9 +1084,17 @@ bool CWrapEngine::LoadCache()
 			languageElement->RemoveChild(dialog);
 	}
 
+	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+	{
+		m_use_cache = cacheValid;
+		return true;
+	}
+
 	wxString error;
 	if (!xml.Save(&error))
 	{
+		m_use_cache = false;
+
 		wxString msg = wxString::Format(_("Could not write \"%s\": %s"), file.GetFullPath().c_str(), error.c_str());
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
